@@ -32,12 +32,10 @@ static DirBlockInfo *main_dir;
 static DirBlockInfo *
 new_dir_block(void)
 {
-  DirBlockInfo *dir = malloc(sizeof (DirBlockInfo));
-  assert(dir);
   /* initialize eveything to zero */
-  memset(dir, 0, sizeof (DirBlockInfo));
-  gp_putl16(&dir->dir[COD_DIR_CODTYPE], 1);
+  DirBlockInfo *dir = GP_Calloc(1, sizeof(DirBlockInfo));
 
+  gp_putl16(&dir->dir[COD_DIR_CODTYPE], 1);
   return dir;
 }
 
@@ -47,13 +45,11 @@ init_dir_block(void)
   DirBlockInfo *dir = new_dir_block();
 
   /* Initialize the directory block with known data. It'll be written
-   * to the .cod file after everything else */
+   * to the .cod file after everything else. */
   gp_cod_strncpy(&dir->dir[COD_DIR_SOURCE],
                  state.codfilename, COD_DIR_DATE - COD_DIR_SOURCE);
-  gp_cod_date(&dir->dir[COD_DIR_DATE],
-              COD_DIR_TIME - COD_DIR_DATE);
-  gp_cod_time(&dir->dir[COD_DIR_TIME],
-              COD_DIR_VERSION - COD_DIR_TIME);
+  gp_cod_date(&dir->dir[COD_DIR_DATE], COD_DIR_TIME - COD_DIR_DATE);
+  gp_cod_time(&dir->dir[COD_DIR_TIME], COD_DIR_VERSION - COD_DIR_TIME);
   gp_cod_strncpy(&dir->dir[COD_DIR_VERSION],
                  VERSION, COD_DIR_COMPILER - COD_DIR_VERSION);
   gp_cod_strncpy(&dir->dir[COD_DIR_COMPILER],
@@ -62,7 +58,7 @@ init_dir_block(void)
                  GPUTILS_COPYRIGHT_STRING,
                  COD_DIR_SYMTAB - COD_DIR_NOTICE);
 
-  /* The address is always two shorts or 4 bytes long */
+  /* The address is always two shorts or 4 bytes long. */
   dir->dir[COD_DIR_ADDRSIZE] = 0;
 
   return dir;
@@ -71,16 +67,15 @@ init_dir_block(void)
 /*
  * init_cod - initialize the cod file
  */
-
 void
 cod_init(void)
 {
-  if (state.codfile != named) {
-    snprintf(state.codfilename, sizeof (state.codfilename),
+  if (state.codfile != OUT_NAMED) {
+    snprintf(state.codfilename, sizeof(state.codfilename),
              "%s.cod", state.basefilename);
   }
 
-  if (state.codfile == suppress) {
+  if (state.codfile == OUT_SUPPRESS) {
     state.cod.f = NULL;
     state.cod.enabled = false;
     unlink(state.codfilename);
@@ -93,8 +88,9 @@ cod_init(void)
     state.cod.enabled = true;
   }
 
-  if (!state.cod.enabled)
+  if (!state.cod.enabled) {
     return;
+  }
 
   main_dir = init_dir_block();
 }
@@ -108,28 +104,30 @@ write_file_block(void)
 {
   BlockList *fb = NULL;
   int id_number = 0;
-  struct file_context *fc;
+  const struct file_context *fc;
 
-  if (!state.files)
+  if (state.files == NULL) {
     return;
+  }
 
   /* Find the head of the file list: */
 
   fc = state.files;
-  while (fc->prev && id_number++ < 1000) {
+  while ((fc->prev != NULL) && (id_number++ < 1000)) {
     fc = fc->prev;
   }
 
   if (id_number >= 1000) {
-    /* Too many files to handle in the .cod file */
+    /* Too many files to handle in the .cod file. */
     assert(0);
   }
 
   id_number = 0;
 
   while (fc != NULL) {
-    if (NULL == fb || main_dir->src.offset >= (FILES_PER_BLOCK * FILE_SIZE))
+    if ((fb == NULL) || (main_dir->src.offset >= (FILES_PER_BLOCK * FILE_SIZE))) {
       fb = gp_blocks_append(&main_dir->src, gp_blocks_new());
+    }
 
     /* The file id is used to define the index at which the file
      * name is written within the file code block. (The id's are
@@ -138,10 +136,7 @@ write_file_block(void)
      * can handle larger file lists...
      */
 
-    gp_cod_strncpy(&fb->block[main_dir->src.offset + 1],
-                   fc->name,
-                   FILE_SIZE - 1);
-
+    gp_cod_strncpy(&fb->block[main_dir->src.offset + 1], fc->name, FILE_SIZE - 1);
     main_dir->src.offset += FILE_SIZE;
 
     fc = fc->next;
@@ -160,13 +155,13 @@ find_dir_block_by_high_addr(int high_addr)
        _64k segment. So we need to create a new segment. */
     if (dbi->next == NULL) {
       dbi->next = new_dir_block();
-      gp_putl16(&dbi->next->dir[COD_DIR_HIGHADDR],
-                high_addr);
+      gp_putl16(&dbi->next->dir[COD_DIR_HIGHADDR], high_addr);
       dbi = dbi->next;
       break;
     }
-    else
+    else {
       dbi = dbi->next;
+    }
   }
   return dbi;
 }
@@ -179,48 +174,54 @@ void
 cod_lst_line(int line_type)
 {
 #define COD_LST_FIRST_LINE  7
+  static DirBlockInfo *dbi = NULL;
+  static int _64k_base = 0;
+
   unsigned char smod_flag;
   BlockList *lb;
   int first_time;
   int address, high_address;
-  static DirBlockInfo *dbi = NULL;
-  static int _64k_base = 0;
 
-  if (!state.cod.enabled)
+  if (!state.cod.enabled) {
     return;
+  }
 
-  /* Don't start until after the source is open */
-  if (state.src == NULL)
+  /* Don't start until after the source is open. */
+  if (state.src == NULL) {
     return;
+  }
 
-  /* Ignore the first few line numbers */
-  if (state.lst.line_number < COD_LST_FIRST_LINE)
+  /* Ignore the first few line numbers. */
+  if (state.lst.line_number < COD_LST_FIRST_LINE) {
     return;
+  }
 
-  address = gp_processor_byte_to_org(state.device.class, state.lst.line.was_org);
+  address = gp_processor_byte_to_org(state.device.class, state.lst.line.was_byte_addr);
   high_address = (address >> 16) & 0xffff;
 
-  if (NULL == dbi || high_address != _64k_base) {
+  if ((dbi == NULL) || (high_address != _64k_base)) {
     _64k_base = high_address;
     dbi = find_dir_block_by_high_addr(_64k_base);
   }
 
-  first_time = (NULL == gp_blocks_get_last(&dbi->lst));
+  first_time = (gp_blocks_get_last(&dbi->lst) == NULL) ? true : false;
 
   lb = gp_blocks_get_last_or_new(&dbi->lst);
 
-  if (dbi->lst.offset >= (COD_MAX_LINE_SYM * COD_LINE_SYM_SIZE))
+  if (dbi->lst.offset >= (COD_MAX_LINE_SYM * COD_LINE_SYM_SIZE)) {
     lb = gp_blocks_append(&dbi->lst, gp_blocks_new());
+  }
 
   assert(state.src->fc != NULL);
   lb->block[dbi->lst.offset + COD_LS_SFILE] = state.src->fc->id;
 
-  smod_flag = (first_time ? 0xff : ((state.cod.emitting != 0) ? 0x80 : 0x90));
+  smod_flag = (first_time) ? COD_LS_SMOD_FLAG_ALL :
+                             ((state.cod.emitting != 0) ? COD_LS_SMOD_FLAG_C1 :
+                                                          (COD_LS_SMOD_FLAG_C1 | COD_LS_SMOD_FLAG_D));
 
   lb->block[dbi->lst.offset + COD_LS_SMOD] = smod_flag;
 
-  /* Write the source file line number corresponding to the list file line
-     number */
+  /* Write the source file line number corresponding to the list file line number. */
   gp_putl16(&lb->block[dbi->lst.offset + COD_LS_SLINE], state.src->line_number);
 
   /* Write the address of the opcode. */
@@ -241,36 +242,38 @@ void
 cod_write_symbols(struct symbol **symbol_list, int num_symbols)
 {
   int i, len, type;
-  struct variable *var;
-  const char *s;
+  const struct variable *var;
+  const char *name;
   BlockList *sb = NULL;
 
-  if (!state.cod.enabled)
+  if (!state.cod.enabled) {
     return;
+  }
 
   for (i = 0; i < num_symbols; i++) {
     var = get_symbol_annotation(symbol_list[i]);
-    s = get_symbol_name(symbol_list[i]);
-    len = strlen(s);
+    name = get_symbol_name(symbol_list[i]);
+    len = strlen(name);
 
     /* If this symbol extends past the end of the cod block
      * then write this block out */
 
-    if (NULL == sb || (main_dir->sym.offset + len + COD_SYM_EXTRA) >= COD_BLOCK_SIZE)
+    if ((sb == NULL) || ((main_dir->sym.offset + len + COD_SYM_EXTRA) >= COD_BLOCK_SIZE)) {
       sb = gp_blocks_append(&main_dir->sym, gp_blocks_new());
+    }
 
-    gp_cod_strncpy(&sb->block[main_dir->sym.offset + 1], s, MAX_SYM_LEN);
+    gp_cod_strncpy(&sb->block[main_dir->sym.offset + 1], name, MAX_SYM_LEN);
 
     switch (var->type) {
-    case gvt_cblock:
+    case GVT_CBLOCK:
       type = COD_ST_C_SHORT;    /* byte craft's nomenclature for a memory byte. */
       break;
 
-    case gvt_address:
+    case GVT_ADDRESS:
       type = COD_ST_ADDRESS;
       break;
 
-    case gvt_constant:
+    case GVT_CONSTANT:
     default:
       type = COD_ST_CONSTANT;
     }
@@ -291,8 +294,9 @@ cod_emit_opcode(DirBlockInfo *dbi, int address, int opcode)
 {
   int block_index;
 
-  if (!state.cod.enabled)
+  if (!state.cod.enabled) {
     return;
+  }
 
   /* The code image blocks are handled in a different manner than the
    * other cod blocks. In theory, it's possible to emit opcodes in a
@@ -306,8 +310,9 @@ cod_emit_opcode(DirBlockInfo *dbi, int address, int opcode)
 
   block_index = (address >> COD_BLOCK_BITS) & (COD_CODE_IMAGE_BLOCKS - 1);
 
-  if (dbi->cod_image_blocks[block_index].block == NULL)
+  if (dbi->cod_image_blocks[block_index].block == NULL) {
     gp_cod_create(&dbi->cod_image_blocks[block_index]);
+  }
 
   gp_putl16(&dbi->cod_image_blocks[block_index].block[address & (COD_BLOCK_SIZE - 1)], opcode);
 }
@@ -317,28 +322,32 @@ cod_emit_opcode(DirBlockInfo *dbi, int address, int opcode)
 static void
 cod_write_code(void)
 {
-  MemBlock *m = state.i_memory;
-  int mem_base, i;
-  int start_address = 0, used_flag = 0;
-  BlockList *rb = NULL;
   static DirBlockInfo *dbi = NULL;
   static int _64k_base = 0;
 
-  while (m) {
+  const MemBlock *m = state.i_memory;
+  int mem_base, i;
+  int start_address = 0, used_flag = 0;
+  BlockList *rb = NULL;
+
+  while (m != NULL) {
     int high_addr;
 
     mem_base = m->base << I_MEM_BITS;
     high_addr = (mem_base >> 16) & 0xffff;
-    if (NULL == dbi || high_addr != _64k_base) {
+
+    if ((dbi == NULL) || (high_addr != _64k_base)) {
       _64k_base = high_addr;
       dbi = find_dir_block_by_high_addr(_64k_base);
     }
 
     for (i = mem_base; (i - mem_base) <= MAX_I_MEM; i += 2) {
       unsigned short insn;
-      if (i - mem_base < MAX_I_MEM &&
-          state.device.class->i_memory_get(state.i_memory, i, &insn)) {
+
+      if (((i - mem_base) < MAX_I_MEM) &&
+          state.device.class->i_memory_get(state.i_memory, i, &insn, NULL, NULL)) {
         cod_emit_opcode(dbi, i, insn);
+
         if (used_flag == 0) {
           /* Save the start address in a range of opcodes */
           start_address = i;
@@ -350,7 +359,7 @@ cod_write_code(void)
         if (used_flag == 1) {
           rb = gp_blocks_get_last_or_new(&dbi->rng);
 
-          if (NULL == rb || dbi->rng.offset + COD_MAPENTRY_SIZE >= COD_BLOCK_SIZE) {
+          if ((rb == NULL) || ((dbi->rng.offset + COD_MAPENTRY_SIZE) >= COD_BLOCK_SIZE)) {
             /* If there are a whole bunch of non-contiguous pieces of
                code then we'll get here. But most pic apps will only need
                one directory block (that will give you 64 ranges or non-
@@ -374,179 +383,22 @@ cod_write_code(void)
   }
 }
 
-static void
-enumerate_blocks(DirBlockInfo *dir, int offset, Blocks *bl, unsigned int *block_num)
-{
-  if (bl->blocks) {
-    /* enumerate block list */
-    gp_putl16(&dir->dir[offset], ++*block_num);
-    *block_num += gp_blocks_count(bl) - 1;
-    gp_putl16(&dir->dir[offset + 2], *block_num);
-  }
-}
-
-static void
-enumerate_directory(void)
-{
-  DirBlockInfo *dbi;
-  unsigned int block_num = 0;
-
-  /* enumerate directory blocks */
-  for (dbi = main_dir; dbi; dbi = dbi->next) {
-    gp_putl16(&dbi->dir[COD_DIR_NEXTDIR], dbi->next ? ++block_num : 0);
-  }
-
-  /* enumerate code blocks */
-  for (dbi = main_dir; dbi; dbi = dbi->next) {
-    int i;
-
-    for (i = 0; i < COD_CODE_IMAGE_BLOCKS; ++i) {
-      if (NULL != dbi->cod_image_blocks[i].block)
-        gp_putl16(&dbi->dir[i * 2], ++block_num);
-    }
-  }
-
-  /* enumerate surce files blocks */
-  for (dbi = main_dir; dbi; dbi = dbi->next)
-    enumerate_blocks(dbi, COD_DIR_NAMTAB, &dbi->src, &block_num);
-
-  /* enumerate list lines blocks */
-  for (dbi = main_dir; dbi; dbi = dbi->next)
-    enumerate_blocks(dbi, COD_DIR_LSTTAB, &dbi->lst, &block_num);
-
-  /* enumerate memory map blocks */
-  for (dbi = main_dir; dbi; dbi = dbi->next)
-    enumerate_blocks(dbi, COD_DIR_MEMMAP, &dbi->rng, &block_num);
-
-  /* enumerate long symbol table blocks */
-  for (dbi = main_dir; dbi; dbi = dbi->next)
-    enumerate_blocks(dbi, COD_DIR_LSYMTAB, &dbi->sym, &block_num);
-
-  /* enumerate debug messages table blocks */
-  for (dbi = main_dir; dbi; dbi = dbi->next)
-    enumerate_blocks(dbi, COD_DIR_MESSTAB, &dbi->dbg, &block_num);
-}
-
-static void
-write_blocks(Blocks *bl)
-{
-  BlockList *p = bl->blocks;
-
-  /* write block list */
-  while (p) {
-    fwrite(p->block, 1, COD_BLOCK_SIZE, state.cod.f);
-    p = p->next;
-  }
-}
-
-static void
-write_directory(void)
-{
-  DirBlockInfo *dbi;
-
-  /* write directory blocks */
-  for (dbi = main_dir; dbi; dbi = dbi->next) {
-    fwrite(dbi->dir, 1, COD_BLOCK_SIZE, state.cod.f);
-  }
-
-  /* write code blocks */
-  for (dbi = main_dir; dbi; dbi = dbi->next) {
-    int i;
-
-    for (i = 0; i < COD_CODE_IMAGE_BLOCKS; ++i) {
-      if (NULL != dbi->cod_image_blocks[i].block)
-        fwrite(dbi->cod_image_blocks[i].block, 1, COD_BLOCK_SIZE, state.cod.f);
-    }
-  }
-
-  /* write surce files blocks */
-  for (dbi = main_dir; dbi; dbi = dbi->next)
-    write_blocks(&dbi->src);
-
-  /* write list lines blocks */
-  for (dbi = main_dir; dbi; dbi = dbi->next)
-    write_blocks(&dbi->lst);
-
-  /* write memory map blocks */
-  for (dbi = main_dir; dbi; dbi = dbi->next)
-    write_blocks(&dbi->rng);
-
-  /* write long symbol table blocks */
-  for (dbi = main_dir; dbi; dbi = dbi->next)
-    write_blocks(&dbi->sym);
-
-  /* write debug messages table blocks */
-  for (dbi = main_dir; dbi; dbi = dbi->next)
-    write_blocks(&dbi->dbg);
-}
-
-static void
-free_blocks(Blocks *bl)
-{
-  BlockList *b = bl->blocks;
-
-  while (b) {
-    BlockList *next = b->next;
-    free(b);
-    b = next;
-  }
-}
-
-static void
-free_directory(void)
-{
-  DirBlockInfo *dbi = main_dir;
-
-  while (dbi) {
-    int i;
-    DirBlockInfo *next;
-
-    /* free code blocks */
-    for (i = 0; i < COD_CODE_IMAGE_BLOCKS; ++i) {
-      if (NULL != dbi->cod_image_blocks[i].block)
-        free(dbi->cod_image_blocks[i].block);
-    }
-
-    /* free surce files blocks */
-    free_blocks(&dbi->src);
-
-    /* free list lines blocks */
-    free_blocks(&dbi->lst);
-
-    /* free memory map blocks */
-    free_blocks(&dbi->rng);
-
-    /* free long symbol table blocks */
-    free_blocks(&dbi->sym);
-
-    /* free debug messages table blocks */
-    free_blocks(&dbi->dbg);
-
-    next = dbi->next;
-    /* free directory blocks */
-    free(dbi);
-    dbi = next;
-  }
-
-  main_dir = NULL;
-}
-
 void
 cod_close_file(void)
 {
-  if (!state.cod.enabled)
+  if (!state.cod.enabled) {
     return;
+  }
 
   /* processor is unknown if not defined in command line at cod_init() call
      so it should be set here */
-  gp_cod_strncpy(&main_dir->dir[COD_DIR_PROCESSOR],
-                 gp_processor_name(state.processor, 2),
-                 COD_DIR_LSYMTAB - COD_DIR_PROCESSOR);  write_file_block();
-
+  gp_cod_strncpy(&main_dir->dir[COD_DIR_PROCESSOR], gp_processor_name(state.processor, 2),
+                 COD_DIR_LSYMTAB - COD_DIR_PROCESSOR);
+  write_file_block();
   cod_write_code();
-
-  enumerate_directory();
-  write_directory();
-  free_directory();
+  gp_blocks_enumerate_directory(main_dir);
+  gp_blocks_write_directory(state.cod.f, main_dir);
+  gp_blocks_free_directory(main_dir);
+  main_dir = NULL;
   fclose(state.cod.f);
 }
