@@ -30,12 +30,12 @@ Boston, MA 02111-1307, USA.  */
 #define CORE_14BIT_MASK 0x3fff
 #define CORE_16BIT_MASK 0xffff
 
-gpasmVal do_insn(char *name, struct pnode *parms);
+gpasmVal do_insn(const char *name, struct pnode *parms);
 void opcode_init(int stage);
-void begin_cblock(struct pnode *c);
+void begin_cblock(const struct pnode *c);
 void continue_cblock(void);
-void cblock_expr(struct pnode *s);
-void cblock_expr_incr(struct pnode *s, struct pnode *incr);
+void cblock_expr(const struct pnode *s);
+void cblock_expr_incr(const struct pnode *s, const struct pnode *incr);
 int asm_enabled(void);
 int check_page(int address);
 
