@@ -31,62 +31,63 @@ Boston, MA 02111-1307, USA.  */
 
 int enforce_arity(int arity, int must_be)
 {
-  if (arity == must_be)
+  if (arity == must_be) {
     return 1;
+  }
   else {
     if (arity < must_be) {
-      gpverror(GPE_MISSING_ARGU);
+      gpverror(GPE_MISSING_ARGU, NULL);
     }
     else {
-      gpverror(GPE_TOO_MANY_ARGU);
+      gpverror(GPE_TOO_MANY_ARGU, NULL);
     }
     return 0;
   }
 }
 
-int enforce_simple(struct pnode *p)
+int enforce_simple(const struct pnode *p)
 {
   switch (p->tag) {
-  case symbol:
+  case PTAG_SYMBOL:
     return 1;
     break;
 
-  case string:
-    gpverror(GPE_ILLEGAL_ARGU, p->value.string);
+  case PTAG_STRING:
+    gpverror(GPE_ILLEGAL_ARGU, NULL, p->value.string);
     break;
 
   default:
-    gperror(GPE_ILLEGAL_ARGU, "Illegal argument");
+    gperror(GPE_ILLEGAL_ARGU, "Illegal argument.");
   }
   return 0;
 }
 
-int list_length(struct pnode *L)
+int list_length(const struct pnode *L)
 {
-  struct pnode *p = L;
+  const struct pnode *p = L;
   int n = 0;
 
-  while (p && list == p->tag) {
+  while ((p != NULL) && (p->tag == PTAG_LIST)) {
     ++n;
     p = TAIL(p);
   }
 
-  return (NULL != p) ? n + 1: n;
+  return ((p != NULL) ? (n + 1) : n);
 }
 
-int can_evaluate(struct pnode *p)
+int can_evaluate(const struct pnode *p)
 {
   switch (p->tag) {
-  case constant:
+  case PTAG_CONSTANT:
     return 1;
 
-  case offset:
+  case PTAG_OFFSET:
     if (state.extended_pic16e == false) {
-      gpverror(GPE_BADCHAR, '[');
+      gpverror(GPE_BADCHAR, NULL, '[');
     }
     return can_evaluate(p->value.offset);
 
-  case symbol:
+  case PTAG_SYMBOL:
     {
       char buf[BUFSIZ];
       struct symbol *s;
@@ -102,18 +103,19 @@ int can_evaluate(struct pnode *p)
         s = get_symbol(state.stTop, p->value.symbol);
 
         if (s == NULL) {
-          if ('\0' == *p->value.symbol)
-            gpverror(GPE_MISSING_ARGU);
-          else
-            gpverror(GPE_NOSYM, p->value.symbol);
+          if (*p->value.symbol == '\0') {
+            gpverror(GPE_MISSING_ARGU, NULL);
+          }
+          else {
+            gpverror(GPE_NOSYM, NULL, p->value.symbol);
+          }
         }
         else {
           var = get_symbol_annotation(s);
 
           if (var == NULL) {
-            snprintf(buf,
-                     sizeof(buf),
-                     "Symbol not assigned a value (%s).",
+            snprintf(buf, sizeof(buf),
+                     "Symbol not assigned a value: \"%s\"",
                      p->value.symbol);
             gpwarning(GPW_UNKNOWN, buf);
           }
@@ -123,14 +125,14 @@ int can_evaluate(struct pnode *p)
       }
     }
 
-  case unop:
+  case PTAG_UNOP:
     return can_evaluate(p->value.unop.p0);
 
-  case binop:
+  case PTAG_BINOP:
     return can_evaluate(p->value.binop.p0) && can_evaluate(p->value.binop.p1);
 
-  case string:
-    gpverror(GPE_ILLEGAL_ARGU, p->value.string);
+  case PTAG_STRING:
+    gpverror(GPE_ILLEGAL_ARGU, NULL, p->value.string);
     return 0;
 
   default:
@@ -140,42 +142,45 @@ int can_evaluate(struct pnode *p)
   return 0;
 }
 
-int can_evaluate_value(struct pnode *p)
+int can_evaluate_value(const struct pnode *p)
 {
   switch (p->tag) {
-  case constant:
+  case PTAG_CONSTANT:
     return 1;
 
-  case offset:
+  case PTAG_OFFSET:
     if (state.extended_pic16e == false) {
-      gpverror(GPE_BADCHAR, '[');
+      gpverror(GPE_BADCHAR, NULL, '[');
     }
     return can_evaluate_value(p->value.offset);
 
-  case symbol:
+  case PTAG_SYMBOL:
     /* '$' means current org, which we can evaluate if section at absolute address */
-    if (strcmp(p->value.symbol, "$") == 0)
-      return (0 != (state.obj.new_sec_flags & STYP_ABS));
+    if (strcmp(p->value.symbol, "$") == 0) {
+      return (((state.obj.new_sec_flags & STYP_ABS) != 0) ? true : false);
+    }
     else {
       /* Otherwise look it up */
       struct symbol *s = get_symbol(state.stTop, p->value.symbol);
 
-      if (s == NULL)
+      if (s == NULL) {
         return 0;
+      }
       else {
         struct variable *var = get_symbol_annotation(s);
+
         if (NULL == var) {
           return 0;
         }
         else {
           switch (var->type) {
-          case gvt_extern:
-          case gvt_global:
-          case gvt_static:
-          case gvt_absolute:
-          case gvt_debug:
+          case GVT_EXTERN:
+          case GVT_GLOBAL:
+          case GVT_STATIC:
+          case GVT_ABSOLUTE:
+          case GVT_DEBUG:
             return 0;
-            
+
           default:
             return 1;
           }
@@ -183,14 +188,14 @@ int can_evaluate_value(struct pnode *p)
       }
     }
 
-  case unop:
+  case PTAG_UNOP:
     return can_evaluate_value(p->value.unop.p0);
 
-  case binop:
+  case PTAG_BINOP:
     return can_evaluate_value(p->value.binop.p0) && can_evaluate_value(p->value.binop.p1);
 
-  case string:
-    gpverror(GPE_ILLEGAL_ARGU, p->value.string);
+  case PTAG_STRING:
+    gpverror(GPE_ILLEGAL_ARGU, NULL, p->value.string);
     return 0;
 
   default:
@@ -200,39 +205,39 @@ int can_evaluate_value(struct pnode *p)
   return 0;
 }
 
-static int is_program_segment(struct pnode *p)
+static int is_program_segment(const struct pnode *p)
 {
-  if (symbol == p->tag && 0 != strcmp(p->value.symbol, "$")) {
+  if ((p->tag == PTAG_SYMBOL) && (strcmp(p->value.symbol, "$") != 0)) {
     struct symbol *s = get_symbol(state.stTop, p->value.symbol);
     struct variable *var = get_symbol_annotation(s);
     assert(var != NULL);
-    /* if var type is gvt_cblock return 1, else return 0 */
-    return gvt_cblock != var->type;
+    /* If var type is GVT_ADDRESS return 1, else return 0. */
+    return ((var->type == GVT_ADDRESS) ? true : false);
+  }
+  else {
+    return false;
   }
-  else
-    return 0;
 }
 
-gpasmVal evaluate(struct pnode *p)
+gpasmVal evaluate(const struct pnode *p)
 {
   struct variable *var;
   gpasmVal p0, p1;
 
   switch (p->tag) {
-  case constant:
+  case PTAG_CONSTANT:
     return p->value.constant;
 
-  case offset:
+  case PTAG_OFFSET:
     return evaluate(p->value.offset);
 
-  case symbol:
+  case PTAG_SYMBOL:
     {
       struct symbol *s;
 
       if (strcmp(p->value.symbol, "$") == 0) {
-        if (IS_RAM_ORG)
-          return state.org;
-        return gp_processor_byte_to_org(state.device.class, state.org);
+        return (IS_RAM_ORG ? state.byte_addr :
+                             gp_processor_byte_to_real(state.processor, state.byte_addr));
       }
       else {
         s = get_symbol(state.stTop, p->value.symbol);
@@ -242,7 +247,7 @@ gpasmVal evaluate(struct pnode *p)
       }
     }
 
-  case unop:
+  case PTAG_UNOP:
     switch (p->value.unop.op) {
     case '!':
       return !evaluate(p->value.unop.p0);
@@ -262,11 +267,10 @@ gpasmVal evaluate(struct pnode *p)
     case HIGH:
       {
         gpasmVal val = (evaluate(p->value.unop.p0) >> 8) & 0xff;
-        /* set 7th bit if in absolute mode and PROC_CLASS_PIC14E and not in cblock;
-         * relative mode is handeled by the lonker */
-        if (absolute == state.mode &&
-          PROC_CLASS_PIC14E == state.device.class &&
-          is_program_segment(p->value.unop.p0)) {
+        /* set 7th bit if in absolute mode and PROC_CLASS_PIC14E or PROC_CLASS_PIC14EX and
+         * address relative mode is handled by the linker */
+        if ((state.mode == MODE_ABSOLUTE) && (IS_PIC14E_CORE || IS_PIC14EX_CORE) &&
+            is_program_segment(p->value.unop.p0)) {
           val |= 0x80;
         }
         return val;
@@ -285,7 +289,7 @@ gpasmVal evaluate(struct pnode *p)
       assert(0);
     }
 
-  case binop:
+  case PTAG_BINOP:
     p0 = evaluate(p->value.binop.p0);
     p1 = evaluate(p->value.binop.p1);
     switch (p->value.binop.op) {
@@ -300,7 +304,7 @@ gpasmVal evaluate(struct pnode *p)
 
     case '/':
       if (p1 == 0){
-        gpverror(GPE_DIVBY0);
+        gpverror(GPE_DIVBY0, NULL);
         return 0;
       }
       else {
@@ -309,7 +313,7 @@ gpasmVal evaluate(struct pnode *p)
 
     case '%':
       if (p1 == 0){
-        gpverror(GPE_DIVBY0);
+        gpverror(GPE_DIVBY0, NULL);
         return 0;
       }
       else {
@@ -330,25 +334,25 @@ gpasmVal evaluate(struct pnode *p)
         /* MPASM compatible:
          * It seems that x << n is actually x << (n % (sizeof(int) * 8))
          * on x86 architectures, so 0x1234 << 32 results 0x1234
-         * which is wrong but compatible with MPASM */
+         * which is wrong but compatible with MPASM. */
         return p0 << p1;
       }
       else {
       /* x << n results sign extension for n >= (sizeof(int) * 8) */
-        return (p1 >= sizeof(int) * 8) ? ((p0 < 0) ? -1 : 0) : p0 << p1;
+        return ((p1 >= (sizeof(int) * 8)) ? ((p0 < 0) ? -1 : 0) : (p0 << p1));
       }
 
     case RSH:
       if (state.mpasm_compatible) {
-        /* MPASM compatible: see https://sourceforge.net/p/gputils/bugs/252/ 
+        /* MPASM compatible: see https://sourceforge.net/p/gputils/bugs/252/
          * It seems that x >> n is actually x >> (n % (sizeof(int) * 8))
          * on x86 architectures, so 0x1234 >> 32 results 0x1234
-         * which is wrong but compatible with MPASM */
+         * which is wrong but compatible with MPASM. */
         return p0 >> p1;
       }
       else {
       /* x >> n results sign extension for n >= (sizeof(int) * 8) */
-        return (p1 >= sizeof(int) * 8) ? ((p0 < 0) ? -1 : 0) : p0 >> p1;
+        return ((p1 >= (sizeof(int) * 8)) ? ((p0 < 0) ? -1 : 0) : (p0 >> p1));
       }
 
     case EQUAL:
@@ -376,8 +380,9 @@ gpasmVal evaluate(struct pnode *p)
       return p0 || p1;
 
     case '=':
-      gpverror(GPE_BADCHAR, '=');
+      gpverror(GPE_BADCHAR, NULL, '=');
       return 0;
+
     default:
       assert(0); /* Unhandled binary operator */
     }
@@ -391,11 +396,11 @@ gpasmVal evaluate(struct pnode *p)
 /* Attempt to evaluate expression 'p'.  Return its value if
  * successful, otherwise generate an error message and return 0.  */
 
-gpasmVal maybe_evaluate(struct pnode *p)
+gpasmVal maybe_evaluate(const struct pnode *p)
 {
   gpasmVal r;
 
-  if (p && can_evaluate(p)) {
+  if ((p != NULL) && can_evaluate(p)) {
     r = evaluate(p);
   }
   else {
@@ -407,22 +412,23 @@ gpasmVal maybe_evaluate(struct pnode *p)
 
 /* count the number of relocatable addesses in the expression */
 
-int count_reloc(struct pnode *p)
+int count_reloc(const struct pnode *p)
 {
   struct symbol *s;
   struct variable *var;
 
-  if (state.mode == absolute)
+  if (state.mode == MODE_ABSOLUTE) {
     return 0;
+  }
 
   switch (p->tag) {
-  case constant:
+  case PTAG_CONSTANT:
     return 0;
 
-  case offset:
+  case PTAG_OFFSET:
     return count_reloc(p->value.offset);
 
-  case symbol:
+  case PTAG_SYMBOL:
     if (strcmp(p->value.symbol, "$") == 0) {
       return 1;
     }
@@ -430,12 +436,13 @@ int count_reloc(struct pnode *p)
       s = get_symbol(state.stTop, p->value.symbol);
       if (s != NULL) {
         var = get_symbol_annotation(s);
+
         if (var != NULL) {
-          switch(var->type) {
-          case gvt_extern:
-          case gvt_global:
-          case gvt_static:
-          case gvt_address:
+          switch (var->type) {
+          case GVT_EXTERN:
+          case GVT_GLOBAL:
+          case GVT_STATIC:
+          case GVT_ADDRESS:
             return 1;
 
           default:
@@ -446,10 +453,10 @@ int count_reloc(struct pnode *p)
     }
     return 0;
 
-  case unop:
+  case PTAG_UNOP:
     return count_reloc(p->value.unop.p0);
 
-  case binop:
+  case PTAG_BINOP:
     return count_reloc(p->value.binop.p0) + count_reloc(p->value.binop.p1);
 
   default:
@@ -463,30 +470,31 @@ int count_reloc(struct pnode *p)
    [UPPER|HIGH|LOW]([<relocatable symbol>] + [<offs>]) */
 
 static void
-add_reloc(struct pnode *p, short offs, unsigned short type)
+add_reloc(const struct pnode *p, short offs, unsigned short type)
 {
   struct symbol *s = NULL;
   struct variable *var = NULL;
 
   switch (p->tag) {
-  case offset:
+  case PTAG_OFFSET:
     add_reloc(p->value.offset, offs, type);
     break;
 
-  case symbol:
+  case PTAG_SYMBOL:
     if (strcmp(p->value.symbol, "$") == 0) {
       char buffer[BUFSIZ];
       unsigned org;
-      if (IS_RAM_ORG)
-        org = state.org;
-      else
-        org = gp_processor_byte_to_org(state.device.class, state.org);
+
+      org = IS_RAM_ORG ? state.byte_addr :
+                         gp_processor_byte_to_real(state.processor, state.byte_addr);
 
       snprintf(buffer, sizeof(buffer), "_%s_%04X", state.obj.new_sec_name, org);
       /* RELOCT_ACCESS has always also RELOCT_F, which has already
          created this symbol.*/
-      if (type != RELOCT_ACCESS)
-        set_global(buffer, org, PERMANENT, IS_RAM_ORG ? gvt_static : gvt_address);
+      if (type != RELOCT_ACCESS) {
+        set_global(buffer, org, PERMANENT, IS_RAM_ORG ? GVT_STATIC : GVT_ADDRESS);
+      }
+
       s = get_symbol(state.stTop, buffer);
     }
     else {
@@ -494,12 +502,13 @@ add_reloc(struct pnode *p, short offs, unsigned short type)
     }
     if (s != NULL) {
       var = get_symbol_annotation(s);
+
       if (var != NULL) {
         switch(var->type) {
-        case gvt_extern:
-        case gvt_global:
-        case gvt_static:
-        case gvt_address:
+        case GVT_EXTERN:
+        case GVT_GLOBAL:
+        case GVT_STATIC:
+        case GVT_ADDRESS:
           coff_reloc(var->coff_num, offs, type);
           return;
 
@@ -510,7 +519,7 @@ add_reloc(struct pnode *p, short offs, unsigned short type)
     }
     return;
 
-  case unop:
+  case PTAG_UNOP:
     switch (p->value.unop.op) {
     case UPPER:
       add_reloc(p->value.unop.p0, offs, RELOCT_UPPER);
@@ -530,14 +539,14 @@ add_reloc(struct pnode *p, short offs, unsigned short type)
     case '~':
     case INCREMENT:
     case DECREMENT:
-      gpverror(GPE_UNRESOLVABLE);
+      gpverror(GPE_UNRESOLVABLE, NULL);
       return;
 
     default:
       assert(0);
     }
 
-  case binop:
+  case PTAG_BINOP:
     switch (p->value.binop.op) {
     case '+':
       /* The symbol can be in either position */
@@ -555,7 +564,7 @@ add_reloc(struct pnode *p, short offs, unsigned short type)
         add_reloc(p->value.binop.p0, offs - maybe_evaluate(p->value.binop.p1), type);
       }
       else {
-        gpverror(GPE_UNRESOLVABLE);
+        gpverror(GPE_UNRESOLVABLE, NULL);
       }
       return;
 
@@ -576,7 +585,7 @@ add_reloc(struct pnode *p, short offs, unsigned short type)
     case LOGICAL_AND:
     case LOGICAL_OR:
     case '=':
-      gpverror(GPE_UNRESOLVABLE);
+      gpverror(GPE_UNRESOLVABLE, NULL);
       return;
 
     default:
@@ -584,7 +593,7 @@ add_reloc(struct pnode *p, short offs, unsigned short type)
     }
     return;
 
-  case constant:
+  case PTAG_CONSTANT:
   default:
     assert(0);
   }
@@ -599,36 +608,38 @@ add_reloc(struct pnode *p, short offs, unsigned short type)
 */
 
 static int
-same_section(struct pnode *p)
+same_section(const struct pnode *p)
 {
-  struct pnode *p0;
-  struct pnode *p1;
+  const struct pnode *p0;
+  const struct pnode *p1;
   struct symbol *sym0;
   struct symbol *sym1;
   struct variable *var0;
   struct variable *var1;
 
-  if (!state.obj.enabled)
+  if (!state.obj.enabled) {
     return 0;
+  }
 
-  if ((p->tag == unop) &&
+  if ((p->tag == PTAG_UNOP) &&
       ((p->value.unop.op == UPPER) ||
        (p->value.unop.op == HIGH) ||
        (p->value.unop.op == LOW))) {
     p = p->value.unop.p0;
   }
 
-  if ((p->tag != binop) ||
+  if ((p->tag != PTAG_BINOP) ||
       (p->value.binop.op != '-') ||
-      (count_reloc(p->value.binop.p0) != 1))
+      (count_reloc(p->value.binop.p0) != 1)) {
     return 0;
+  }
 
   p0 = p->value.binop.p0;
   p1 = p->value.binop.p1;
 
-  if ((p0->tag != symbol) ||
-      (p1->tag != symbol))
+  if ((p0->tag != PTAG_SYMBOL) || (p1->tag != PTAG_SYMBOL)) {
     return 0;
+  }
 
   sym0 = get_symbol(state.stTop, p0->value.symbol);
   sym1 = get_symbol(state.stTop, p1->value.symbol);
@@ -637,22 +648,22 @@ same_section(struct pnode *p)
 
   /* They must come from the same section. Debug symbols are not placed
      in the global symbol table, so don't worry about symbol type. */
-  /* Fail if sections are not known (== 0) or not the same */
-  if (0 == var0->coff_section_num ||
-    0 == var1->coff_section_num ||
-    var0->coff_section_num != var1->coff_section_num)
+  /* Fail if sections are not known (== 0) or not the same. */
+  if ((var0->coff_section_num == 0) ||
+      (var1->coff_section_num == 0) ||
+      (var0->coff_section_num != var1->coff_section_num)) {
     return 0;
+  }
 
   return 1;
-
 }
 
-gpasmVal reloc_evaluate(struct pnode *p, unsigned short type)
+gpasmVal reloc_evaluate(const struct pnode *p, unsigned short type)
 {
   gpasmVal r = 0;
   int count = 0;
 
-  if (state.mode == absolute) {
+  if (state.mode == MODE_ABSOLUTE) {
     r = maybe_evaluate(p);
   }
   else {
@@ -669,7 +680,7 @@ gpasmVal reloc_evaluate(struct pnode *p, unsigned short type)
       }
       else {
         /* too many relocatable addresses */
-        gpverror(GPE_UNRESOLVABLE);
+        gpverror(GPE_UNRESOLVABLE, NULL);
         r = 0;
       }
     }
@@ -684,13 +695,16 @@ gpasmVal reloc_evaluate(struct pnode *p, unsigned short type)
 }
 
 /* evaluate the number of passes for the "fill" directive*/
-int eval_fill_number(struct pnode *p)
+int eval_fill_number(const struct pnode *p)
 {
   int number;
 
   number = maybe_evaluate(p);
-  if (state.device.class->rom_width == 8 && (number & 0x1) == 1)
-    gpverror(GPE_FILL_ODD);
+
+  if ((state.device.class->rom_width == 8) && ((number & 0x1) == 1)) {
+    gpverror(GPE_FILL_ODD, NULL);
+  }
+
   number = gp_processor_org_to_byte(state.device.class, number) >> 1;
 
   return number;
