@@ -23,13 +23,15 @@ Boston, MA 02111-1307, USA.  */
 #ifndef __GPERROR_H__
 #define __GPERROR_H__
 
+void gperror_init(void);
+
 void gperror(unsigned int code, char *message);
 void gpwarning(unsigned int code, char *message);
 void gpmessage(unsigned int code, char *message);
 
-void gpverror(unsigned int code, ...);
-void gpvwarning(unsigned int code, ...);
-void gpvmessage(unsigned int code, ...);
+void gpverror(unsigned int code, const char *message, ...);
+void gpvwarning(unsigned int code, const char *message, ...);
+void gpvmessage(unsigned int code, const char *message, ...);
 
 /* Alternate message functions. Only the prototypes are provided, the user
    must link their own function into gpasm. */
@@ -39,6 +41,8 @@ void user_message(unsigned int code, char *message);
 
 void add_code(int code);
 
+void gperror_close(void);
+
 /* Error codes
  *
  * The error codes gpasm defines here are identical to MPASM's
@@ -105,7 +109,11 @@ enum GPE_codes {
   /* gputils special errors */
   GPE_INTERNAL         = 1101,  /* Internal error */
   GPE_PARSER           = 1102,  /* Parser error */
-  GPE_SCANNER          = 1103   /* Scanner error */
+  GPE_SCANNER          = 1103,  /* Scanner error */
+
+  GPE_IDLOCS_P16E      = 1501,  /* IDLOCS directive use solely to the PIC16E family. */
+  GPE_NOF              = 1502,  /* The destination of the storage is not selected. */
+  GPE_NOA              = 1503   /* The access of RAM is not selected. */
 };
 
 /* Warning codes
@@ -164,7 +172,10 @@ enum GPM_codes {
   GPM_CBLOCK           = 313,
   GPM_W_MODIFIED       = 316,
   GPM_SPECIAL_MNEMONIC = 318,
-  GPM_UNKNOWN          = 319
+  GPM_UNKNOWN          = 319,
+
+  /* gputils special errors */
+  GPM_NOA              = 1301
 };
 
 #endif
