@@ -36,13 +36,13 @@ Boston, MA 02111-1307, USA.  */
 #define STRINGIFY(s) _str(s)
 #define _str(s) #s
 
-#define IS_EEPROM (PROC_CLASS_EEPROM8 == state.device.class || PROC_CLASS_EEPROM16 == state.device.class)
-#define IS_BYTE   (IS_16BIT_CORE || IS_EEPROM)
+#define IS_EEPROM (IS_EEPROM8 || IS_EEPROM16)
+#define IS_BYTE   (IS_PIC16E_CORE || IS_EEPROM)
 
 void
 lst_throw(void)
 {
-  if (state.lst.f) {
+  if (state.lst.f != NULL) {
     state.lst.page++;
     fprintf(state.lst.f,
             "%s%s %*.*s   %-28sPAGE %2d\n%s\n%s\n",
@@ -68,12 +68,12 @@ lst_page_start(void)
 {
   lst_throw();
   switch (state.lst.lst_state) {
-  case in_mem:
-    lst_line("LOC  OBJECT CODE     LINE SOURCE TEXT");
+  case LST_IN_MEM:
+    lst_line("LOC    OBJECT CODE    LINE  SOURCE TEXT");
     lst_line("  VALUE");
     break;
 
-  case in_symtab:
+  case LST_IN_SYMTAB:
     lst_line("SYMBOL TABLE");
     lst_line("%-32s  %-8s", "  LABEL", "  VALUE");
     break;
@@ -82,16 +82,18 @@ lst_page_start(void)
     lst_line(NULL);
     break;
   }
+
   lst_line(NULL);
 }
 
 static void
 lst_check_page_start(void)
 {
-  if (state.lst.linesperpage != 0 &&
-      (state.lst.lineofpage == 0 ||
-       state.lst.lineofpage > state.lst.linesperpage))
+  if ((state.lst.linesperpage != 0) &&
+      ((state.lst.lineofpage == 0) ||
+       (state.lst.lineofpage > state.lst.linesperpage))) {
     lst_page_start();
+  }
 }
 
 static int
@@ -100,8 +102,9 @@ lst_spaces(int n)
   int i = n;
 
   lst_check_page_start();
-  while (i--)
+  while (i--) {
     putc(' ', state.lst.f);
+  }
 
   return n;
 }
@@ -109,7 +112,7 @@ lst_spaces(int n)
 static void
 lst_eol(void)
 {
-  if (state.lst.f) {
+  if (state.lst.f != NULL) {
     putc('\n', state.lst.f);
     state.lst.line_number++;
     state.lst.lineofpage++;
@@ -120,8 +123,8 @@ lst_eol(void)
 void
 lst_line(const char *format, ...)
 {
-  if (state.lst.f) {
-    if (format) {
+  if (state.lst.f != NULL) {
+    if (format != NULL) {
       va_list args;
 
       lst_check_page_start();
@@ -136,9 +139,9 @@ lst_line(const char *format, ...)
 void
 lst_err_line(const char *type, unsigned int code, const char *format, va_list args)
 {
-  if (state.lst.f) {
+  if (state.lst.f != NULL) {
     lst_check_page_start();
-    fprintf(state.lst.f, "%s[%03d]%s: ", type, code, (0 == strcmp(type, "Error")) ? "  " : "");
+    fprintf(state.lst.f, "%s[%03d]%s: ", type, code, (strcmp(type, "Error") == 0) ? "  " : "");
     vfprintf(state.lst.f, format, args);
     lst_eol();
   }
@@ -150,8 +153,10 @@ static int
 lst_printf(const char *format, ...)
 {
   int r = 0;
-  if (state.lst.f) {
+
+  if (state.lst.f != NULL) {
     va_list args;
+
     lst_check_page_start();
     va_start(args, format);
     r = vfprintf(state.lst.f, format, args);
@@ -167,9 +172,9 @@ lst_init(void)
   state.lst.page = 0;
   state.lst.linesperpage = 59;
   state.lst.line_number = 1;
-  state.lst.memorymap = 1;
-  state.lst.symboltable = 1;
-  state.lst.lst_state = in_mem;
+  state.lst.memorymap = true;
+  state.lst.symboltable = true;
+  state.lst.lst_state = LST_IN_MEM;
 
   /* Determine state.startdate */
   gp_date_string(state.lst.startdate, sizeof(state.lst.startdate));
@@ -178,23 +183,19 @@ lst_init(void)
     state.lst.expand = true;
   }
 
-  if (state.cmd_line.lst_force)
-    state.lst.force = true;
-  else
-    state.lst.force = false;
-
+  state.lst.force = (state.cmd_line.lst_force) ? true : false;
   state.lst.config_address = 0;
   state.lst.title_name[0] = '\0';
   state.lst.subtitle_name[0] = '\0';
   state.lst.tabstop = 8;        /* Default tabstop every 8 */
-  state.lst.line_width = 132;   /* Default line widst is 132 */
+  state.lst.line_width = 132;   /* Default line width is 132 */
 
-  if (state.lstfile != named) {
+  if (state.lstfile != OUT_NAMED) {
     snprintf(state.lstfilename, sizeof(state.lstfilename),
              "%s.lst", state.basefilename);
   }
 
-  if (state.lstfile == suppress) {
+  if (state.lstfile == OUT_SUPPRESS) {
     state.lst.f = NULL;
     state.lst.enabled = false;
     unlink(state.lstfilename);
@@ -214,7 +215,7 @@ lst_init(void)
 void
 lst_memory_map(MemBlock *m)
 {
-#define MEM_IS_USED(m, i)  ((m)->memory ? (IS_BYTE ? ((m)->memory[i] & BYTE_USED_MASK) : (((m)->memory[2 * (i)] & BYTE_USED_MASK) || ((m)->memory[2 * (i) + 1] & BYTE_USED_MASK))) : 0)
+#define MEM_IS_USED(m, i)  (((m)->memory != NULL) ? (IS_BYTE ? ((m)->memory[i].data & BYTE_USED_MASK) : (((m)->memory[2 * (i)].data & BYTE_USED_MASK) || ((m)->memory[2 * (i) + 1].data & BYTE_USED_MASK))) : 0)
 
   int i, j, base, row_used, num_per_line, num_per_block;
 
@@ -241,9 +242,16 @@ lst_memory_map(MemBlock *m)
         }
       }
 
-      if(row_used) {
-        /* MPASM(X) compatible: print only lower 4 addres bytes */
-        lst_printf("%04X :", (i + base) & 0xffff);
+      if (row_used) {
+        if ((state.device.class != NULL) && state.show_full_addr && IS_PIC16E_CORE) {
+          /* Gpasm mode: Print all address digits. */
+          lst_printf("%0*X :", state.device.class->addr_digits, (i + base));
+        }
+        else {
+          /* MPASM(X) compatible: Print only lower 4 address digits. */
+          lst_printf("%04X :", (i + base) & 0xffff);
+        }
+
         for (j = 0; j < num_per_line; j++) {
           if ((j % num_per_block) == 0) {
             lst_printf(" ");
@@ -266,20 +274,24 @@ lst_memory_map(MemBlock *m)
   /* it seems that MPASM includes config bytes into program memory usage
    * count for 16 bit cores. See gpasm testsuite:
    * gpasm/testsuite/gpasm.mchip/listfiles/configX.lst */
-#define IS_PIC16  (PROC_CLASS_PIC16 == state.device.class  || PROC_CLASS_PIC16E == state.device.class)
+#define IS_PIC16  (IS_PIC16_CORE || IS_PIC16E_CORE)
 
   if (IS_EEPROM) {
     lst_line("Memory Bytes Used: %5i", b_memory_used(state.i_memory));
   }
   else {
-    unsigned int used = gp_processor_byte_to_org(state.device.class, (!IS_PIC16 && state.processor) ?
+    unsigned int used = gp_processor_byte_to_real(state.processor, ((!IS_PIC16) && (state.processor != NULL)) ?
       b_range_memory_used(state.i_memory, 0,
-        gp_processor_org_to_byte(state.device.class, state.processor->prog_mem_size)) :
+                          gp_processor_org_to_byte(state.device.class, state.processor->prog_mem_size)) :
       b_memory_used(state.i_memory));
+
     lst_line("Program Memory %s Used: %5i", IS_BYTE ? "Bytes" : "Words", used);
-    if (NULL != state.processor && 0 <= state.processor->prog_mem_size)
-      lst_line("Program Memory %s Free: %5u", IS_BYTE ? "Bytes" : "Words",
-        (used <= state.processor->prog_mem_size) ? state.processor->prog_mem_size - used : 0);
+
+    if ((state.processor != NULL) && (state.processor->prog_mem_size >= 0)) {
+      lst_line("Program Memory %s Free: %5u",
+               IS_BYTE ? "Bytes" : "Words",
+               (used <= state.processor->prog_mem_size) ? (state.processor->prog_mem_size - used) : 0);
+    }
   }
   lst_line(NULL);
 }
@@ -289,9 +301,9 @@ lst_close(void)
 {
   cod_lst_line(COD_LAST_LST_LINE);
 
-  state.lst.lst_state = in_none;
+  state.lst.lst_state = LST_IN_NONE;
 
-  if (state.lst.f) {
+  if (state.lst.f != NULL) {
     lst_line(NULL);
     lst_line("Errors   : %5d", state.num.errors);
     lst_line("Warnings : %5d reported, %5d suppressed",
@@ -313,9 +325,10 @@ find_reloc_by_address(unsigned short address)
 {
   gp_reloc_type *p;
 
-  for (p = state.obj.section->relocations; p; p = p->next) {
-    if (p->address == address)
+  for (p = state.obj.section->relocations; p != NULL; p = p->next) {
+    if (p->address == address) {
       break;
+    }
   }
 
   return p;
@@ -327,13 +340,15 @@ prev_reloc_type(void)
 {
   gp_reloc_type *p;
 
-  if (state.obj.section->relocations == state.obj.section->relocations_tail)
+  if (state.obj.section->relocations == state.obj.section->relocations_tail) {
     return 0;
+  }
 
-  for (p = state.obj.section->relocations; p->next && p->next != state.obj.section->relocations_tail; p = p->next)
+  for (p = state.obj.section->relocations;
+       (p->next != NULL) && (p->next != state.obj.section->relocations_tail); p = p->next)
     ;
   assert(p->next);
-  return (p->address == p->next->address) ? p->type : 0;
+  return ((p->address == p->next->address) ? p->type : 0);
 }
 
 /* print word value with undefined nibbles repleced by "?" */
@@ -351,17 +366,19 @@ prev_reloc_type(void)
 static int
 print_reloc(unsigned short type, unsigned short current_value)
 {
+  proc_class_t class = state.device.class;
+
   switch (type) {
   case RELOCT_CALL:
-    if (PROC_CLASS_PIC12 == state.device.class || PROC_CLASS_PIC12E == state.device.class ||
-      PROC_CLASS_SX == state.device.class || PROC_CLASS_PIC16E == state.device.class) {
+    if ((class == PROC_CLASS_PIC12) || (class == PROC_CLASS_PIC12E) ||
+        (class == PROC_CLASS_SX)    || (class == PROC_CLASS_PIC16E)) {
       ASSERT(0 == (current_value & 0xff));
       return lst_printf("%02X?? ", (current_value >> 8) & 0x00ff);
-    } else if (PROC_CLASS_PIC14 == state.device.class || PROC_CLASS_PIC14E == state.device.class) {
-      ASSERT(0 == (current_value & 0x7ff));
+    } else if ((class == PROC_CLASS_PIC14) || (class == PROC_CLASS_PIC14E) || (class == PROC_CLASS_PIC14EX)) {
+      ASSERT(0 == (current_value & MASK_PIC14_BRANCH));
       return lst_printf("%01X??? ", (current_value >> 12) & 0x000f);
-    } else if (PROC_CLASS_PIC16 == state.device.class) {
-      ASSERT(0 == (current_value & 0x1fff));
+    } else if (class == PROC_CLASS_PIC16) {
+      ASSERT(0 == (current_value & MASK_PIC16_BRANCH));
       return lst_printf("???? ");
     } else {
       ASSERT(0);
@@ -370,17 +387,17 @@ print_reloc(unsigned short type, unsigned short current_value)
     break;
 
   case RELOCT_GOTO:
-    if (PROC_CLASS_PIC12 == state.device.class || PROC_CLASS_PIC12E == state.device.class
-      || PROC_CLASS_SX == state.device.class) {
-      ASSERT(0 == (current_value & 0x1ff));
+    if ((class == PROC_CLASS_PIC12) || (class == PROC_CLASS_PIC12E) ||
+        (class == PROC_CLASS_SX)) {
+      ASSERT(0 == (current_value & MASK_PIC12_GOTO));
       return lst_printf("%01X??? ", (current_value >> 12) & 0x000f);
-    } else if (PROC_CLASS_PIC14 == state.device.class || PROC_CLASS_PIC14E == state.device.class) {
-      ASSERT(0 == (current_value & 0x7ff));
+    } else if ((class == PROC_CLASS_PIC14) || (class == PROC_CLASS_PIC14E) || (class == PROC_CLASS_PIC14EX)) {
+      ASSERT(0 == (current_value & MASK_PIC14_BRANCH));
       return lst_printf("%01X??? ", (current_value >> 12) & 0x000f);
-    } else if (PROC_CLASS_PIC16 == state.device.class) {
-      ASSERT(0 == (current_value & 0x1fff));
+    } else if (class == PROC_CLASS_PIC16) {
+      ASSERT(0 == (current_value & MASK_PIC16_BRANCH));
       return lst_printf("???? ");
-    } else if (PROC_CLASS_PIC16E == state.device.class) {
+    } else if (class == PROC_CLASS_PIC16E) {
       ASSERT(0 == (current_value & 0xff));
       return lst_printf("%02X?? ", (current_value >> 8) & 0x00ff);
     } else {
@@ -390,11 +407,11 @@ print_reloc(unsigned short type, unsigned short current_value)
     break;
 
   case RELOCT_LOW:
-    /* on non-16bits devices DB directive generates velues with
+    /* On non-16bits devices DB directive generates values with
      * low byte != 0 if more then one byte is defined, for example
      * DB 0x12, 0x34 generates 0x1234, so the following assertion fails
      * in such cases.
-     * TODO: this should be solved in DB directive handling function
+     * TODO: This should be solved in DB directive handling function
      * do_db(), file directive.c.
      * ASSERT(0 == (current_value & 0xff)); */
     return lst_printf("%02X?? ", (current_value >> 8) & 0x00ff);
@@ -430,12 +447,12 @@ print_reloc(unsigned short type, unsigned short current_value)
     break;
 
   case RELOCT_IBANKSEL:
-    if (PROC_CLASS_PIC14 == state.device.class) {
+    if (class == PROC_CLASS_PIC14) {
       ASSERT(0 == (current_value & 0x0f00));  /* 1383 or 1783 */
       return lst_printf("1?83 ");
-    } else if (PROC_CLASS_PIC14E == state.device.class) {
+    } else if ((class == PROC_CLASS_PIC14E) || (class == PROC_CLASS_PIC14EX)) {
       return lst_printf("???? ");
-    } else if (PROC_CLASS_PIC16 == state.device.class) {
+    } else if (class == PROC_CLASS_PIC16) {
       ASSERT(0 == (current_value & 0x00ff));
       return lst_printf("%02X?? ", (current_value >> 8) & 0x00ff);
     } else {
@@ -445,15 +462,15 @@ print_reloc(unsigned short type, unsigned short current_value)
     break;
 
   case RELOCT_F:
-    if (PROC_CLASS_SX == state.device.class) {
-      ASSERT(0 == (current_value & 0x0007));
+    if (class == PROC_CLASS_SX) {
+      ASSERT((current_value & 0x0007) == 0);
       return lst_printf("%03X?? ", (current_value >> 4) & 0x000f);
-    } else if (PROC_CLASS_PIC12 == state.device.class || PROC_CLASS_PIC12E == state.device.class) {
-      ASSERT(0 == (current_value & 0x001f));
-    } else if (PROC_CLASS_PIC14 == state.device.class || PROC_CLASS_PIC14E == state.device.class) {
-      ASSERT(0 == (current_value & 0x007f));
-    } else if (PROC_CLASS_PIC16 == state.device.class || PROC_CLASS_PIC16E == state.device.class) {
-      ASSERT(0 == (current_value & 0x00ff));
+    } else if ((class == PROC_CLASS_PIC12) || (class == PROC_CLASS_PIC12E)) {
+      ASSERT(0 == (current_value & MASK_PIC12_FILE));
+    } else if ((class == PROC_CLASS_PIC14) || (class == PROC_CLASS_PIC14E) || (class == PROC_CLASS_PIC14EX)) {
+      ASSERT(0 == (current_value & MASK_PIC14_FILE));
+    } else if ((class == PROC_CLASS_PIC16) || (class == PROC_CLASS_PIC16E)) {
+      ASSERT(0 == (current_value & MASK_PIC16_FILE));
     } else {
       ASSERT(0);
       return 0;
@@ -477,7 +494,7 @@ print_reloc(unsigned short type, unsigned short current_value)
     break;
 
   case RELOCT_GOTO2:
-    /* This is only used for PIC16E (pic18) */
+    /* This is only used for PIC16E (pic18). */
   case RELOCT_FF1:
   case RELOCT_FF2:
     /* removed assertion since it fails during sdcc pic16 library
@@ -491,10 +508,10 @@ print_reloc(unsigned short type, unsigned short current_value)
     break;
 
   case RELOCT_BRA:
-    if (PROC_CLASS_PIC14E == state.device.class) {
-      ASSERT(0 == (current_value & 0x01ff));
-    } else if (PROC_CLASS_PIC16E == state.device.class) {
-      ASSERT(0 == (current_value & 0x07ff));
+    if ((class == PROC_CLASS_PIC14E) || (class == PROC_CLASS_PIC14EX)) {
+      ASSERT(0 == (current_value & MASK_PIC14E_RBRA9));
+    } else if (class == PROC_CLASS_PIC16E) {
+      ASSERT(0 == (current_value & MASK_PIC16E_RBRA11));
     } else {
       ASSERT(0);
       return 0;
@@ -503,14 +520,14 @@ print_reloc(unsigned short type, unsigned short current_value)
     break;
 
   case RELOCT_MOVLB:
-    if (PROC_CLASS_PIC12E == state.device.class) {
-      ASSERT(0 == (current_value & 0x0007));
+    if (class == PROC_CLASS_PIC12E) {
+      ASSERT(0 == (current_value & MASK_PIC12E_BANK));
       return lst_printf("%03X? ", (current_value >> 4) & 0x0fff);
-    } else if (PROC_CLASS_PIC14E == state.device.class) {
-      ASSERT(0 == (current_value & 0x001f));
+    } else if ((class == PROC_CLASS_PIC14E) || (class == PROC_CLASS_PIC14EX)) {
+      ASSERT(0 == (current_value & MASK_PIC14E_BANK));
       return lst_printf("%02X?? ", (current_value >> 8) & 0x00ff);
-    } else if (PROC_CLASS_PIC16 == state.device.class || PROC_CLASS_PIC16E == state.device.class) {
-      ASSERT(0 == (current_value & 0x000f));
+    } else if ((class == PROC_CLASS_PIC16) || (class == PROC_CLASS_PIC16E)) {
+      ASSERT(0 == (current_value & MASK_PIC16E_BANK));
       return lst_printf("%03X? ", (current_value >> 4) & 0x0fff);
     } else {
       ASSERT(0);
@@ -519,7 +536,7 @@ print_reloc(unsigned short type, unsigned short current_value)
     break;
 
   case RELOCT_ACCESS:
-    if (PROC_CLASS_PIC16E == state.device.class) {
+    if (class == PROC_CLASS_PIC16E) {
       ASSERT(0 == (current_value & 0x00ff));
     } else {
       ASSERT(0);
@@ -535,8 +552,8 @@ print_reloc(unsigned short type, unsigned short current_value)
     break;
 
   case RELOCT_PAGESEL_MOVLP:
-    if (PROC_CLASS_PIC14E == state.device.class) {
-      ASSERT(0 == (current_value & 0x007f));
+    if ((class == PROC_CLASS_PIC14E) || (class == PROC_CLASS_PIC14EX)) {
+      ASSERT(0 == (current_value & MASK_PIC14E_PAGE));
     } else {
       ASSERT(0);
       return 0;
@@ -560,56 +577,53 @@ print_reloc(unsigned short type, unsigned short current_value)
 }
 
 static unsigned int
-lst_data(unsigned int pos, MemBlock *m, unsigned int byte_org,
-                      unsigned int bytes_emitted, unsigned short reloc_type)
+lst_data(unsigned int pos, MemBlock *m, unsigned int byte_addr,
+         unsigned int bytes_emitted, unsigned short reloc_type)
 {
+  unsigned char emit_byte;
+  unsigned short emit_word;
   int lst_bytes = 0;
+  gp_boolean is_eeprom_area = (gp_processor_is_eeprom_byte_addr(state.processor, byte_addr) >= 0) ? true : false;
 
-  /* when in a idata or byte packed section, print byte by byte */
-  if (PROC_CLASS_EEPROM8 == state.device.class || state.obj.new_sec_flags & (STYP_DATA | STYP_BPACK)) {
-    while (bytes_emitted > lst_bytes && pos + 3 <= LST_LINENUM_POS) {
-      unsigned char emit_byte;
-
-      b_memory_get(m, byte_org, &emit_byte);
+  /* When in a idata or byte packed section or eeprom area, print byte by byte. */
+  if (IS_EEPROM8 || is_eeprom_area || (state.obj.new_sec_flags & (STYP_DATA | STYP_BPACK))) {
+    while ((bytes_emitted > lst_bytes) && ((pos + 3) <= LST_LINENUM_POS)) {
+      b_memory_get(m, byte_addr, &emit_byte, NULL, NULL);
       pos += lst_printf("%02X ", emit_byte);
-      ++byte_org;
+      ++byte_addr;
       ++lst_bytes;
     }
   }
   else {    /* non-code pack section */
     /* list first byte on odd address */
-    if (bytes_emitted && (byte_org & 1) != 0) {
-      unsigned char emit_byte;
-
-      b_memory_get(m, byte_org, &emit_byte);
+    if (bytes_emitted && (byte_addr & 1)) {
+      b_memory_get(m, byte_addr, &emit_byte, NULL, NULL);
       pos += lst_printf("%02X ", emit_byte);
-      ++byte_org;
+      ++byte_addr;
       ++lst_bytes;
     }
     /* list full words */
-    while (bytes_emitted - lst_bytes > 1 && pos + 5 <= LST_LINENUM_POS) {
-      unsigned short emit_word;
+    while (((bytes_emitted - lst_bytes) > 1) && ((pos + 5) <= LST_LINENUM_POS)) {
+      state.device.class->i_memory_get(m, byte_addr, &emit_word, NULL, NULL);
 
-      state.device.class->i_memory_get(m, byte_org, &emit_word);
-
-      /* display '?' for undefined bytes if it is a relocatable code */
-      if (0 != reloc_type) {
+      /* Display '?' for undefined bytes if it is a relocatable code. */
+      if (reloc_type != 0) {
         int n = print_reloc(reloc_type, emit_word);
-        pos += (0 == n) ? lst_printf("%04X ", emit_word) : n;
-      }
-      else
-        pos += lst_printf("%04X ", emit_word);
 
-      byte_org += 2;
+        pos += (n == 0) ? lst_printf("%04X ", emit_word) : n;
+      }
+      else {
+        pos += lst_printf("%04X ", emit_word);
+      }
+
+      byte_addr += 2;
       lst_bytes += 2;
     }
 
-    if (bytes_emitted - lst_bytes == 1 && pos + 3 <= LST_LINENUM_POS) {
-      unsigned char emit_byte;
-
-      b_memory_get(m, byte_org, &emit_byte);
+    if (((bytes_emitted - lst_bytes) == 1) && ((pos + 3) <= LST_LINENUM_POS)) {
+      b_memory_get(m, byte_addr, &emit_byte, NULL, NULL);
       pos += lst_printf("%02X ", emit_byte);
-      ++byte_org;
+      ++byte_addr;
       ++lst_bytes;
     }
   }
@@ -625,10 +639,10 @@ lst_format_line(const char *src_line, int value)
 {
   unsigned int emitted = 0;
   unsigned int emitted_lines = 0;
-  unsigned int byte_org = 0;
+  unsigned int byte_addr = 0;
   unsigned int bytes_emitted = 0;
   unsigned int lst_bytes;
-  const char *addr_fmt = IS_16BIT_CORE ? "%06X " : (IS_EEPROM ? "%04X " : "%04X   ");
+  const char *addr_fmt = IS_PIC16E_CORE ? "%06X " : (IS_EEPROM ? "%04X " : "%04X   ");
 #define ADDR_LEN 7
   unsigned int pos = 0;
   unsigned short reloc_type;
@@ -636,31 +650,31 @@ lst_format_line(const char *src_line, int value)
 
   assert(src_line != NULL);
 
-  if (state.mode == relocatable &&
-    state.obj.section &&
-    state.obj.new_sec_flags & STYP_TEXT &&
-    state.obj.section->relocations_tail) {
-      if (state.obj.section->address + state.obj.section->relocations_tail->address > state.lst.line.was_org) {
+  if ((state.mode == MODE_RELOCATABLE) && (state.obj.section != NULL) &&
+      (state.obj.new_sec_flags & STYP_TEXT) && (state.obj.section->relocations_tail != NULL)) {
+      if ((state.obj.section->address + state.obj.section->relocations_tail->address) > state.lst.line.was_byte_addr) {
         /* already passed it, go back to the history */
-        gp_reloc_type *p = find_reloc_by_address(state.lst.line.was_org);
-        reloc_type = p ? p->type : 0;
+        gp_reloc_type *p = find_reloc_by_address(state.lst.line.was_byte_addr);
+        reloc_type = (p != NULL) ? p->type : 0;
       }
-      else if (state.obj.section->address + state.obj.section->relocations_tail->address == state.lst.line.was_org) {
+      else if ((state.obj.section->address + state.obj.section->relocations_tail->address) == state.lst.line.was_byte_addr) {
         reloc_type = state.obj.section->relocations_tail->type;
       }
-      else
+      else {
         reloc_type = 0;
+      }
     }
-   else
-     reloc_type = 0;
+  else {
+    reloc_type = 0;
+  }
 
   switch (state.lst.line.linetype) {
-  case insn:
-    emitted_lines = emitted = state.org - state.lst.line.was_org;
+  case LTY_INSN:
+    emitted_lines = emitted = state.byte_addr - state.lst.line.was_byte_addr;
     break;
 
-  case data:
-    emitted = state.org - state.lst.line.was_org;
+  case LTY_DATA:
+    emitted = state.byte_addr - state.lst.line.was_byte_addr;
     if ((SECTION_FLAGS & (STYP_TEXT | (state.mpasm_compatible ? STYP_BPACK : 0))) == STYP_TEXT) {
       /* generate line numbers for data directives in program memory;
        * in mpasm compatibility mode code_pack doesn't generate line numbers */
@@ -668,18 +682,18 @@ lst_format_line(const char *src_line, int value)
     }
     break;
 
-  case res:
+  case LTY_RES:
     if (SECTION_FLAGS & STYP_DATA) {
       /* generate data listing for idata */
-      emitted = state.org - state.lst.line.was_org;
+      emitted = state.byte_addr - state.lst.line.was_byte_addr;
     }
     else if (!IS_RAM_ORG) {
       /* generate line numbers for res directives in program memory */
-      emitted_lines = emitted = state.org - state.lst.line.was_org;
+      emitted_lines = emitted = state.byte_addr - state.lst.line.was_byte_addr;
     }
     break;
 
-  case idlocs:
+  case LTY_IDLOCS:
     /* always 8 bytes (4 words) */
     emitted = 8;
     break;
@@ -691,86 +705,90 @@ lst_format_line(const char *src_line, int value)
   coff_linenum(emitted_lines);
 
   /* Don't write to file if list is disabled with NOLIST directive */
-  if (!state.lst.enabled)
+  if (!state.lst.enabled) {
     return;
+  }
 
   switch (state.lst.line.linetype) {
-  case equ:
-  case set:
+  case LTY_EQU:
+  case LTY_SET:
     pos += lst_printf("  %08X", value);
     lst_spaces(LST_LINENUM_POS - pos);
     break;
 
-  case set4:
+  case LTY_SET4:
     pos += lst_printf("  %04X", value & 0xffff);
     lst_spaces(LST_LINENUM_POS - pos);
     break;
 
-  case org:
-    pos += lst_printf(addr_fmt, gp_processor_byte_to_org(state.device.class, state.org));
+  case LTY_ORG:
+    pos += lst_printf(addr_fmt, gp_processor_byte_to_real(state.processor, state.byte_addr));
     lst_spaces(LST_LINENUM_POS - pos);
     break;
 
-  case idlocs:
+  case LTY_IDLOCS:
     /* not used for 16 bit devices, config is used */
     m = state.c_memory;
-    pos += lst_printf(addr_fmt, gp_processor_byte_to_org(state.device.class, state.device.id_location));
+    pos += lst_printf(addr_fmt, gp_processor_byte_to_real(state.processor, state.device.id_location));
     lst_bytes = lst_data(pos, m, state.device.id_location, emitted, reloc_type);
-    byte_org = state.device.id_location + lst_bytes;
+    byte_addr = state.device.id_location + lst_bytes;
     bytes_emitted = emitted - lst_bytes;
     break;
 
-  case data:
-  case res:
-    pos += lst_printf(addr_fmt, state.lst.line.was_org);
+  case LTY_DATA:
+  case LTY_RES:
+    pos += lst_printf(addr_fmt, state.lst.line.was_byte_addr);
     goto lst_data;
 
-  case insn:
-    pos += lst_printf(addr_fmt, gp_processor_byte_to_org(state.device.class, state.lst.line.was_org));
-  lst_data:
-    lst_bytes = lst_data(pos, m, state.lst.line.was_org, emitted, reloc_type);
-    byte_org = state.lst.line.was_org + lst_bytes;
+  case LTY_INSN:
+    pos += lst_printf(addr_fmt, gp_processor_byte_to_real(state.processor, state.lst.line.was_byte_addr));
+
+lst_data:
+
+    lst_bytes = lst_data(pos, m, state.lst.line.was_byte_addr, emitted, reloc_type);
+    byte_addr = state.lst.line.was_byte_addr + lst_bytes;
     bytes_emitted = emitted - lst_bytes;
     break;
 
-  case config:
-    if (IS_16BIT_CORE) {
+  case LTY_CONFIG:
+    if (IS_PIC16E_CORE) {
       /* config data is byte addressable, but we only want to print
          words in the list file. */
       if (state.lst.config_address == CONFIG4L) {
         /* Special case */
         unsigned short word;
+
         state.device.class->i_memory_get(state.c_memory,
-                                         state.lst.config_address, &word);
+                                         state.lst.config_address, &word, NULL, NULL);
         pos += lst_printf(addr_fmt, state.lst.config_address);
         pos += lst_printf("%04X", word);
         lst_spaces(LST_LINENUM_POS - pos);
-      } else if((state.lst.config_address & 0x1) == 0) {
+      } else if ((state.lst.config_address & 1) == 0) {
         /* if it is an even address don't print anything */
         lst_spaces(LST_LINENUM_POS);
       } else {
         unsigned short word;
+
         state.device.class->i_memory_get(state.c_memory,
-                                         state.lst.config_address - 1, &word);
+                                         state.lst.config_address - 1, &word, NULL, NULL);
         pos += lst_printf(addr_fmt, state.lst.config_address - 1);
         pos += lst_printf("%04X", word);
         lst_spaces(LST_LINENUM_POS - pos);
       }
     } else {
       unsigned short word;
-      state.device.class->i_memory_get(state.c_memory,
-                                       state.lst.config_address, &word);
-      pos += lst_printf(addr_fmt,
-                        gp_processor_byte_to_org(state.device.class,
-                                                 state.lst.config_address));
+
+      state.device.class->i_memory_get(state.c_memory, state.lst.config_address, &word, NULL, NULL);
+      pos += lst_printf(addr_fmt, gp_processor_byte_to_real(state.processor,
+                        state.lst.config_address));
       pos += lst_printf("%04X", word);
       lst_spaces(LST_LINENUM_POS - pos);
     }
     break;
 
-  case sec:
-  case dir:
-  case none:
+  case LTY_SEC:
+  case LTY_DIR:
+  case LTY_NONE:
   default:
     lst_spaces(LST_LINENUM_POS);
     break;
@@ -788,9 +806,9 @@ lst_format_line(const char *src_line, int value)
     int lst_column = LST_SRC_POS; /* current listing column after the SRC_POS */
     const char *p = src_line;
 
-    while (*p) {
+    while (*p != '\0') {
       if (*p == '\t') {
-        int len = state.lst.tabstop - src_column % state.lst.tabstop;
+        int len = state.lst.tabstop - (src_column % state.lst.tabstop);
 
         while (len--) {
           if (lst_column >= state.lst.line_width) {
@@ -826,8 +844,8 @@ lst_format_line(const char *src_line, int value)
       /* data left to print on separate lines */
 
       pos = lst_spaces(ADDR_LEN);
-      lst_bytes = lst_data(pos, m, byte_org, bytes_emitted, reloc_type);
-      byte_org += lst_bytes;
+      lst_bytes = lst_data(pos, m, byte_addr, bytes_emitted, reloc_type);
+      byte_addr += lst_bytes;
       bytes_emitted -= lst_bytes;
       lst_eol();
     }
@@ -841,16 +859,19 @@ cod_symbol_table(void)
   int i;
   struct symbol **lst, **ps, *s;
 
-  ps = lst = malloc(state.stGlobal->count * sizeof (struct symbol *));
+  ps = lst = GP_Malloc(state.stGlobal->count * sizeof(struct symbol *));
 
-  for (i = 0; i < HASH_SIZE; i++)
-    for (s = state.stGlobal->hash_table[i]; s; s = s->next)
-      if (NULL != s)
+  for (i = 0; i < HASH_SIZE; i++) {
+    for (s = state.stGlobal->hash_table[i]; s != NULL; s = s->next) {
+      if (s != NULL) {
         *ps++ = s;
+      }
+    }
+  }
 
   assert(ps == &lst[state.stGlobal->count]);
 
-  qsort(lst, state.stGlobal->count, sizeof (struct symbol *), symbol_compare);
+  qsort(lst, state.stGlobal->count, sizeof(struct symbol *), symbol_compare);
 
   cod_write_symbols(lst, state.stGlobal->count);
 
@@ -859,14 +880,19 @@ cod_symbol_table(void)
 
 /* append the symbol table to the .lst file */
 struct lst_symbol_s {
-  struct symbol *sym;
-  enum lst_sym_type_e { LST_SYMBOL, LST_DEFINE, LST_MACRO } type;
+  const struct symbol *sym;
+  enum lst_sym_type_e {
+    LST_SYMBOL,
+    LST_DEFINE,
+    LST_MACRO
+  } type;
 };
 
 static int
 lst_symbol_compare(const void *p0, const void *p1)
 {
-  return strcmp(((struct lst_symbol_s *)p0)->sym->name, ((struct lst_symbol_s *)p1)->sym->name);
+  return strcmp(((const struct lst_symbol_s *)p0)->sym->name,
+                ((const struct lst_symbol_s *)p1)->sym->name);
 }
 
 void
@@ -876,7 +902,7 @@ lst_symbol_table(void)
   struct lst_symbol_s *lst, *ps;
   struct symbol *s;
   int count = state.stGlobal->count + state.stDefines->count + state.stMacros->count;
-  state.lst.lst_state = in_symtab;
+  state.lst.lst_state = LST_IN_SYMTAB;
 
   lst_line("SYMBOL TABLE");
   lst_line("  LABEL                             VALUE");
@@ -884,50 +910,53 @@ lst_symbol_table(void)
 
   cod_symbol_table();
 
-  ps = lst = malloc(count * sizeof (struct lst_symbol_s));
+  ps = lst = GP_Malloc(count * sizeof(struct lst_symbol_s));
 
-  for (i = 0; i < HASH_SIZE; i++)
-    for (s = state.stGlobal->hash_table[i]; s; s = s->next) {
+  for (i = 0; i < HASH_SIZE; i++) {
+    for (s = state.stGlobal->hash_table[i]; s != NULL; s = s->next) {
       ps->sym = s;
       ps->type = LST_SYMBOL;
       ++ps;
     }
+  }
 
-  for (i = 0; i < HASH_SIZE; i++)
-    for (s = state.stDefines->hash_table[i]; s; s = s->next) {
+  for (i = 0; i < HASH_SIZE; i++) {
+    for (s = state.stDefines->hash_table[i]; s != NULL; s = s->next) {
       ps->sym = s;
       ps->type = LST_DEFINE;
       ++ps;
     }
+  }
 
-  for (i = 0; i < HASH_SIZE; i++)
-    for (s = state.stMacros->hash_table[i]; s; s = s->next) {
+  for (i = 0; i < HASH_SIZE; i++) {
+    for (s = state.stMacros->hash_table[i]; s != NULL; s = s->next) {
       ps->sym = s;
       ps->type = LST_MACRO;
       ++ps;
     }
+  }
 
   assert(ps == &lst[count]);
 
-  qsort(lst, count, sizeof (struct lst_symbol_s), lst_symbol_compare);
+  qsort(lst, count, sizeof(struct lst_symbol_s), lst_symbol_compare);
 
   for (i = 0; i < count; i++) {
-    void *p = get_symbol_annotation(lst[i].sym);
+    const void *p = get_symbol_annotation(lst[i].sym);
     const char *name = get_symbol_name(lst[i].sym);
 
     switch (lst[i].type) {
     case LST_SYMBOL:
       /* symbol */
-      lst_line("%-32s  %08X", name, p ? ((struct variable *)p)->value : 0);
+      lst_line("%-32s  %08X", name, (p != NULL) ? ((const struct variable *)p)->value : 0);
       break;
 
     case LST_DEFINE:
       /* define */
-      if (p) {
-        assert(list == ((struct pnode *)p)->tag);
-        assert(string == HEAD((struct pnode *)p)->tag);
+      if (p != NULL) {
+        assert(((const struct pnode *)p)->tag == PTAG_LIST);
+        assert(HEAD((const struct pnode *)p)->tag == PTAG_STRING);
       }
-      lst_line("%-32s  %s", name, p ? HEAD((struct pnode *)p)->value.string : "");
+      lst_line("%-32s  %s", name, (p != NULL) ? HEAD((const struct pnode *)p)->value.string : "");
       break;
 
     case LST_MACRO:
@@ -947,14 +976,15 @@ lst_symbol_table(void)
 void
 preproc_init(void)
 {
-  if (NULL != state.preproc.preprocfilename) {
-    if ('-' == state.preproc.preprocfilename[0] && '\0' == state.preproc.preprocfilename[1])
+  const char *name = state.preproc.preprocfilename;
+
+  if (name != NULL) {
+    if ((name[0] == '-') && (name[1] == '\0')) {
       state.preproc.f = stdout;
-    else {
-      if (NULL == (state.preproc.f = fopen(state.preproc.preprocfilename, "wt"))) {
-        perror(state.preproc.preprocfilename);
-        exit(1);
-      }
+    }
+    else if ((state.preproc.f = fopen(name, "wt")) == NULL) {
+      perror(name);
+      exit(1);
     }
   }
 }
@@ -962,10 +992,12 @@ preproc_init(void)
 void
 preproc_emit(void)
 {
-  if (NULL != state.preproc.f) {
-    if (state.preproc.do_emit && asm_enabled())
+  if (state.preproc.f != NULL) {
+    if (state.preproc.do_emit && asm_enabled()) {
       fprintf(state.preproc.f, "%s\n", state.preproc.curr_src_line.line);
-    else
+    }
+    else {
       state.preproc.do_emit = true;
+    }
   }
 }
