@@ -22,12 +22,15 @@ Boston, MA 02111-1307, USA.  */
 #ifndef __SCAN_H__
 #define __SCAN_H__
 
-void open_src(char *name, int is_include_file);
-void close_file();
-void execute_exitm();
-void execute_macro(struct macro_head *h, int is_while);
+int yylex_destroy(void);
 
-extern int force_ident;
-extern int force_decimal;
+void open_src(const char *name, gp_boolean is_include_file);
+void close_file(void);
+void execute_exitm(void);
+void execute_macro(struct macro_head *h, gp_boolean is_while);
+int found_end(void);
+
+extern gp_boolean force_ident;
+extern gp_boolean force_decimal;
 
 #endif
