@@ -29,20 +29,70 @@ Boston, MA 02111-1307, USA.  */
 
 static struct file_context *last = NULL;
 
+/* MPASM compatible stripped version of strtoul:
+ * gp_strtoi returns low sizeof(int) * 8 bits of the value:
+ * value & ((2 ^ (sizeof(int) * 8)) - 1);
+ * strtoul returns ULONG_MAX if the correct value is outside the range
+ * of representable values (see man strtoul). The behavior is also
+ * platform dependent: (int)strtoul("123456789", 16) returns -1 on 32 bit
+ * platforms and 0x23456789 on 64 bit platforms. */
+static int
+gp_strtoi(const char *string, char **endptr, int radix)
+{
+  int value = 0;
+  int sign  = 1;
+  int digit = 0;
+  int ch;
+
+  if (*string == '-') {
+    sign = -1;
+    ++string;
+  }
+
+  while (1) {
+    ch = *string;
+
+    if (isdigit(ch)) {
+      digit = ch - '0';
+    }
+    else if (islower(ch)) {
+      digit = ch - ('a' - 10);
+    }
+    else if (isupper(ch)) {
+      digit = ch - ('A' - 10);
+    }
+    else {
+      break;
+    }
+
+    if (digit < radix) {
+      value = value * radix + digit;
+    }
+    else {
+      break;
+    }
+
+    ++string;
+  }
+
+  *endptr = (char *)string;
+  return (value * sign);
+}
+
 int
-stringtolong(char *string, int radix)
+stringtolong(const char *string, int radix)
 {
   char *endptr;
   int value;
-  
-  value = strtoul(string, &endptr, radix);                           
+
+  value = gp_strtoi(string, &endptr, radix);
   if ((endptr == NULL) || (*endptr != '\0')) {
     char complaint[80];
 
     snprintf(complaint, sizeof(complaint),
              isprint(*endptr) ?
-             "Illegal character '%c' in numeric constant " :
-             "Illegal character %#x in numeric constant" ,
+             "Illegal character '%c' in numeric constant." :
+             "Illegal character %#x in numeric constant.",
              *endptr);
     gperror(GPE_UNKNOWN, complaint);
   }
@@ -50,7 +100,8 @@ stringtolong(char *string, int radix)
   return value;
 }
 
-int gpasm_magic(char *c)
+int
+gpasm_magic(const char *c)
 {
   if (c[0] == '\\') {
     switch (c[1]) {
@@ -96,31 +147,34 @@ convert_escaped_char(char *str, char c)
   char *src  = str;
   char *dest = str;
 
-  if (!str)
+  if (str == NULL) {
     return str;
+  }
 
   while (*src) {
-    if (*src =='\\' && src[1] == c)
+    if ((*src =='\\') && (src[1] == c)) {
       src++;
+    }
     *dest++ = *src++;
   }
-  *dest=0;
+
+  *dest = '\0';
 
   return str;
 }
 
 /* Determine the value of the escape char pointed to by ps.  Return a pointer
-to the next character. */ 
+to the next character. */
 
-char *
-convert_escape_chars(char *ps, int *value)
+const char *
+convert_escape_chars(const char *ps, int *value)
 {
   int count;
-  
+
   if (*ps != '\\') {
     *value = *ps++;
   } else {
-    /* escape char, convert its value and write to the new string */    
+    /* escape char, convert its value and write to the new string */
     switch (ps[1]) {
     case '0':
     case '1':
@@ -136,34 +190,39 @@ convert_escape_chars(char *ps, int *value)
       ps++;
 
       while (count < 3) {
-        if (*ps < '0' || *ps > '7')
+        if ((*ps < '0') || (*ps > '7')) {
           break;
+        }
+
         *value = (*value << 3) + *ps - '0';
         ps++;
         count++;
-      }        
+      }
       break;
     case 'x':
       /* hex number */
       if ((ps[2] == '\0') || (ps[3] == '\0')) {
-        gperror(GPE_UNKNOWN, "missing hex value in \\x escape character");
+        gperror(GPE_UNKNOWN, "Missing hex value in \\x escape character.");
         *value = 0;
         /* return a NULL character */
-        ps[2] = '\0';
         ps += 2;
+
+        if (*ps != '\0') {
+          ++ps;
+        }
       } else {
         char buffer[3];
 
         buffer[0] = ps[2];
         buffer[1] = ps[3];
-        buffer[2] = 0;
+        buffer[2] = '\0';
         *value = stringtolong(buffer, 16);
         ps += 4;
       }
       break;
     default:
       if (ps[1] == '\0') {
-        gperror(GPE_UNKNOWN, "missing value in \\ escape character");
+        gperror(GPE_UNKNOWN, "Missing value in \\ escape character.");
         *value = 0;
         /* return a NULL character */
         ps++;
@@ -173,7 +232,7 @@ convert_escape_chars(char *ps, int *value)
       }
     }
   }
-  
+
   return ps;
 
 }
@@ -182,20 +241,24 @@ convert_escape_chars(char *ps, int *value)
  * single-character string literal in an expression can be coerced to a
  * character literal. coerce_str1 converts a string-type pnode to a
  * constant-type pnode in-place. */
-void coerce_str1(struct pnode *exp)
+void
+coerce_str1(struct pnode *exp)
 {
-  if ((exp != NULL) && (exp->tag == string)) {
+  if ((exp != NULL) && (exp->tag == PTAG_STRING)) {
     int value;
-    char *pc = convert_escape_chars(exp->value.string, &value);
+
+    const char *pc = convert_escape_chars(exp->value.string, &value);
+
     if (*pc == '\0') {
       /* castable string, make the conversion */
-      exp->tag = constant;
+      exp->tag = PTAG_CONSTANT;
       exp->value.constant = value;
     }
   }
 }
 
-void set_global(char *name,
+void
+set_global(const char *name,
                 gpasmVal value,
                 enum globalLife lifetime,
                 enum gpasmValTypes type)
@@ -207,12 +270,16 @@ void set_global(char *name,
      tables) for the symbol.  If not found, then add it to the global
      symbol table.  */
   sym = get_symbol(state.stTop, name);
-  if (sym == NULL)
+
+  if (sym == NULL) {
     sym = add_symbol(state.stGlobal, name);
+  }
+
   var = get_symbol_annotation(sym);
+
   if (var == NULL) {
     /* new symbol */
-    var = malloc(sizeof(*var));
+    var = GP_Malloc(sizeof(*var));
     annotate_symbol(sym, var);
     var->value = value;
     var->coff_num = state.obj.symbol_num;
@@ -223,24 +290,24 @@ void set_global(char *name,
 
     /* increment the index into the coff symbol table for the relocations */
     switch(type) {
-    case gvt_extern:
-    case gvt_global:
-    case gvt_static:
-    case gvt_address:
-    case gvt_debug:
-    case gvt_absolute:
+    case GVT_EXTERN:
+    case GVT_GLOBAL:
+    case GVT_STATIC:
+    case GVT_ADDRESS:
+    case GVT_DEBUG:
+    case GVT_ABSOLUTE:
       state.obj.symbol_num++;
       break;
+
     default:
       break;
     }
-
   } else if (lifetime == TEMPORARY) {
     /*
      * TSD - the following embarrassing piece of code is a hack
      *       to fix a problem when global variables are changed
      *       during the expansion of a macro. Macros are expanded
-     *       by running through them twice. if you have a stetement
+     *       by running through them twice. if you have a statement
      *       like:
      *       some_var set some_var + 1
      *       then this is incremented twice! So the if statement
@@ -253,42 +320,46 @@ void set_global(char *name,
     char *coff_name;
 
     if (var->value != value) {
-      char message[BUFSIZ];
-
-      snprintf(message, sizeof(message),
-               "Value of symbol \"%s\" differs on second pass\n pass 1=%d,  pass 2=%d",
-               name,var->value,value);
-      gperror(GPE_DIFFLAB, message);      
+      gpverror(GPE_DIFFLAB, NULL, name);
     }
 
     coff_name = coff_local_name(name);
     coff_add_sym(coff_name, value, var->type);
 
-    if (coff_name != NULL)
+    if (coff_name != NULL) {
       free(coff_name);
+    }
   }
-
 }
 
-void purge_temp_symbols(struct symbol_table *table) {
+void
+purge_temp_symbols(struct symbol_table *table) {
   int i;
+
   if (table != NULL) {
     for (i = 0; i < HASH_SIZE; ++i) {
       struct symbol *cur_symbol;
       struct symbol *last_symbol = NULL;
+
       cur_symbol = table->hash_table[i];
       while (cur_symbol != NULL) {
         if (cur_symbol != NULL) {
           struct variable *var = (struct variable *)get_symbol_annotation(cur_symbol);
+
           if (var != NULL) {
             if (var->lifetime == TEMPORARY) {
               struct symbol *next_symbol = cur_symbol->next;
+
               free(cur_symbol);
               table->count--;
-              if (last_symbol == NULL)
+
+              if (last_symbol == NULL) {
                 table->hash_table[i] = next_symbol;
-              else
+              }
+              else {
                 last_symbol->next = next_symbol;
+              }
+
               cur_symbol = next_symbol;
               continue;
             }
@@ -303,22 +374,26 @@ void purge_temp_symbols(struct symbol_table *table) {
   }
 }
 
-void select_errorlevel(int level)
+void
+select_errorlevel(int level)
 {
   if (state.cmd_line.error_level) {
-    gpmessage(GPM_SUPVAL, NULL);
+    gpvmessage(GPM_SUPVAL, NULL);
   } else {
-    if (level == 0) {
-      state.error_level = 0;
-    } else if (level == 1) {
-      state.error_level = 1;
-    } else if (level == 2) {
-      state.error_level = 2;
+    if ((level >= 0) && (level <= 2)) {
+      if (state.cmd_line.strict_level && (state.strict_level > 0)) {
+        /*
+        The "strict messages" more important than the other messages, therefore
+        enable each messages.
+        */
+        state.error_level = 0;
+      }
+      else {
+        state.error_level = level;
+      }
     } else {
       if (state.pass == 0) {
-	fprintf(stderr,
-		"Error: invalid warning level \"%i\"\n",
-		level);
+        fprintf(stderr, "Error: Invalid warning level \"%i\".\n", level);
       } else {
         gperror(GPE_ILLEGAL_ARGU, "Expected w= 0, 1, 2");
       }
@@ -326,77 +401,107 @@ void select_errorlevel(int level)
   }
 }
 
-void select_expand(char *expand)
+void
+select_strictlevel(int level)
+{
+  if (state.cmd_line.strict_level) {
+    gpvmessage(GPM_SUPVAL, NULL);
+  } else {
+    if ((level >= 0) && (level <= 2)) {
+      state.strict_level = level;
+
+      if ((level > 0) && state.cmd_line.error_level) {
+        /* Enable each messages. */
+        state.error_level = 0;
+      }
+    } else {
+      if (state.pass == 0) {
+        fprintf(stderr, "Error: Invalid strict level \"%i\".\n", level);
+      } else {
+        gperror(GPE_ILLEGAL_ARGU, "Expected S= 0, 1, 2");
+      }
+    }
+  }
+}
+
+void
+select_expand(const char *expand)
 {
   if (state.cmd_line.macro_expand) {
-    gpmessage(GPM_SUPLIN, NULL);
+    gpvmessage(GPM_SUPLIN, NULL);
   } else {
-    if (strcasecmp(expand, "ON") == 0) {
+    if (strcasecmp(expand, "on") == 0) {
       state.lst.expand = true;
-    } else if (strcasecmp(expand, "OFF") == 0) {
+    } else if (strcasecmp(expand, "off") == 0) {
       state.lst.expand = false;
     } else {
       state.lst.expand = true;
+
       if (state.pass == 0) {
-	fprintf(stderr,
-		"Error: invalid option \"%s\"\n",
-		expand);
+        fprintf(stderr, "Error: Invalid option \"%s\".\n", expand);
       } else {
-        gpwarning(GPE_ILLEGAL_ARGU, "Expected ON or OFF");
+        gperror(GPE_ILLEGAL_ARGU, "Expected ON or OFF.");
       }
     }
   }
 }
 
-void select_hexformat(char *format_name)
+#define STR_INHX8M              "inhx8m"
+#define STR_INHX8S              "inhx8s"
+#define STR_INHX16              "inhx16"
+#define STR_INHX32              "inhx32"
+
+void
+select_hexformat(const char *format_name)
 {
   if (state.cmd_line.hex_format) {
-    gpwarning(GPW_CMDLINE_HEXFMT, NULL);
+    gpvwarning(GPW_CMDLINE_HEXFMT, NULL);
   } else {
-    if (strcasecmp(format_name, "inhx8m") == 0) {
-      state.hex_format = inhx8m;
-    } else if (strcasecmp(format_name, "inhx8s") == 0) {
-      state.hex_format = inhx8s;
-    } else if (strcasecmp(format_name, "inhx16") == 0) {
-      state.hex_format = inhx16;
-    } else if (strcasecmp(format_name, "inhx32") == 0) {
-      state.hex_format = inhx32;
+    if (strcasecmp(format_name, STR_INHX8M) == 0) {
+      state.hex_format = INHX8M;
+    } else if (strcasecmp(format_name, STR_INHX8S) == 0) {
+      state.hex_format = INHX8S;
+    } else if (strcasecmp(format_name, STR_INHX16) == 0) {
+      state.hex_format = INHX16;
+    } else if (strcasecmp(format_name, STR_INHX32) == 0) {
+      state.hex_format = INHX32;
     } else {
-      state.hex_format = inhx8m;
+      state.hex_format = INHX8M;
+
       if (state.pass == 0) {
-	fprintf(stderr,
-		"Error: invalid format \"%s\"\n",
-		format_name);
+        fprintf(stderr, "Error: Invalid format \"%s\".\n", format_name);
       } else {
-        gperror(GPE_ILLEGAL_ARGU, "Expected inhx8m, inhx8s, inhx16, or inhx32");
+        gperror(GPE_ILLEGAL_ARGU, "Expected " STR_INHX8M ", " STR_INHX8S ", " STR_INHX16 ", or " STR_INHX32 ".");
       }
     }
   }
 }
 
-void select_radix(char *radix_name)
+void
+select_radix(const char *radix_name)
 {
   if (state.cmd_line.radix) {
-    gpwarning(GPW_CMDLINE_RADIX, NULL);
+    gpvwarning(GPW_CMDLINE_RADIX, NULL);
   } else {
-    if (strcasecmp(radix_name, "hex") == 0) {
+    if ((strcasecmp(radix_name, "h") == 0) ||
+        (strcasecmp(radix_name, "hex") == 0) ||
+        (strcasecmp(radix_name, "hexadecimal") == 0)) {
       state.radix = 16;
-    } else if (strcasecmp(radix_name, "dec") == 0) {
+    } else if ((strcasecmp(radix_name, "d") == 0) ||
+               (strcasecmp(radix_name, "dec") == 0) ||
+               (strcasecmp(radix_name, "decimal") == 0)) {
       state.radix = 10;
-    } else if (strcasecmp(radix_name, "decimal") == 0) {
-      state.radix = 10;
-    } else if (strcasecmp(radix_name, "oct") == 0) {
-      state.radix = 8;
-    } else if (strcasecmp(radix_name, "octal") == 0) {
+    } else if ((strcasecmp(radix_name, "o") == 0) ||
+               (strcasecmp(radix_name, "oct") == 0) ||
+               (strcasecmp(radix_name, "octal") == 0)) {
       state.radix = 8;
     } else {
       state.radix = 16;
+
       if (state.pass == 0) {
-	fprintf(stderr,
-		"invalid radix \"%s\", will use hex.\n",
-		radix_name);
+        fprintf(stderr, "Error: Invalid radix \"%s\", will use hex.\n", radix_name);
       } else {
-        gpwarning(GPW_RADIX, NULL);
+        gpvwarning(GPW_RADIX, NULL);
       }
     }
   }
@@ -405,72 +510,90 @@ void select_radix(char *radix_name)
 /************************************************************************/
 
 /* Function to append a line to an ongoing macro definition */
-void macro_append(void)
+void
+macro_append(void)
 {
-  struct macro_body *body = malloc(sizeof(*body));
+  struct macro_body *body = GP_Malloc(sizeof(*body));
 
   body->src_line = NULL;
 
-  *state.mac_prev = body;	/* append this to the chain */
-  state.mac_prev = &body->next;	/* this is the new end of the chain */
+  *state.mac_prev = body;       /* append this to the chain */
+  state.mac_prev = &body->next; /* this is the new end of the chain */
   state.mac_body = body;
-  body->next = NULL;		/* make sure it's terminated */
+  body->next = NULL;            /* make sure it's terminated */
 }
 
-gpasmVal do_or_append_insn(char *op, struct pnode *parms)
+gpasmVal
+do_or_append_insn(char *op, struct pnode *parms)
 {
-  gpasmVal r;
-
-  if (!state.mac_prev ||
-      (strcasecmp(op, "endm") == 0) ||
-      (state.while_head && (strcasecmp(op, "endw") == 0))) {
-    r = do_insn(op, parms);
-  } else {
+  if (IN_MACRO_WHILE_DEFINITION) {
+    if (strcasecmp(op, "endm") == 0) {
+      return do_insn(op, parms);
+    }
+    else if (IN_WHILE_DEFINITION) {
+      if (strcasecmp(op, "while") == 0) {
+        assert(state.while_depth != 0);
+        ++state.while_depth;
+      }
+      else if ((state.while_head != NULL) && (strcasecmp(op, "endw") == 0)) {
+        assert(state.while_depth != 0);
+        if (--state.while_depth == 0) {
+          return do_insn(op, parms);
+        }
+      }
+    }
     macro_append();
-    r = 0;
+    return 0;
+  }
+  else {
+    return do_insn(op, parms);
   }
-
-  return r;
 }
 
-void print_pnode(struct pnode *p)
+void
+print_pnode(struct pnode *p)
 {
-  if(!p) {
+  if (p == NULL) {
     printf("Null\n");
     return;
   }
 
   switch(p->tag) {
-  case constant:
-    printf("  constant: %d\n",p->value.constant);
-    break;
-  case symbol:
-    printf("  symbol: %s\n",p->value.symbol);
-    break;
-  case unop:
-    printf("  unop: %d\n",p->value.unop.op);
+  case PTAG_CONSTANT:
+    printf("  constant: %d\n", p->value.constant);
     break;
 
-  case binop:
-    printf("  binop: %d\n",p->value.binop.op);
+  case PTAG_SYMBOL:
+    printf("  symbol: %s\n", p->value.symbol);
     break;
-  case string:
-    printf("  string: %s\n",p->value.string);
+
+  case PTAG_UNOP:
+    printf("  unop: %d\n", p->value.unop.op);
     break;
-  case list:
+
+  case PTAG_BINOP:
+    printf("  binop: %d\n", p->value.binop.op);
+    break;
+
+  case PTAG_STRING:
+    printf("  string: %s\n", p->value.string);
+    break;
+
+  case PTAG_LIST:
     printf("  list:\n");
     break;
-   
+
   default:
     printf("unknown type\n");
-
   }
 }
 
-void print_macro_node(struct macro_body *mac)
+void
+print_macro_node(struct macro_body *mac)
 {
-  if(mac->src_line)
-    printf(" src_line = %s\n",mac->src_line);
+  if (mac->src_line) {
+    printf(" src_line = %s\n", mac->src_line);
+  }
 }
 
 void print_macro_body(struct macro_body *mac)
@@ -491,88 +614,88 @@ void print_macro_body(struct macro_body *mac)
 /* add_file: add a file of type 'type' to the file_context stack.
  */
 
-struct file_context * add_file(unsigned int type, char *name)
+struct file_context *
+add_file(unsigned int type, const char *name)
 {
   static unsigned int file_id = 0;
   struct file_context *new;
 
-  /* First check to make sure this file is not already in the list */
+  /* First check to make sure this file is not already in the list. */
 
-  if(last) {
+  if (last != NULL) {
     new = last;
     do {
-      if(strcmp(new->name, name) == 0)
-	return(new);
+      if (strcmp(new->name, name) == 0) {
+        return new;
+      }
       new = new->prev;
     } while(new != NULL);
   }
 
-  new = malloc(sizeof(*new));
+  new = GP_Malloc(sizeof(*new));
 
-  new->name = strdup(name);
-  new->ft = type;
+  new->name = GP_Strdup(name);
+  new->ft   = type;
   new->prev = last;
-  new->id = file_id++;
+  new->id   = file_id++;
   new->next = NULL;
-  if(last)
+
+  if (last != NULL) {
     last->next = new;
+  }
 
   last = new;
   state.files = new;
-  return(new);
+  return new;
 }
 
 /* free_files: free memory allocated to the file_context stack
  */
 
-void free_files(void)
+void
+free_files(void)
 {
   struct file_context *old;
 
-  while(last != NULL) {
+  while (last != NULL) {
     old = last;
     last = old->prev;
     free(old->name);
     free(old);
-  } 
-
+  }
 }
 
-void hex_init(void)
+void
+hex_init(void)
 {
 
-  if (state.hexfile == suppress) {
+  if (state.hexfile == OUT_SUPPRESS) {
     /* Must delete hex file when suppressed. */
-    writehex(state.basefilename, 
-             state.i_memory, 
-             state.hex_format, 
+    writehex(state.basefilename,
+             state.i_memory,
+             state.hex_format,
              1,
-             0, 
-             state.dos_newlines);
+             state.dos_newlines,
+             1);
     return;
   }
 
   if (check_writehex(state.i_memory, state.hex_format)) {
-    gperror(GPE_IHEX,NULL); 
-  } else {
-    int byte_words;
-  
-    if (state.device.core_size > 0xff) {
-      byte_words = 0;
-    } else {
-      byte_words = 1;
-      if (state.hex_format != inhx8m) {
-        gpwarning(GPW_UNKNOWN,"Must use inhx8m format for EEPROM8");
-        state.hex_format = inhx8m;
-      }
-    }
-  
-    if (writehex(state.basefilename, state.i_memory, 
+    gpverror(GPE_IHEX, NULL);
+    writehex(state.basefilename, state.i_memory,
+             state.hex_format, 1,
+             state.dos_newlines, 1);
+  } else if (state.device.class != NULL) {
+    if (writehex(state.basefilename, state.i_memory,
                  state.hex_format, state.num.errors,
-                 byte_words, state.dos_newlines)) {
-      gperror(GPE_UNKNOWN,"Error generating hex file");
+                 state.dos_newlines,
+                 state.device.class->core_mask)) {
+      gperror(GPE_UNKNOWN, "Error generating hex file.");
     }
+  } else {
+    /* Won't have anything to write, just remove any old files */
+    writehex(state.basefilename, state.i_memory,
+             state.hex_format, 1,
+             state.dos_newlines, 1);
   }
-  
-  return;
 }
