@@ -54,16 +54,15 @@ object_append(gp_object_type *file, char *name)
   } else {
     gp_object_type *list = state.object;
 
-    while(list->next != NULL){
+    while (list->next != NULL) {
       list = list->next;
     }
     list->next = file;
 
     if (file->class != state.class) {
-      gp_error("processor family mismatch in \"%s\"", file->filename);
-    } else if ((0 != processor_mismatch_warning) &&
-               (file->processor != state.processor)) {
-      gp_warning("processor mismatch in \"%s\"", file->filename);
+      gp_error("Processor family mismatch in \"%s\".", file->filename);
+    } else if ((processor_mismatch_warning != 0) && (file->processor != state.processor)) {
+      gp_warning("Processor mismatch in \"%s\".", file->filename);
     }
   }
 
@@ -78,8 +77,8 @@ archive_append(gp_archive_type *file, char *name)
   struct archivelist *new;
 
   /* make the new entry */
-  new = (struct archivelist *)malloc(sizeof(*new));
-  new->name = strdup(name);
+  new = (struct archivelist *)GP_Malloc(sizeof(*new));
+  new->name = GP_Strdup(name);
   new->archive = file;
   new->next = NULL;
 
@@ -89,7 +88,7 @@ archive_append(gp_archive_type *file, char *name)
   } else {
     struct archivelist *list = state.archives;
 
-    while(list->next != NULL){
+    while (list->next != NULL){
       list = list->next;
     }
     list->next = new;
@@ -121,11 +120,12 @@ scan_index(struct symbol_table *table, gp_archive_type *archive)
       for (s = state.symbol.missing->hash_table[i]; s; s = s->next) {
         name = get_symbol_name(s);
         assert(name != NULL);
-        /* Search for missing symbol name in archive symbol table */
+        /* Search for missing symbol name in archive symbol table. */
         m = get_symbol(table, name);
+
         if (m != NULL) {
           /* Fetch the archive member, convert its binary data to an object
-             file, and add the object to the object list */
+             file, and add the object to the object list. */
           member = get_symbol_annotation(m);
           object_name = gp_archive_member_name(member);
           object = gp_convert_file(object_name, &member->data);
@@ -133,12 +133,12 @@ scan_index(struct symbol_table *table, gp_archive_type *archive)
           gp_link_add_symbols(state.symbol.definition,
                               state.symbol.missing,
                               object);
-          /* The symbol tables have been modified.  Need to take another
+          /* The symbol tables have been modified. Need to take another
              pass to make sure we get everything. */
           num_added++;
           modified = true;
           free(object_name);
-          /* This branch of the table has been modified. Go to the next one */
+          /* This branch of the table has been modified. Go to the next one. */
           break;
         }
       }
@@ -162,11 +162,11 @@ scan_archive(gp_archive_type *archive, char *name)
     archive_tbl = push_symbol_table(NULL, true);
     gp_archive_make_index(archive, archive_tbl);
     archive = gp_archive_add_index(archive_tbl, archive);
-    gp_warning("\"%s\" is missing symbol index", name);
+    gp_warning("\"%s\" is missing symbol index.", name);
     archive_tbl = pop_symbol_table(archive_tbl);
   }
 
-  /* Read the symbol index */
+  /* Read the symbol index. */
   gp_archive_read_index(state.symbol.archive, archive);
 
   /* Scan the symbol index for symbols in the missing symbol table.  If
@@ -178,7 +178,7 @@ scan_archive(gp_archive_type *archive, char *name)
   return modified;
 }
 
-/* Remove a symbol the linker created from the missing table */
+/* Remove a symbol the linker created from the missing table. */
 
 static void
 remove_linker_symbol(char *name)
@@ -194,7 +194,7 @@ remove_linker_symbol(char *name)
 /* Add a symbol the linker created to the symbol table. */
 
 static void
-add_linker_symbol(char *name)
+add_linker_symbol(const char *name)
 {
   gp_symbol_type *current = state.object->symbols;
   gp_symbol_type *found = NULL;
@@ -234,8 +234,8 @@ search_idata(void)
   }
 }
 
-/* Build the symbol tables.  Determine which objects from the archives are
-   required for linking */
+/* Build the symbol tables. Determine which objects from the archives are
+   required for linking. */
 
 void
 build_tables(void)
@@ -248,15 +248,13 @@ build_tables(void)
   const char *name;
   gp_coffsymbol_type *var;
 
-  /* Create the object file symbol tables */
+  /* Create the object file symbol tables. */
   while (list != NULL) {
-    gp_link_add_symbols(state.symbol.definition,
-                        state.symbol.missing,
-                        list);
+    gp_link_add_symbols(state.symbol.definition, state.symbol.missing, list);
     list = list->next;
   }
 
-  /* All of the objects have been scanned.  If there are remaining references
+  /* All of the objects have been scanned. If there are remaining references
      to symbols, then the archives must contain the missing references. */
   if (count_missing() && (state.archives != NULL)) {
     modified = false;
@@ -295,8 +293,8 @@ build_tables(void)
     remove_linker_symbol("_stack_end");
   }
 
-  /* All of the archives have been scanned.  If there are still missing
-     references, it is an error */
+  /* All of the archives have been scanned. If there are still missing
+     references, it is an error. */
   if (count_missing()) {
     for (i = 0; i < HASH_SIZE; i++) {
       for (s = state.symbol.missing->hash_table[i]; s; s = s->next) {
@@ -304,7 +302,7 @@ build_tables(void)
         assert(name != NULL);
         var = get_symbol_annotation(s);
         assert(var != NULL);
-        gp_error("missing definition for symbol \"%s\", required by \"%s\"",
+        gp_error("Missing definition for symbol \"%s\", required by \"%s\".",
                  name,
                  var->file->filename);
       }
@@ -313,9 +311,9 @@ build_tables(void)
   }
 }
 
-/* Read a coff object or archive.  gplink doesn't care about file extensions.
+/* Read a coff object or archive. gplink doesn't care about file extensions.
    This allows alternate extensions such as .a archives and .obj coff
-   objects.  */
+   objects. */
 
 void
 gplink_open_coff(const char *name)
@@ -329,13 +327,13 @@ gplink_open_coff(const char *name)
 
   coff = fopen(file_name, "rb");
   if ((coff == NULL) && (strchr(file_name, PATH_CHAR) == 0)) {
-    /* If no "/" in name, try searching include pathes */
+    /* If no "/" in name, try searching include pathes. */
     int i;
 
-    for(i = 0; i < state.numpaths; i++) {
-      snprintf(file_name, sizeof(file_name),
-               "%s" COPY_CHAR "%s", state.paths[i], name);
+    for (i = 0; i < state.numpaths; i++) {
+      snprintf(file_name, sizeof(file_name), "%s" COPY_CHAR "%s", state.paths[i], name);
       coff = fopen(file_name, "rb");
+
       if (coff != NULL) {
         break;
       }
@@ -347,26 +345,30 @@ gplink_open_coff(const char *name)
     exit(1);
   }
 
-  /* FIXME: Three files are opened, surely one is sufficent */
+  /* FIXME: Three files are opened, surely one is sufficent. */
 
   switch(gp_identify_coff_file(file_name)) {
-  case object_file_v2:
-  case object_file:
+  case GP_COFF_OBJECT_V2:
+  case GP_COFF_OBJECT:
     /* read the object */
     object = gp_read_coff(file_name);
     object_append(object, file_name);
     break;
-  case archive_file:
+
+  case GP_COFF_ARCHIVE:
     /* read the archive */
     archive = gp_archive_read(file_name);
     archive_append(archive, file_name);
     break;
-  case sys_err_file:
-    gp_error("can't open file \"%s\"", file_name);
+
+  case GP_COFF_SYS_ERR:
+    gp_error("Can't open file \"%s\".", file_name);
     break;
-  case unknown_file:
-    gp_error("\"%s\" is not a valid coff object or archive", file_name);
+
+  case GP_COFF_UNKNOWN:
+    gp_error("\"%s\" is not a valid coff object or archive.", file_name);
     break;
+
   default:
     assert(0);
   }
@@ -391,37 +393,39 @@ set_optimize_level(void)
   case 0:
     break;
   default:
-    gp_error("invalid optimization level");
+    gp_error("Invalid optimization level: %i", state.optimize.level);
   }
 }
 
 #define GET_OPTIONS "?a:cdf:hI:lmo:O:qrs:t:u:vw"
 
 enum {
-  OPT_MPLINK_COMPATIBLE = 0x100
+  OPT_MPLINK_COMPATIBLE = 0x100,
+  OPT_STRICT_OPTIONS
 };
 
 static struct option longopts[] =
 {
-  { "hex-format",          required_argument, NULL, 'a' },
-  { "object",              no_argument,       NULL, 'c' },
-  { "debug",               no_argument,       NULL, 'd' },
-  { "fill",                required_argument, NULL, 'f' },
-  { "help",                no_argument,       NULL, 'h' },
-  { "include",             required_argument, NULL, 'I' },
-  { "no-list",             no_argument,       NULL, 'l' },
-  { "map",                 no_argument,       NULL, 'm' },
-  { "output",              required_argument, NULL, 'o' },
-  { "optimize",            required_argument, NULL, 'O' },
-  { "quiet",               no_argument,       NULL, 'q' },
-  { "use-shared",          no_argument,       NULL, 'r' },
-  { "script",              required_argument, NULL, 's' },
-  { "stack",               required_argument, NULL, 't' },
-  { "macro",               required_argument, NULL, 'u' },
-  { "version",             no_argument,       NULL, 'v' },
-  { "processor-mismatch",  no_argument,       NULL, 'w' },
-  { "mplink-compatible",   no_argument,       NULL, OPT_MPLINK_COMPATIBLE },
-  { NULL,                  no_argument,       NULL, '\0'}
+  { "hex-format",         required_argument, NULL, 'a' },
+  { "object",             no_argument,       NULL, 'c' },
+  { "debug",              no_argument,       NULL, 'd' },
+  { "fill",               required_argument, NULL, 'f' },
+  { "help",               no_argument,       NULL, 'h' },
+  { "include",            required_argument, NULL, 'I' },
+  { "no-list",            no_argument,       NULL, 'l' },
+  { "map",                no_argument,       NULL, 'm' },
+  { "output",             required_argument, NULL, 'o' },
+  { "optimize",           required_argument, NULL, 'O' },
+  { "quiet",              no_argument,       NULL, 'q' },
+  { "use-shared",         no_argument,       NULL, 'r' },
+  { "script",             required_argument, NULL, 's' },
+  { "stack",              required_argument, NULL, 't' },
+  { "strict-options",     no_argument,       NULL, OPT_STRICT_OPTIONS },
+  { "macro",              required_argument, NULL, 'u' },
+  { "version",            no_argument,       NULL, 'v' },
+  { "processor-mismatch", no_argument,       NULL, 'w' },
+  { "mplink-compatible",  no_argument,       NULL, OPT_MPLINK_COMPATIBLE },
+  { NULL,                 no_argument,       NULL, '\0'}
 };
 
 void
@@ -431,15 +435,15 @@ init(void)
 
   /* initialize */
   gp_date_string(state.startdate, sizeof(state.startdate));
-  state.hex_format = inhx32;
+  state.hex_format = INHX32;
   state.numpaths = 0;
   state.processor = NULL;
   state.optimize.level = 1;
-  state.codfile = normal;
-  state.hexfile = normal;
-  state.lstfile = normal;
-  state.mapfile = suppress;
-  state.objfile = suppress;
+  state.codfile = OUT_NORMAL;
+  state.hexfile = OUT_NORMAL;
+  state.lstfile = OUT_NORMAL;
+  state.mapfile = OUT_SUPPRESS;
+  state.objfile = OUT_SUPPRESS;
   state.fill_enable = false;
   state.fill_value = 0;
   state.has_stack = false;
@@ -455,7 +459,7 @@ init(void)
   state.ifdef = NULL;
   state.script_symbols = push_symbol_table(NULL, false);
 
-  /* The symbols are case sensitive */
+  /* The symbols are case sensitive. */
   state.symbol.definition = push_symbol_table(NULL, false);
   state.symbol.missing = push_symbol_table(NULL, false);
   state.section.definition = push_symbol_table(NULL, false);
@@ -466,9 +470,9 @@ void
 gplink_add_path(const char *path)
 {
   if (state.numpaths < MAX_PATHS) {
-    state.paths[state.numpaths++] = strdup(path);
+    state.paths[state.numpaths++] = GP_Strdup(path);
   } else {
-    gp_error("too many -I paths");
+    gp_error("Too many -I paths.");
   }
 }
 
@@ -490,7 +494,7 @@ parse_define(const char *optarg, void (*func)(const char* name, long value))
 void
 show_usage(void)
 {
-  printf("Usage: gplink [options] [objects] [libraries] \n");
+  printf("Usage: gplink [options] [objects] [libraries]\n");
   printf("Options: [defaults in brackets after descriptions]\n");
   printf("  -a FMT, --hex-format FMT       Select hex file format.\n");
   printf("  -c, --object                   Output executable object file.\n");
@@ -500,24 +504,27 @@ show_usage(void)
   printf("  -I DIR, --include DIR          Specify include directory.\n");
   printf("  -l, --no-list                  Disable list file output.\n");
   printf("  -m, --map                      Output a map file.\n");
-  printf("      --mplink-compatible         MPLINK compatibility mode\n");
+  printf("      --mplink-compatible        MPLINK compatibility mode.\n");
   printf("  -o FILE, --output FILE         Alternate name of output file.\n");
   printf("  -O OPT, --optimize OPT         Optimization level [1].\n");
   printf("  -q, --quiet                    Quiet.\n");
   printf("  -r, --use-shared               Use shared memory if necessary.\n");
   printf("  -s FILE, --script FILE         Linker script.\n");
   printf("  -t SIZE, --stack SIZE          Create a stack section.\n");
+  printf("      --strict-options           If this is set, then an option may not be parameter\n"
+         "                                 of an another option. For example: -s --quiet\n");
   printf("  -u, --macro symbol[=value]     Add macro value for script.\n");
   printf("  -v, --version                  Show version.\n");
   printf("  -w, --processor-mismatch       Disable \"processor mismatch\" warning.\n");
   printf("\n");
 #ifdef USE_DEFAULT_PATHS
-  if (gp_lkr_path) {
+  if (gp_lkr_path != NULL) {
     printf("Default linker script path %s\n", gp_lkr_path);
   } else {
     printf("Default linker script path NOT SET\n");
   }
-  if (gp_lib_path) {
+
+  if (gp_lib_path != NULL) {
     printf("Default library path %s\n", gp_lib_path);
   } else {
     printf("Default library path NOT SET\n");
@@ -532,16 +539,38 @@ show_usage(void)
 void
 process_args( int argc, char *argv[])
 {
-  extern char *optarg;
-  extern int optind;
+  int option_index;
+  const char *command;
   int c;
-  gp_boolean usage = false;
+  gp_boolean strict_options = false;
+  gp_boolean usage          = false;
   char *pc;
 
-  /* first pass through options */
-  while ((c = getopt_long(argc, argv, GET_OPTIONS, longopts, 0)) != EOF) {
-    switch (c) {
-    case 'O':
+  /* Scan through the options for the --strict-options flag. */
+  while ((c = getopt_long(argc, argv, GET_OPTIONS, longopts, NULL)) != EOF) {
+    if (c == OPT_STRICT_OPTIONS) {
+      strict_options = true;
+      break;
+    }
+  }
+
+  /* Restores the getopt_long index. */
+  optind = 1;
+
+  /* second pass through options */
+  while (true) {
+    /* This is necessary for the gp_exit_is_excluded_arg() function. */
+    option_index = -1;
+    command = argv[optind];
+    if ((c = getopt_long(argc, argv, GET_OPTIONS, longopts, &option_index)) == EOF) {
+      break;
+    }
+
+    if (strict_options) {
+      gp_exit_if_arg_an_option(longopts, ARRAY_SIZE(longopts), option_index, optarg, c, command);
+    }
+
+    if (c == 'O') {
       state.optimize.level = atoi(optarg);
       break;
     }
@@ -552,24 +581,24 @@ process_args( int argc, char *argv[])
 
   set_optimize_level();
 
-  /* second pass through options */
-  while ((c = getopt_long(argc, argv, GET_OPTIONS, longopts, 0)) != EOF) {
+  /* third pass through options */
+  while ((c = getopt_long(argc, argv, GET_OPTIONS, longopts, NULL)) != EOF) {
     switch (c) {
     case 'a':
       if (strcasecmp(optarg, "inhx8m") == 0) {
-        state.hex_format = inhx8m;
+        state.hex_format = INHX8M;
       } else if (strcasecmp(optarg, "inhx16") == 0) {
-        state.hex_format = inhx16;
+        state.hex_format = INHX16;
       } else if (strcasecmp(optarg, "inhx32") == 0) {
-        state.hex_format = inhx32;
+        state.hex_format = INHX32;
       } else {
-        gp_error("invalid hex format \"%s\", expected inhx8m, inhx16, or inhx32",
+        gp_error("Invalid hex format \"%s\", expected inhx8m, inhx16, or inhx32.",
                  optarg);
       }
       break;
 
     case 'c':
-      state.objfile = normal;
+      state.objfile = OUT_NORMAL;
       break;
 
     case 'd':
@@ -580,9 +609,9 @@ process_args( int argc, char *argv[])
     case 'f':
       state.fill_value = strtol(optarg, &pc, 16);
       if ((pc == NULL) || (*pc != '\0')) {
-        gp_error("invalid character %#x in number constant", *pc);
+        gp_error("Invalid character %#x in number constant.", *pc);
       } else if (state.fill_value > 0xffff) {
-        gp_error("fill value exceeds 0xffff", *pc);
+        gp_error("Fill value exceeds 0xffff: %#x", *pc);
       } else {
         state.fill_enable = true;
       }
@@ -598,18 +627,20 @@ process_args( int argc, char *argv[])
       break;
 
     case 'l':
-      state.lstfile = suppress;
+      state.lstfile = OUT_SUPPRESS;
       break;
 
     case 'm':
-      state.mapfile = normal;
+      state.mapfile = OUT_NORMAL;
       break;
 
     case 'o':
       strncpy(state.basefilename, optarg, sizeof(state.basefilename));
       pc = strrchr(state.basefilename, '.');
-      if (pc)
-        *pc = 0;
+
+      if (pc != NULL) {
+        *pc = '\0';
+      }
       break;
 
     case 'O':
@@ -628,12 +659,13 @@ process_args( int argc, char *argv[])
       {
         struct srcfns *fn;
 
-        fn = malloc(sizeof (struct srcfns));
-        fn->filename = strdup(optarg);
+        fn = GP_Malloc(sizeof(struct srcfns));
+        fn->filename = GP_Strdup(optarg);
         fn->next = NULL;
 
-        if (NULL == state.srcfilenames)
+        if (state.srcfilenames == NULL) {
           state.srcfilenames = fn;
+        }
         else {
           struct srcfns *p;
 
@@ -647,7 +679,7 @@ process_args( int argc, char *argv[])
     case 't':
       state.stack_size = strtol(optarg, &pc, 10);
       if ((pc == NULL) || (*pc != '\0')) {
-        gp_error("invalid character %#x in number constant", *pc);
+        gp_error("Invalid character %#x in number constant.", *pc);
       } else {
         state.has_stack = true;
       }
@@ -669,37 +701,44 @@ process_args( int argc, char *argv[])
     case OPT_MPLINK_COMPATIBLE:
       state.mplink_compatible = true;
       break;
-    }
-    if (usage)
+
+    case OPT_STRICT_OPTIONS:
+      /* do nothing */
       break;
+    }
+
+    if (usage) {
+      break;
+    }
   }
 
   /* check if the first file is the linker script */
-  if (argv[optind]) {
+  if (argv[optind] != NULL) {
     pc = strrchr(argv[optind], '.');
-    if (pc && strcasecmp(pc, ".lkr") == 0) {
+
+    if ((pc != NULL) && (strcasecmp(pc, ".lkr") == 0)) {
       struct srcfns *fn;
 
-      fn = malloc(sizeof (struct srcfns));
-      fn->filename = strdup(argv[optind++]);
+      fn = GP_Malloc(sizeof(struct srcfns));
+      fn->filename = GP_Strdup(argv[optind++]);
       fn->next = NULL;
 
-      if (NULL == state.srcfilenames)
+      if (state.srcfilenames == NULL) {
         state.srcfilenames = fn;
+      }
       else {
         struct srcfns *p;
 
-        for (p = state.srcfilenames; p->next; p = p->next)
+        for (p = state.srcfilenames; p->next != NULL; p = p->next)
           ;
         p->next = fn;
       }
     }
   }
 
-  if ((state.srcfilenames == NULL) &&
-      (optind >= argc)) {
+  if ((state.srcfilenames == NULL) && (optind >= argc)) {
     /* No linker script was specified and no object filenames were provided,
-       so print the usage */
+       so print the usage. */
     usage = true;
   }
 
@@ -708,12 +747,12 @@ process_args( int argc, char *argv[])
   }
 
   /* Add the library path to the include paths list last, so that the user
-     specified directories are searched first */
-  if (gp_lib_path) {
+     specified directories are searched first. */
+  if (gp_lib_path != NULL) {
     gplink_add_path(gp_lib_path);
   }
 
-  if (gp_lkr_path) {
+  if (gp_lkr_path != NULL) {
     gplink_add_path(gp_lkr_path);
   }
 
@@ -736,17 +775,18 @@ linker(void)
   snprintf(state.objfilename, sizeof(state.objfilename),
            "%s.cof", state.basefilename);
 
-  /* Read the script */
-  if (state.srcfilenames) {
+  /* Read the script. */
+  if (state.srcfilenames != NULL) {
     struct srcfns *p = state.srcfilenames;
 
     do {
-      open_src(p->filename, 0);
+      open_src(p->filename, false);
       yyparse();
       p = p->next;
     } while (NULL != p);
 #ifdef USE_DEFAULT_PATHS
-  } else if ((state.object) && (gp_lkr_path)) {
+  }
+  else if ((state.object != NULL) && (gp_lkr_path != NULL)) {
     /* The processor is known because an object was on the command line. So
        use one of the default scripts that are distributed with gputils. */
     char file_name[BUFSIZ];
@@ -754,31 +794,35 @@ linker(void)
 
     assert(state.processor);
     script_name = gp_processor_script(state.processor);
+
     if (script_name == NULL) {
-      gp_error("linker script not specified and can't determine default script");
+      gp_error("Linker script not specified and can't determine default script.");
       return EXIT_FAILURE;
     }
+
     snprintf(file_name, sizeof(file_name),
              "%s" COPY_CHAR "%s", gp_lkr_path, script_name);
-    gp_message("using default linker script \"%s\"", file_name);
-    open_src(file_name, 0);
+    gp_message("Using default linker script \"%s\".", file_name);
+    open_src(file_name, false);
     yyparse();
 #endif
-  } else {
+  }
+  else {
     /* The user must supply the linker script name.  The processor isn't
        commanded so the linker has no way to pick. */
-    gp_error("linker script not specified");
+    gp_error("Linker script not specified.");
     return EXIT_FAILURE;
   }
 
   if (state.object == NULL) {
-    gp_error("missing input object file");
+    gp_error("Missing input object file.");
     return EXIT_FAILURE;
   }
 
-  /* An error occured while reading the input files, no need to continue */
-  if (gp_num_errors)
+  /* An error occured while reading the input files, no need to continue. */
+  if (gp_num_errors > 0) {
     return EXIT_FAILURE;
+  }
 
   /* Construct the symbol tables. Determine which archive members are
      required to resolve external references. */
@@ -820,14 +864,14 @@ linker(void)
   program = i_memory_create();
 
   /* allocate memory for absolute sections */
-  gp_debug("verifying absolute sections.");
-  gp_cofflink_reloc_abs(program,
+  gp_debug("Verifying absolute sections.");
+  gp_cofflink_reloc_abs(state.object,
+                        program,
                         state.class->org_to_byte_shift,
-                        state.object->sections,
                         STYP_TEXT | STYP_DATA_ROM);
-  gp_cofflink_reloc_abs(data,
+  gp_cofflink_reloc_abs(state.object,
+                        data,
                         0,
-                        state.object->sections,
                         STYP_DATA | STYP_BSS | STYP_SHARED |
                         STYP_OVERLAY | STYP_ACCESS);
 
@@ -838,8 +882,9 @@ linker(void)
     cinit_section = gp_coffgen_findsection(state.object,
                        state.object->sections,
                        ".cinit");
-    if (cinit_section) {
-      gp_cofflink_reloc_cinit(program,
+    if (cinit_section != NULL) {
+      gp_cofflink_reloc_cinit(state.object,
+                              program,
                               state.class->org_to_byte_shift,
                               cinit_section,
                               state.section.definition);
@@ -849,16 +894,16 @@ linker(void)
   /* FIXME: allocate assigned stacks */
 
   /* allocate memory for relocatable assigned sections */
-  gp_debug("relocating assigned sections.");
-  gp_cofflink_reloc_assigned(program,
+  gp_debug("Relocating assigned sections.");
+  gp_cofflink_reloc_assigned(state.object,
+                             program,
                              state.class->org_to_byte_shift,
-                             state.object->sections,
                              STYP_TEXT | STYP_DATA_ROM,
                              state.section.definition,
                              state.section.logical);
-  gp_cofflink_reloc_assigned(data,
+  gp_cofflink_reloc_assigned(state.object,
+                             data,
                              0,
-                             state.object->sections,
                              STYP_DATA | STYP_BSS | STYP_SHARED |
                              STYP_OVERLAY | STYP_ACCESS,
                              state.section.definition,
@@ -867,15 +912,15 @@ linker(void)
   /* FIXME: allocate unassigned stacks */
 
   /* allocate memory for relocatable unassigned sections */
-  gp_debug("relocating unassigned sections.");
-  gp_cofflink_reloc_unassigned(program,
+  gp_debug("Relocating unassigned sections.");
+  gp_cofflink_reloc_unassigned(state.object,
+                               program,
                                state.class->org_to_byte_shift,
-                               state.object->sections,
                                STYP_TEXT | STYP_DATA_ROM,
                                state.section.definition);
-  gp_cofflink_reloc_unassigned(data,
+  gp_cofflink_reloc_unassigned(state.object,
+                               data,
                                0,
-                               state.object->sections,
                                STYP_DATA | STYP_BSS | STYP_SHARED |
                                STYP_OVERLAY | STYP_ACCESS,
                                state.section.definition);
@@ -885,9 +930,7 @@ linker(void)
 
   gp_cofflink_update_table(state.object, state.class->org_to_byte_shift);
 
-  gp_cofflink_fill_pages(state.object,
-                         program,
-                         state.section.definition);
+  gp_cofflink_fill_pages(state.object, program, state.section.definition);
 
   i_memory_free(data);
   i_memory_free(program);
@@ -896,10 +939,10 @@ linker(void)
   gp_cofflink_patch(state.object);
 
   /* modify the executable object data */
-  state.object->filename = strdup(state.objfilename);
+  state.object->filename = GP_Strdup(state.objfilename);
   state.object->flags |= F_EXEC;
 
-  if (state.objfile == normal) {
+  if (state.objfile == OUT_NORMAL) {
     /* write the executable object in memory */
     gp_write_coff(state.object, gp_num_errors);
   } else {
@@ -915,7 +958,7 @@ linker(void)
            state.hex_format,
            gp_num_errors,
            0,
-           state.class->core_size);
+           state.class->core_mask);
 
   /* convert the executable object into a cod file and list file */
   cod_init();
@@ -927,7 +970,7 @@ linker(void)
 
   i_memory_free(state.i_memory);
 
-  return gp_num_errors <= 0;
+  return (gp_num_errors <= 0);
 }
 
 int
@@ -936,5 +979,5 @@ main(int argc, char *argv[])
   init();
   process_args(argc, argv);
 
-  return linker() ? EXIT_SUCCESS : EXIT_FAILURE;
+  return (linker() ? EXIT_SUCCESS : EXIT_FAILURE);
 }
