@@ -1,14 +1,13 @@
-/* A Bison parser, made by GNU Bison 2.3.  */
+/* A Bison parser, made by GNU Bison 3.0.2.  */
 
-/* Skeleton interface for Bison's Yacc-like parsers in C
+/* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2013 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,9 +15,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -33,46 +30,68 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-/* Tokens.  */
+#ifndef YY_YY_PARSE_H_INCLUDED
+# define YY_YY_PARSE_H_INCLUDED
+/* Debug traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 1
+#endif
+#if YYDEBUG
+extern int yydebug;
+#endif
+
+/* Token type.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
-   /* Put the tokens into the symbol table, so that GDB and other debuggers
-      know about them.  */
-   enum yytokentype {
-     SYMBOL = 258,
-     LIBPATH = 259,
-     LKRPATH = 260,
-     PATH = 261,
-     LEXEOF = 262,
-     NUMBER = 263
-   };
+  enum yytokentype
+  {
+    LEXEOF = 0,
+    SYMBOL = 258,
+    LIBPATH = 259,
+    LKRPATH = 260,
+    STRING = 261,
+    ERROR = 262,
+    NUMBER = 263,
+    DEFINE = 264,
+    IFDEF = 265,
+    ELSE = 266,
+    FI = 267
+  };
 #endif
 /* Tokens.  */
+#define LEXEOF 0
 #define SYMBOL 258
 #define LIBPATH 259
 #define LKRPATH 260
-#define PATH 261
-#define LEXEOF 262
+#define STRING 261
+#define ERROR 262
 #define NUMBER 263
+#define DEFINE 264
+#define IFDEF 265
+#define ELSE 266
+#define FI 267
 
-
-
-
+/* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef union YYSTYPE
-#line 103 "parse.y"
+typedef union YYSTYPE YYSTYPE;
+union YYSTYPE
 {
+#line 111 "parse.y" /* yacc.c:1909  */
+
   int i;
+  long l;
   char *s;
   struct pnode *p;
-}
-/* Line 1489 of yacc.c.  */
-#line 71 "parse.h"
-	YYSTYPE;
-# define yystype YYSTYPE /* obsolescent; will be withdrawn */
-# define YYSTYPE_IS_DECLARED 1
+
+#line 87 "parse.h" /* yacc.c:1909  */
+};
 # define YYSTYPE_IS_TRIVIAL 1
+# define YYSTYPE_IS_DECLARED 1
 #endif
 
+
 extern YYSTYPE yylval;
 
+int yyparse (void);
+
+#endif /* !YY_YY_PARSE_H_INCLUDED  */
