@@ -2,7 +2,7 @@
    Copyright (C) 2001, 2002, 2003, 2004, 2005
    Craig Franklin
 
-    Copyright (C) 2014 Molnar Karoly <molnarkaroly@users.sf.net>
+    Copyright (C) 2014-2015 Molnar Karoly <molnarkaroly@users.sf.net>
 
 This file is part of gputils.
 
@@ -30,14 +30,18 @@ Boston, MA 02111-1307, USA.  */
 #include "parse.h"
 #include "scan.h"
 
+#ifdef STDC_HEADERS
+#include <stdarg.h>
+#endif
+
 #define USER_LABEL_ALIGN        15
 
 extern int yyparse(void);
 
 struct gpdasm_state state = {
-  NULL,                 /* processor type */
-  PROC_CLASS_GENERIC,   /* 12 bit device */
-  1                     /* output format */
+  .processor = NULL,                /* processor type */
+  .class     = PROC_CLASS_GENERIC,  /* 12 bit device */
+  .format    = 1,                   /* output format */
 };
 
 static const char *processor_name = NULL;
@@ -54,6 +58,178 @@ static gp_boolean prev_empty_line = false;
 
 static char border[] = "===============================================================================";
 
+static char out_buffer[BUFSIZ];
+
+enum {
+  OPT_STRICT = 0x100,
+  OPT_STRICT_OPTIONS
+};
+
+//#define GET_OPTIONS "?chijk:lmnop:stvy"
+#define GET_OPTIONS "chijk:lmnop:stvy"
+
+static struct option longopts[] =
+{
+  { "mnemonics",      no_argument,       NULL, 'c' },
+  { "help",           no_argument,       NULL, 'h' },
+  { "hex-info",       no_argument,       NULL, 'i' },
+  { "mov-fsrn",       no_argument,       NULL, 'j' },
+  { "label-list",     required_argument, NULL, 'k' },
+  { "list-chips",     no_argument,       NULL, 'l' },
+  { "dump",           no_argument,       NULL, 'm' },
+  { "show-names",     no_argument,       NULL, 'n' },
+  { "show-config",    no_argument,       NULL, 'o' },
+  { "processor",      required_argument, NULL, 'p' },
+  { "short",          no_argument,       NULL, 's' },
+  { "strict-options", no_argument,       NULL, OPT_STRICT_OPTIONS },
+  { "use-tab",        no_argument,       NULL, 't' },
+  { "version",        no_argument,       NULL, 'v' },
+  { "extended",       no_argument,       NULL, 'y' },
+  { "strict",         no_argument,       NULL, OPT_STRICT },
+  { NULL,             no_argument,       NULL, 0   }
+};
+
+/*------------------------------------------------------------------------------------------------*/
+
+static size_t
+unexpand(char *Dst, size_t Size, const char *Src)
+{
+  size_t in_read_idx;
+  size_t in_read_virt_idx;
+  size_t out_write_idx;
+  size_t first_tab;
+  size_t last_char;
+  size_t space_num;
+
+  if (Size <= 1) {
+    return 0;
+  }
+
+  --Size;
+
+  in_read_idx      = 0;
+  in_read_virt_idx = 0;
+  out_write_idx    = 0;
+  first_tab        = 0;
+  last_char        = 0;
+  space_num        = 0;
+
+  while ((Src[in_read_idx] != '\0') && (out_write_idx < Size)) {
+    if (Src[in_read_idx] == ' ') {
+      ++space_num;
+    }
+    else if (Src[in_read_idx] == '\n') {
+      /* This a newline character. */
+      Dst[out_write_idx++] = Src[in_read_idx];
+      in_read_virt_idx = 0;
+      last_char        = 0;
+      space_num        = 0;
+      ++in_read_idx;
+      continue;
+    }
+    else {
+      /* This a not-space character. */
+      if (space_num > 1) {
+        first_tab = ((last_char + TABULATOR_SIZE) / TABULATOR_SIZE) * TABULATOR_SIZE;
+
+        if (first_tab > in_read_virt_idx) {
+          first_tab = 0;
+        }
+
+        if ((last_char + 1) < first_tab) {
+          if (first_tab <= in_read_virt_idx) {
+            Dst[out_write_idx++] = '\t';
+
+            if (out_write_idx >= Size) {
+              goto _exit;
+            }
+
+            space_num = in_read_virt_idx - first_tab;
+          }
+
+          while (space_num >= TABULATOR_SIZE) {
+            Dst[out_write_idx++] = '\t';
+
+            if (out_write_idx >= Size) {
+              goto _exit;
+            }
+
+            space_num -= TABULATOR_SIZE;
+          }
+        }
+      }
+
+      while (space_num > 0) {
+        Dst[out_write_idx++] = ' ';
+
+        if (out_write_idx >= Size) {
+          goto _exit;
+        }
+
+        --space_num;
+      }
+
+      Dst[out_write_idx++] = Src[in_read_idx];
+      last_char = in_read_idx;
+    }
+
+    ++in_read_idx;
+    ++in_read_virt_idx;
+  } /* while ((Src[in_read_idx] != '\0') && */
+
+  while (space_num >= TABULATOR_SIZE) {
+    Dst[out_write_idx++] = '\t';
+    if (out_write_idx >= Size) {
+      goto _exit;
+    }
+
+    space_num -= TABULATOR_SIZE;
+  }
+
+_exit:
+
+  Dst[out_write_idx] = '\0';
+  return out_write_idx;
+}
+
+/*------------------------------------------------------------------------------------------------*/
+
+static void
+ux_print(gp_boolean newline, const char *format, ...)
+{
+  va_list(ap);
+  char buffer[BUFSIZ];
+  const char *bptr;
+  size_t len;
+
+  va_start(ap, format);
+  len = (size_t)vsnprintf(buffer, sizeof(buffer), format, ap);
+  va_end(ap);
+
+  if ((int)len < 0) {
+    return;
+  }
+
+  bptr = buffer;
+
+  if (newline) {
+    if (state.use_tab) {
+      unexpand(out_buffer, sizeof(out_buffer), buffer);
+      bptr = out_buffer;
+    }
+
+    puts(bptr);
+  }
+  else {
+    if (state.use_tab) {
+      len = unexpand(out_buffer, sizeof(out_buffer), buffer);
+      bptr = out_buffer;
+    }
+
+    printf("%s", bptr);
+  }
+}
+
 /*------------------------------------------------------------------------------------------------*/
 
 static void
@@ -62,7 +238,7 @@ select_processor(void)
   pic_processor_t found = NULL;
 
   if (processor_name == NULL) {
-    fprintf(stderr, "error: Must select the processor.\n");
+    fprintf(stderr, "Error: Must select the processor.\n");
     exit(1);
   }
 
@@ -71,16 +247,16 @@ select_processor(void)
   if (found != NULL) {
     state.processor = found;
   } else {
-    fprintf(stderr, "error: Didn't find any processor named: %s\n", processor_name);
+    fprintf(stderr, "Error: Didn't find any processor named: %s\n", processor_name);
     printf("Here are the supported processors:\n");
-    gp_dump_processor_list(true, NULL, NULL);
+    gp_dump_processor_list(true, PROC_CLASS_UNKNOWN, PROC_CLASS_UNKNOWN, PROC_CLASS_UNKNOWN);
     exit(1);
   }
 
   state.class = gp_processor_class(state.processor);
 
   if (state.class->instructions == NULL) {
-    fprintf(stderr, "error: Unsupported processor class.\n");
+    fprintf(stderr, "Error: Unsupported processor class.\n");
     exit(1);
   }
 
@@ -94,11 +270,12 @@ static void
 write_header(void)
 {
   if (!state.format) {
-    printf("\n");
-    printf("        processor %s\n        radix dec\n", state.processor->names[1]);
+    ux_print(true, "        processor %s\n"
+                   "        radix dec", state.processor->names[1]);
 
     if (state.processor->header != NULL) {
-      printf("        include %s\n", state.processor->header);
+      ux_print(true, "\n"
+                     "        include %s", state.processor->header);
     }
   }
 }
@@ -116,25 +293,29 @@ write_core_sfr_list(void)
     return;
   }
 
+  ux_print(true, "\n"
+                 "F       equ     1\n"
+                 "W       equ     0");
+
   if (state.class == PROC_CLASS_PIC16E) {
-    printf("\nF       equ     1\nW       equ     0\nA       equ     0\n\n");
-  }
-  else {
-    printf("\nF       equ     1\nW       equ     0\n\n");
+    ux_print(true, "A       equ     0\n"
+                   "B       equ     1");
   }
 
+  ux_print(true, "");
+
   table = state.class->core_sfr_table;
   for (i = state.class->core_sfr_number; i > 0; ++table, --i) {
-    if (state.class == PROC_CLASS_PIC14E) {
+    if ((state.class == PROC_CLASS_PIC14E) || (state.class == PROC_CLASS_PIC14EX)) {
       if (strcmp(table->name, "FSR0L") == 0) {
-        printf("FSR0    equ     0x%03x\n", table->address);
+        ux_print(true, "FSR0    equ     0x%03x", table->address);
       }
       else if (strcmp(table->name, "FSR1L") == 0) {
-        printf("FSR1    equ     0x%03x\n", table->address);
+        ux_print(true, "FSR1    equ     0x%03x", table->address);
       }
     }
 
-    printf("%s\tequ     0x%03x\n", table->name, table->address);
+    ux_print(true, "%-7s equ     0x%03x", table->name, table->address);
   }
 }
 
@@ -144,7 +325,8 @@ static void
 end_asm(void)
 {
   if (!state.format) {
-    printf("\n        end\n");
+    ux_print(true, "\n"
+                   "        end");
   }
 }
 
@@ -157,10 +339,10 @@ write_org(int org, int addr_digits, const char *title, const char *Address_name,
   char buffer[BUFSIZ];
 
   if (!state.format) {
-    printf("\n");
+    ux_print(true, "");
 
     if (title != NULL) {
-      printf("        ; %s\n", title);
+      ux_print(true, "        ; %s", title);
     }
 
     if (Address_name != NULL) {
@@ -173,13 +355,13 @@ write_org(int org, int addr_digits, const char *title, const char *Address_name,
       }
 
       gp_exclamation(buffer, sizeof(buffer), length, "; address: 0x%0*x", addr_digits, org);
-      printf("\n%s\n\n", buffer);
-      prev_empty_line = true;
     }
     else {
-      printf("        org     0x%0*x\n\n", addr_digits, org);
-      prev_empty_line = true;
+      snprintf(buffer, sizeof(buffer), "        org     0x%0*x", addr_digits, org);
     }
+
+    ux_print(true, "\n%s\n", buffer);
+    prev_empty_line = true;
   }
 }
 
@@ -251,7 +433,7 @@ recognize_labels_and_spec_words(MemBlock *memory)
   if (state.show_config) {
     dev = gp_cfg_find_pic_multi_name(state.processor->names, ARRAY_SIZE(state.processor->names));
     if (dev == NULL) {
-      fprintf(stderr, "warning: The %s processor has no entries in the config db.",
+      fprintf(stderr, "Warning: The %s processor has no entries in the config db.",
               state.processor->names[2]);
     }
   }
@@ -294,6 +476,15 @@ recognize_labels_and_spec_words(MemBlock *memory)
         }
         else {
           if (state.class->i_memory_get(m, i, &data, NULL, NULL) == W_USED_ALL) {
+            sym = lset_symbol_find_addr(state.lset_root.sections[SECT_SPEC_CODE], org, -1, true);
+
+            if ((sym != NULL) && !(sym->attr & CSYM_ORG)) {
+              if (sym->start == (long)org) {
+                b_memory_set_addr_name(m, i, sym->name);
+                sym->attr |= CSYM_USED;
+              }
+            }
+
             idlocs_pack.words[index] = data;
             ++idlocs_pack.number;
           }
@@ -335,7 +526,7 @@ recognize_labels_and_spec_words(MemBlock *memory)
             }
           }
           else {
-            fprintf(stderr, "warning: The value of GP_CFG_ADDR_PACK_MAX too little: %u",
+            fprintf(stderr, "Warning: The value of GP_CFG_ADDR_PACK_MAX too little: %u",
                     GP_CFG_ADDR_PACK_MAX);
           }
         } /* if (dev != NULL) */
@@ -552,13 +743,13 @@ show_config(void)
     return;
   }
 
-  printf("\n");
+  ux_print(true, "");
   for (m = 0; m < addr_pack.hit_count; ++m) {
     hit = &addr_pack.hits[m];
 
     for (n = 0; n < hit->pair_count; ++n) {
-      printf("        CONFIG  %-*s = %s\n", addr_pack.max_dir_width,
-             hit->pairs[n].directive->name, hit->pairs[n].option->name);
+      ux_print(true, "        CONFIG  %-*s = %s", addr_pack.max_dir_width,
+               hit->pairs[n].directive->name, hit->pairs[n].option->name);
     }
   }
 }
@@ -612,18 +803,19 @@ show_idlocs(void)
         }
 
         buffer[j] = '\0';
-        printf("\n        IDLOCS  \"%s\"\n", buffer);
+        ux_print(true, "\n"
+                       "        IDLOCS  \"%s\"", buffer);
       }
       else {
         /* Not only printable characters are there in it. */
-        printf("\n");
+        ux_print(true, "");
         for (i = 0; i < PIC16E_IDLOCS_SIZE; ++i) {
           if ((word = idlocs_pack.words[i]) >= 0) {
             if (isalnum(word)) {
-              printf("        __idlocs _IDLOC%u, '%c'\n", i, word);
+              ux_print(true, "        __idlocs _IDLOC%u, '%c'", i, word);
             }
             else {
-              printf("        __idlocs _IDLOC%u, 0x%02X\n", i, word);
+              ux_print(true, "        __idlocs _IDLOC%u, 0x%02X", i, word);
             }
           }
         }
@@ -633,16 +825,16 @@ show_idlocs(void)
       /* There are several separate words or characters, or do not start at the beginning. */
       prev_exist = false;
       act_exist  = false;
-      printf("\n");
+      ux_print(true, "");
       for (i = 0; i < PIC16E_IDLOCS_SIZE; ++i) {
         if ((word = idlocs_pack.words[i]) >= 0) {
           act_exist = true;
 
           if (isalnum(word)) {
-            printf("        __idlocs _IDLOC%u, '%c'\n", i, word);
+            ux_print(true, "        __idlocs _IDLOC%u, '%c'", i, word);
           }
           else {
-            printf("        __idlocs _IDLOC%u, 0x%02X\n", i, word);
+            ux_print(true, "        __idlocs _IDLOC%u, 0x%02X", i, word);
           }
         }
         else {
@@ -651,7 +843,7 @@ show_idlocs(void)
 
       if ((prev_exist == true) && (act_exist == false)) {
         /* Indicates the break. */
-        printf("\n");
+        ux_print(true, "");
       }
 
       prev_exist = act_exist;
@@ -667,7 +859,8 @@ show_idlocs(void)
     word |= (idlocs_pack.words[1] & 0x000F) << 8;
     word |= (idlocs_pack.words[2] & 0x000F) << 4;
     word |= (idlocs_pack.words[3] & 0x000F);
-    printf("\n        __idlocs 0x%04X\n", word);
+    ux_print(true, "\n"
+                   "        __idlocs 0x%04X", word);
   }
 }
 
@@ -713,7 +906,7 @@ list_user_labels(int Addr_digits)
                            sym->end - sym->start + 1);
           }
 
-          printf("%s\n", buffer);
+          ux_print(true, "%s", buffer);
         }
       }
     }
@@ -722,7 +915,9 @@ list_user_labels(int Addr_digits)
   if (((sect = state.lset_root.sections[SECT_SPEC_EEDATA]) != NULL) &&
       (sect->symbol_table != NULL)) {
     if (need_section_print(sect)) {
-      printf("\n;%s\n; EEDATA address definitions\n\n", border);
+      ux_print(true, "\n"
+                     ";%s\n"
+                     "; EEDATA address definitions\n", border);
       for (i = 0; i < sect->symbol_number; ++i) {
         sym = sect->symbol_table[i];
 
@@ -735,7 +930,7 @@ list_user_labels(int Addr_digits)
                            sym->end - sym->start + 1);
           }
 
-          printf("%s\n", buffer);
+          ux_print(true, "%s", buffer);
         }
       }
     }
@@ -751,10 +946,9 @@ add_constant(lset_section_t *Section, const char *Name, long Start, unsigned int
     return;
   }
 
-  lset_symbol_new(Section, Name, Start, -1, CSYM_START | Attr, 1);
+  lset_symbol_new(Section, Name, Start, -1, CSYM_START | Attr, 0);
 }
 
-
 /*------------------------------------------------------------------------------------------------*/
 
 static void
@@ -773,17 +967,27 @@ load_processor_constants(void)
   /* The code section. */
 
   if ((sect = state.lset_root.sections[SECT_SPEC_CODE]) == NULL) {
-    sect = lset_section_new(&state.lset_root, "CODE", 1);
+    sect = lset_section_new(&state.lset_root, "CODE", 0);
   }
 
   if (sect != NULL) {
     addr = processor->prog_mem_size;
 
     if (addr > 0) {
-      add_constant(sect, "__CODE_START", 0, CSYM_ORG);
+      add_constant(sect, "__CODE_START",        0, CSYM_ORG);
       add_constant(sect, "__CODE_END",   addr - 1, CSYM_ORG);
     }
 
+    if ((pair = gp_processor_idlocs_exist(processor)) != NULL) {
+      lset_symbol_new(sect, "__IDLOCS_START", pair[0], -1, CSYM_START, 0);
+      lset_symbol_new(sect, "__IDLOCS_END",   pair[1], -1, CSYM_END,   0);
+    }
+
+    if ((pair = gp_processor_config_exist(processor)) != NULL) {
+      lset_symbol_new(sect, "__CONFIG_START", pair[0], -1, CSYM_START, 0);
+      lset_symbol_new(sect, "__CONFIG_END",   pair[1], -1, CSYM_END,   0);
+    }
+
     if ((processor->class->vector_table != NULL) || (processor->class->vector_number > 0)) {
       vec = processor->class->vector_table;
       num = processor->class->vector_number;
@@ -813,19 +1017,19 @@ load_processor_constants(void)
   /* The data section. */
 
   if ((sect = state.lset_root.sections[SECT_SPEC_DATA]) == NULL) {
-    sect = lset_section_new(&state.lset_root, "DATA", 1);
+    sect = lset_section_new(&state.lset_root, "DATA", 0);
   }
 
   if (sect != NULL) {
     if ((pair = gp_processor_common_ram_exist(processor)) != NULL) {
       if (lset_symbol_find_addr(sect, pair[0], pair[1], false) == NULL) {
-        lset_symbol_new(sect, "Common_RAM", pair[0], pair[1], CSYM_START | CSYM_END, 1);
+        lset_symbol_new(sect, "Common_RAM", pair[0], pair[1], CSYM_START | CSYM_END, 0);
       }
     }
 
     if ((pair = gp_processor_linear_ram_exist(processor)) != NULL) {
       if (lset_symbol_find_addr(sect, pair[0], pair[1], false) == NULL) {
-        lset_symbol_new(sect, "Linear_RAM", pair[0], pair[1], CSYM_START | CSYM_END, 1);
+        lset_symbol_new(sect, "Linear_RAM", pair[0], pair[1], CSYM_START | CSYM_END, 0);
       }
     }
   }
@@ -833,7 +1037,7 @@ load_processor_constants(void)
   /* The eedata section. */
 
   if ((sect = state.lset_root.sections[SECT_SPEC_EEDATA]) == NULL) {
-    sect = lset_section_new(&state.lset_root, "EEDATA", 1);
+    sect = lset_section_new(&state.lset_root, "EEDATA", 0);
   }
 
   if (sect != NULL) {
@@ -872,12 +1076,13 @@ dasm(MemBlock *memory)
   const lset_symbol_t *sym;
   char buffer[BUFSIZ];
 
-  if (state.show_names && ((state.class == PROC_CLASS_PIC12)  ||
-                           (state.class == PROC_CLASS_PIC12E) ||
-                           (state.class == PROC_CLASS_SX)     ||
-                           (state.class == PROC_CLASS_PIC14)  ||
-                           (state.class == PROC_CLASS_PIC14E) ||
-                           (state.class == PROC_CLASS_PIC16)  ||
+  if (state.show_names && ((state.class == PROC_CLASS_PIC12)   ||
+                           (state.class == PROC_CLASS_PIC12E)  ||
+                           (state.class == PROC_CLASS_SX)      ||
+                           (state.class == PROC_CLASS_PIC14)   ||
+                           (state.class == PROC_CLASS_PIC14E)  ||
+                           (state.class == PROC_CLASS_PIC14EX) ||
+                           (state.class == PROC_CLASS_PIC16)   ||
                            (state.class == PROC_CLASS_PIC16E))) {
     if (state.class == PROC_CLASS_PIC16E) {
       mark_false_addresses(memory);
@@ -895,8 +1100,9 @@ dasm(MemBlock *memory)
   write_header();
 
   if (state.show_names) {
-    printf("\n; The recognition of labels and registers is not always good, therefore\n"
-           "; be treated cautiously the results.\n");
+    ux_print(true, "\n"
+                   "; The recognition of labels and registers is not always good, therefore\n"
+                   "; be treated cautiously the results.");
     if (state.need_sfr_equ) {
       write_core_sfr_list();
     }
@@ -937,7 +1143,9 @@ dasm(MemBlock *memory)
             if (b_memory_get(m, i, &byte, NULL, NULL)) {
               if (last_loc != (i - insn_size)) {
                 if (state.show_names && (offset == 0)) {
-                  printf("\n;%s\n; IDLOCS area\n", border);
+                  ux_print(true, "\n"
+                                 ";%s\n"
+                                 "; IDLOCS area", border);
                 }
 
                 write_org(org, addr_digits, "idlocs", NULL, 0);
@@ -953,7 +1161,7 @@ dasm(MemBlock *memory)
               }
 
               byte_exclamation(buffer, sizeof(buffer), length, byte);
-              printf("%s\n", buffer);
+              ux_print(true, "%s", buffer);
             }
             else {
               last_loc = 0;
@@ -966,18 +1174,34 @@ dasm(MemBlock *memory)
           if (!state.show_config) {
             if (state.class->i_memory_get(m, i, &data, NULL, NULL)) {
               if (last_loc != (i - insn_size)) {
-                write_org(org, addr_digits, "idlocs", NULL, 0);
+                sym = lset_symbol_find_addr(state.lset_root.sections[SECT_SPEC_CODE], org, -1, true);
+
+                if ((sym != NULL) && (sym->attr & CSYM_ORG)) {
+                  write_org(org, addr_digits, "idlocs", sym->name, 0);
+                } else {
+                  write_org(org, addr_digits, "idlocs", NULL, 0);
+                }
               }
 
               last_loc = i;
 
               if (state.format) {
-                printf("%0*x:  %0*x  ", addr_digits, org, word_digits, (unsigned int)data);
+                length = snprintf(buffer, sizeof(buffer), "%0*x:  %0*x  ",
+                                  addr_digits, org, word_digits, (unsigned int)data);
               } else {
-                printf("        ");
+                length = snprintf(buffer, sizeof(buffer), "        ");
               }
 
-              printf("%-*s0x%0*x\n", TABULATOR_SIZE, "dw", word_digits, (unsigned int)data);
+              length += snprintf(buffer + length, sizeof(buffer) - length, "%-*s0x%0*x",
+                                 TABULATOR_SIZE, "dw", word_digits, (unsigned int)data);
+
+              if (state.processor->idlocs_mask != 0) {
+                unsigned int tmp = (~state.processor->idlocs_mask) & data;
+
+                gp_exclamation(buffer, sizeof(buffer), length, "; in fact: 0x%0*x", word_digits, tmp);
+              }
+
+              ux_print(true, "%s", buffer);
             }
             else {
               last_loc = 0;
@@ -994,7 +1218,9 @@ dasm(MemBlock *memory)
             if (b_memory_get(m, i, &byte, NULL, NULL)) {
               if (last_loc != (i - insn_size)) {
                 if (state.show_names && (offset == 0)) {
-                  printf("\n;%s\n; CONFIG Bits area\n", border);
+                  ux_print(true, "\n"
+                                 ";%s\n"
+                                 "; CONFIG Bits area", border);
                 }
 
                 write_org(org, addr_digits, "config", NULL, 0);
@@ -1003,12 +1229,12 @@ dasm(MemBlock *memory)
               last_loc = i;
 
               if (state.format) {
-                printf("%0*x:  %02x  ", addr_digits, org, (unsigned int)byte);
+                ux_print(false, "%0*x:  %02x  ", addr_digits, org, (unsigned int)byte);
               } else {
-                printf("        ");
+                ux_print(false, "        ");
               }
 
-              printf("%-*s0x%02x\n", TABULATOR_SIZE, "db", (unsigned int)byte);
+              ux_print(true, "%-*s0x%02x", TABULATOR_SIZE, "db", (unsigned int)byte);
             }
             else {
               last_loc = 0;
@@ -1022,7 +1248,9 @@ dasm(MemBlock *memory)
             if (state.class->i_memory_get(m, i, &data, NULL, NULL)) {
               if (last_loc != (i - insn_size)) {
                 if (state.show_names && (offset == 0)) {
-                  printf("\n;%s\n; CONFIG Bits area\n", border);
+                  ux_print(true, "\n"
+                                 ";%s\n"
+                                 "; CONFIG Bits area", border);
                 }
 
                 write_org(org, addr_digits, "config", NULL, 0);
@@ -1031,12 +1259,12 @@ dasm(MemBlock *memory)
               last_loc = i;
 
               if (state.format) {
-                printf("%0*x:  %0*x  ", addr_digits, org, word_digits, (unsigned int)data);
+                ux_print(false, "%0*x:  %0*x  ", addr_digits, org, word_digits, (unsigned int)data);
               } else {
-                printf("        ");
+                ux_print(false, "        ");
               }
 
-              printf("%-*s0x%0*x\n", TABULATOR_SIZE, "dw", word_digits, (unsigned int)data);
+              ux_print(true, "%-*s0x%0*x", TABULATOR_SIZE, "dw", word_digits, (unsigned int)data);
             }
             else {
               last_loc = 0;
@@ -1052,7 +1280,9 @@ dasm(MemBlock *memory)
             sym = lset_symbol_find_addr(state.lset_root.sections[SECT_SPEC_EEDATA], org, -1, true);
 
             if (state.show_names && (offset == 0)) {
-              printf("\n;%s\n; EEDATA area\n", border);
+              ux_print(true, "\n"
+                             ";%s\n"
+                             "; EEDATA area", border);
             }
 
             if ((sym != NULL) && (sym->attr & CSYM_ORG)) {
@@ -1079,12 +1309,12 @@ dasm(MemBlock *memory)
             gp_exclamation(buffer, sizeof(buffer), length, "; address: 0x%0*x", addr_digits, org);
 
             if (! prev_empty_line) {
-              printf("\n");
+              ux_print(true, "");
             }
 
-            printf("%s\n\n", buffer);
+            ux_print(true, "%s\n", buffer);
             prev_empty_line = true;
-	  }
+          }
 
           if (state.format) {
             length = snprintf(buffer, sizeof(buffer), "%0*x:  %02x    ",
@@ -1094,7 +1324,7 @@ dasm(MemBlock *memory)
           }
 
           byte_exclamation(buffer, sizeof(buffer), length, byte);
-          printf("%s\n", buffer);
+          ux_print(true, "%s", buffer);
           prev_empty_line = false;
         }
         else {
@@ -1110,7 +1340,9 @@ dasm(MemBlock *memory)
             sym = lset_symbol_find_addr(state.lset_root.sections[SECT_SPEC_CODE], org, -1, true);
 
             if (state.show_names && (org == 0)) {
-              printf("\n;%s\n; CODE area\n", border);
+              ux_print(true, "\n"
+                             ";%s\n"
+                             "; CODE area", border);
             }
 
             if ((sym != NULL) && (sym->attr & CSYM_ORG)) {
@@ -1128,12 +1360,12 @@ dasm(MemBlock *memory)
             gp_exclamation(buffer, sizeof(buffer), length, "; address: 0x%0*x", addr_digits, org);
 
             if (! prev_empty_line) {
-              printf("\n");
+              ux_print(true, "");
             }
 
-            printf("%s\n\n", buffer);
+            ux_print(true, "%s\n", buffer);
             prev_empty_line = true;
-	  }
+          }
 
           if (state.format) {
             length = snprintf(buffer, sizeof(buffer), "%0*x:  %0*x  ",
@@ -1146,22 +1378,22 @@ dasm(MemBlock *memory)
 
           if (type & W_CONST_DATA) {
             gp_disassemble_show_data(m, i, state.class, behavior, buffer, sizeof(buffer), length);
-            printf("%s\n", buffer);
+            ux_print(true, "%s", buffer);
             prev_empty_line = false;
             num_words = 1;
           }
           else {
             num_words = gp_disassemble(m, i, state.class, bsr_boundary, state.processor->prog_mem_size,
                                        behavior, buffer, sizeof(buffer), length);
-            printf("%s\n", buffer);
+            ux_print(true, "%s", buffer);
             prev_empty_line = false;
 
             if (num_words != 1) {
               /* Some 18xx instructions use two words. */
               if (state.format) {
                 state.class->i_memory_get(m, i + 2, &data, NULL, NULL);
-                printf("%0*x:  %0*x\n", addr_digits, gp_processor_byte_to_real(state.processor, i + 2),
-                       word_digits, (unsigned int)data);
+                ux_print(true, "%0*x:  %0*x", addr_digits, gp_processor_byte_to_real(state.processor, i + 2),
+                         word_digits, (unsigned int)data);
                 prev_empty_line = false;
               }
 
@@ -1208,10 +1440,14 @@ show_usage(void)
   printf("  -p PROC, --processor PROC      Select the processor.\n");
   printf("  -s, --short                    Print short format output. (Creates a compilable\n"
          "                                 source. See also the -k, -n and -o options.)\n");
+  printf("  -t, --use-tab                  Uses tabulator character in the written disassembled\n"
+         "                                 text.\n");
   printf("  -v, --version                  Print the gpdasm version information and exit.\n");
   printf("  -y, --extended                 Enable 18xx extended mode.\n");
   printf("      --strict                   Disassemble only opcodes generated by gpasm\n"
          "                                 in case of instructions with several opcodes.\n\n");
+  printf("      --strict-options           If this is set, then an option may not be parameter\n"
+         "                                 of an another option. For example: -p --dump\n");
   printf("For example:\n"
          "  gpdasm -nos -k program.ulist -p12f1822 program.hex > program.dis\n\n");
   printf("Report bugs to:\n");
@@ -1221,41 +1457,17 @@ show_usage(void)
 
 /*------------------------------------------------------------------------------------------------*/
 
-#define GET_OPTIONS "?chijk:lmnop:svy"
-
-  /* Used: himpsv */
-static struct option longopts[] =
-  {
-    { "mnemonics",   no_argument,       NULL, 'c' },
-    { "help",        no_argument,       NULL, 'h' },
-    { "hex-info",    no_argument,       NULL, 'i' },
-    { "mov-fsrn",    no_argument,       NULL, 'j' },
-    { "label-list",  required_argument, NULL, 'k' },
-    { "list-chips",  no_argument,       NULL, 'l' },
-    { "dump",        no_argument,       NULL, 'm' },
-    { "show-names",  no_argument,       NULL, 'n' },
-    { "show-config", no_argument,       NULL, 'o' },
-    { "processor",   required_argument, NULL, 'p' },
-    { "short",       no_argument,       NULL, 's' },
-    { "version",     no_argument,       NULL, 'v' },
-    { "extended",    no_argument,       NULL, 'y' },
-    { "strict",      no_argument,       NULL, 't' },
-    { NULL,          0,                 NULL,  0  }
-  };
-
-#define GETOPT_FUNC getopt_long(argc, argv, GET_OPTIONS, longopts, 0)
-
 int main(int argc, char *argv[])
 {
-  extern char *optarg;
-  extern int optind;
-
+  int option_index;
+  const char *command;
   int c;
+  gp_boolean strict_options = false;
   gp_boolean print_hex_info = false;
-  gp_boolean memory_dump = false;
-  gp_boolean strict = false;
-  gp_boolean usage = false;
-  const char *filename = NULL;
+  gp_boolean memory_dump    = false;
+  gp_boolean strict         = false;
+  gp_boolean usage          = false;
+  const char *filename        = NULL;
   const char *label_list_name = NULL;
   const int *pair;
 
@@ -1265,8 +1477,30 @@ int main(int argc, char *argv[])
   state.show_names  = false;
   state.show_fsrn   = false;
   state.show_config = false;
+  state.use_tab     = false;
+
+  /* Scan through the options for the --strict-options flag. */
+  while ((c = getopt_long(argc, argv, GET_OPTIONS, longopts, NULL)) != EOF) {
+    if (c == OPT_STRICT_OPTIONS) {
+      strict_options = true;
+      break;
+    }
+  }
+
+  /* Restores the getopt_long index. */
+  optind = 1;
+  while (true) {
+    /* This is necessary for the gp_exit_is_excluded_arg() function. */
+    option_index = -1;
+    command = argv[optind];
+    if ((c = getopt_long(argc, argv, GET_OPTIONS, longopts, &option_index)) == EOF) {
+      break;
+    }
+
+    if (strict_options) {
+      gp_exit_if_arg_an_option(longopts, ARRAY_SIZE(longopts), option_index, optarg, c, command);
+    }
 
-  while ((c = GETOPT_FUNC) != EOF) {
     switch (c) {
     case '?':
     case 'h':
@@ -1290,7 +1524,7 @@ int main(int argc, char *argv[])
       break;
 
     case 'l':
-      gp_dump_processor_list(true, NULL, NULL);
+      gp_dump_processor_list(true, PROC_CLASS_UNKNOWN, PROC_CLASS_UNKNOWN, PROC_CLASS_UNKNOWN);
       exit(0);
       break;
 
@@ -1314,8 +1548,8 @@ int main(int argc, char *argv[])
       state.format = 0;
       break;
 
-    case 'y':
-      gp_decode_extended = true;
+    case 't':
+      state.use_tab = true;
       break;
 
     case 'v':
@@ -1323,9 +1557,17 @@ int main(int argc, char *argv[])
       exit(0);
       break;
 
-    case 't':
+    case 'y':
+      gp_decode_extended = true;
+      break;
+
+    case OPT_STRICT:
       strict = true;
       break;
+
+    case OPT_STRICT_OPTIONS:
+      /* do nothing */
+      break;
     }
 
     if (usage) {
@@ -1369,8 +1611,7 @@ int main(int argc, char *argv[])
       printf("UNKNOWN\n");
     }
 
-    printf("number of bytes: %i\n", state.hex_info->size);
-    printf("\n");
+    printf("number of bytes: %i\n\n", state.hex_info->size);
   }
 
   if (label_list_name != NULL) {
@@ -1395,7 +1636,7 @@ int main(int argc, char *argv[])
                                           0, pair[1] - pair[0]);
       }
       else {
-        fprintf(stderr, "error: The processor does not have EEPROM.\n");
+        fprintf(stderr, "Error: The processor does not have EEPROM.\n");
         exit(1);
       }
     }
