@@ -26,7 +26,7 @@ Boston, MA 02111-1307, USA.  */
 
 struct gpvo_state state;
 
-void print_header(gp_object_type *object)
+static void print_header(const gp_object_type *object)
 {
 #define NELEM(x)  (sizeof(x) / sizeof(*(x)))
 
@@ -63,21 +63,27 @@ void print_header(gp_object_type *object)
   if (object->flags & F_RELFLG) {
     printf("  Relocation info has been stripped.\n");
   }
+
   if (object->flags & F_EXEC) {
     printf("  File is executable.\n");
   }
-  if (object->flags & F_LNNO) {
+
+  if (object->flags & F_LINENO) {
     printf("  Line numbers have been stripped.\n");
   }
+
   if (object->flags & F_ABSOLUTE) {
     printf("  The MPASM assembler object file is from absolute assembly code.\n");
   }
+
   if (object->flags & L_SYMS) {
     printf("  Local symbols have been stripped.\n");
   }
+
   if (object->flags & F_EXTENDED18) {
     printf("  The COFF file produced utilizing the Extended mode.\n");
   }
+
   if (object->flags & F_GENERIC) {
     printf("  Processor independent file for a core.\n");
   }
@@ -125,14 +131,15 @@ static const char *format_reloc_type(unsigned short type, char *buffer, size_t s
     "RELOCT_PAGESEL_MOVLP",
   };
 
-  if (type >= NELEM(type_str))
+  if (type >= NELEM(type_str)) {
     type = 0;
+  }
 
   snprintf(buffer, sizeof_buffer, "%#-4x %-20s", type, type_str[type]);
   return buffer;
 }
 
-void print_reloc_list(proc_class_t class, gp_reloc_type *relocation)
+static void print_reloc_list(proc_class_t class, const gp_reloc_type *relocation)
 {
   int word_addr = class->org_to_byte_shift;
   char buffer[32];
@@ -153,7 +160,7 @@ void print_reloc_list(proc_class_t class, gp_reloc_type *relocation)
   printf("\n");
 }
 
-void print_linenum_list(proc_class_t class, gp_linenum_type *linenumber)
+static void print_linenum_list(proc_class_t class, const gp_linenum_type *linenumber)
 {
   const char *filename;
 
@@ -178,7 +185,7 @@ void print_linenum_list(proc_class_t class, gp_linenum_type *linenumber)
   printf("\n");
 }
 
-void print_data(proc_class_t class, gp_section_type *section)
+static void print_data(proc_class_t class, const gp_section_type *section)
 {
   char buffer[BUFSIZ];
   int address;
@@ -190,34 +197,35 @@ void print_data(proc_class_t class, gp_section_type *section)
 
   printf("Data\n");
   while (1) {
-    if ((section->flags & STYP_TEXT) && class->find_insn != NULL) {
+    if ((section->flags & STYP_TEXT) && (class->find_insn != NULL)) {
       unsigned short memory;
 
-      if (!class->i_memory_get(section->data, address, &memory))
+      if (!class->i_memory_get(section->data, address, &memory, NULL, NULL)) {
         break;
-      num_words = gp_disassemble(section->data,
-                                 address,
-                                 class,
-                                 buffer,
-                                 sizeof(buffer));
+      }
+
+      num_words = gp_disassemble(section->data, address, class, 0x80, 0, GPDIS_SHOW_ALL_BRANCH,
+                                 buffer, sizeof(buffer), 0);
       printf("%06x:  %04x  %s\n", gp_processor_byte_to_org(class, address), memory, buffer);
+
       if (num_words != 1) {
-        class->i_memory_get(section->data, address + 2, &memory);
+        class->i_memory_get(section->data, address + 2, &memory, NULL, NULL);
         printf("%06x:  %04x\n", gp_processor_byte_to_org(class, address + 2), memory);
       }
+
       address += 2 * num_words;
     }
-    else if (section->flags & STYP_DATA_ROM || class == PROC_CLASS_EEPROM16) {
+    else if ((section->flags & STYP_DATA_ROM) || (class == PROC_CLASS_EEPROM16)) {
       unsigned short word;
 
-      if (class->i_memory_get(section->data, address, &word)) {
+      if (class->i_memory_get(section->data, address, &word, NULL, NULL)) {
         printf("%06x:  %04x\n", gp_processor_byte_to_org(class, address), word);
         address += 2;
       }
       else {
         unsigned char byte;
 
-        if (b_memory_get(section->data, address, &byte)) {
+        if (b_memory_get(section->data, address, &byte, NULL, NULL)) {
           printf("%06x:  %02x\n", gp_processor_byte_to_org(class, address), byte);
         }
         break;
@@ -227,8 +235,10 @@ void print_data(proc_class_t class, gp_section_type *section)
       /* STYP_DATA or STYP_ACTREC, or EEPROM8 */
       unsigned char byte;
 
-      if (!b_memory_get(section->data, address, &byte))
+      if (!b_memory_get(section->data, address, &byte, NULL, NULL)) {
         break;
+      }
+
       printf("%06x:  %02x\n", address, byte);
       ++address;
     }
@@ -236,14 +246,11 @@ void print_data(proc_class_t class, gp_section_type *section)
   printf("\n");
 }
 
-void print_sec_header(proc_class_t class, gp_section_type *section)
+static void print_sec_header(proc_class_t class, const gp_section_type *section)
 {
   int org_to_byte_shift;
 
-  if (section->flags & (STYP_TEXT|STYP_DATA_ROM))
-    org_to_byte_shift = class->org_to_byte_shift;
-  else
-    org_to_byte_shift = 0;
+  org_to_byte_shift = (section->flags & (STYP_TEXT | STYP_DATA_ROM)) ? class->org_to_byte_shift : 0;
 
   printf("Section Header\n");
   printf("Name                    %s\n",   section->name);
@@ -253,30 +260,39 @@ void print_sec_header(proc_class_t class, gp_section_type *section)
   printf("Number of Relocations   %i\n",   section->num_reloc);
   printf("Number of Line Numbers  %i\n",   section->num_lineno);
   printf("Flags                   %#lx\n", section->flags);
+
   if (section->flags & STYP_TEXT) {
     printf("  Executable code.\n");
   }
+
   if (section->flags & STYP_DATA) {
     printf("  Initialized data.\n");
   }
+
   if (section->flags & STYP_BSS) {
     printf("  Uninitialized data.\n");
   }
+
   if (section->flags & STYP_DATA_ROM) {
     printf("  Initialized data for ROM.\n");
   }
+
   if (section->flags & STYP_ABS) {
     printf("  Absolute.\n");
   }
+
   if (section->flags & STYP_SHARED) {
     printf("  Shared across banks.\n");
   }
+
   if (section->flags & STYP_OVERLAY) {
     printf("  Overlaid with other sections from different objects modules.\n");
   }
+
   if (section->flags & STYP_ACCESS) {
     printf("  Available using access bit.\n");
   }
+
   if (section->flags & STYP_ACTREC) {
     printf("  Contains the activation record for a function.\n");
   }
@@ -284,20 +300,22 @@ void print_sec_header(proc_class_t class, gp_section_type *section)
   printf("\n");
 }
 
-void print_sec_list(gp_object_type *object)
+static void print_sec_list(const gp_object_type *object)
 {
   gp_section_type *section = object->sections;
 
   while (section != NULL) {
     print_sec_header(object->class, section);
 
-    if (section->size && section->data_ptr) {
+    if ((section->size > 0) && (section->data_ptr > 0)) {
       print_data(object->class, section);
     }
-    if (section->num_reloc) {
+
+    if ((section->num_reloc > 0)) {
       print_reloc_list(object->class, section->relocations);
     }
-    if (section->num_lineno) {
+
+    if ((section->num_lineno > 0)) {
       print_linenum_list(object->class, section->line_numbers);
     }
 
@@ -305,85 +323,104 @@ void print_sec_list(gp_object_type *object)
   }
 }
 
-void coff_type(int type, char *buffer, size_t sizeof_buffer)
+static void coff_type(int type, char *buffer, size_t sizeof_buffer)
 {
   switch (type) {
   case T_NULL:
     /* null */
     snprintf(buffer, sizeof_buffer, "NULL");
     break;
+
   case T_VOID:
     /* void */
     snprintf(buffer, sizeof_buffer, "void");
     break;
+
   case T_CHAR:
     /* character */
     snprintf(buffer, sizeof_buffer, "char");
     break;
+
   case T_SHORT:
     /* short integer */
     snprintf(buffer, sizeof_buffer, "short");
     break;
+
   case T_INT:
     /* integer */
     snprintf(buffer, sizeof_buffer, "int");
     break;
+
   case T_LONG:
     /* long integer */
     snprintf(buffer, sizeof_buffer, "long int");
     break;
+
   case T_FLOAT:
     /* floating point */
     snprintf(buffer, sizeof_buffer, "float");
     break;
+
   case T_DOUBLE:
     /* double length floating point */
     snprintf(buffer, sizeof_buffer, "double");
     break;
+
   case T_STRUCT:
     /* structure */
     snprintf(buffer, sizeof_buffer, "struct");
     break;
+
   case T_UNION:
     /* union */
     snprintf(buffer, sizeof_buffer, "union");
     break;
+
   case T_ENUM:
     /* enumeration */
     snprintf(buffer, sizeof_buffer, "enum");
     break;
+
   case T_MOE:
     /* member of enumeration */
     snprintf(buffer, sizeof_buffer, "enum");
     break;
+
   case T_UCHAR:
     /* unsigned character */
     snprintf(buffer, sizeof_buffer, "unsigned char");
     break;
+
   case T_USHORT:
     /* unsigned short */
     snprintf(buffer, sizeof_buffer, "unsigned short");
     break;
+
   case T_UINT:
     /* unsigned integer */
     snprintf(buffer, sizeof_buffer, "unsigned int");
     break;
+
   case T_ULONG:
     /* unsigned long */
     snprintf(buffer, sizeof_buffer, "unsigned long");
     break;
+
   case T_LNGDBL:
     /* long double floating point */
     snprintf(buffer, sizeof_buffer, "long double");
     break;
+
   case T_SLONG:
     /* short long */
     snprintf(buffer, sizeof_buffer, "short long");
     break;
+
   case T_USLONG:
     /* unsigned short long */
     snprintf(buffer, sizeof_buffer, "unsigned short long");
     break;
+
   default:
     snprintf(buffer, sizeof_buffer, "unknown(%d)", type);
     break;
@@ -413,8 +450,11 @@ static const char *format_sym_type(int type, char *buffer, size_t sizeof_buffer)
     "T_SLONG",
     "T_USLONG"
   };
-  if (type < NELEM(type_str))
+
+  if (type < NELEM(type_str)) {
     return type_str[type];
+  }
+
   snprintf(buffer, sizeof_buffer, "%d", type);
   return buffer;
 }
@@ -429,8 +469,11 @@ static const char *format_sym_derived_type(int type, char *buffer, size_t sizeof
     "DT_ROMPTR",
     "DT_FARROMPTR",
   };
-  if (type < NELEM(type_str))
+
+  if (type < NELEM(type_str)) {
     return type_str[type];
+  }
+
   snprintf(buffer, sizeof_buffer, "%d", type);
   return buffer;
 }
@@ -438,37 +481,37 @@ static const char *format_sym_derived_type(int type, char *buffer, size_t sizeof
 static const char *format_sym_class(int cls, char *buffer, size_t sizeof_buffer)
 {
   switch(cls) {
-  case C_EFCN: return "C_EFCN";
-  case C_NULL: return "C_NULL";
-  case C_AUTO: return "C_AUTO";
-  case C_EXT: return "C_EXT";
-  case C_STAT: return "C_STAT";
-  case C_REG: return "C_REG";
-  case C_EXTDEF: return "C_EXTDEF";
-  case C_LABEL: return "C_LABEL";
-  case C_ULABEL: return "C_ULABEL";
-  case C_MOS: return "C_MOS";
-  case C_ARG: return "C_ARG";
-  case C_STRTAG: return "C_STRTAG";
-  case C_MOU: return "C_MOU";
-  case C_UNTAG: return "C_UNTAG";
-  case C_TPDEF: return "C_TPDEF";
+  case C_EFCN:    return "C_EFCN";
+  case C_NULL:    return "C_NULL";
+  case C_AUTO:    return "C_AUTO";
+  case C_EXT:     return "C_EXT";
+  case C_STAT:    return "C_STAT";
+  case C_REG:     return "C_REG";
+  case C_EXTDEF:  return "C_EXTDEF";
+  case C_LABEL:   return "C_LABEL";
+  case C_ULABEL:  return "C_ULABEL";
+  case C_MOS:     return "C_MOS";
+  case C_ARG:     return "C_ARG";
+  case C_STRTAG:  return "C_STRTAG";
+  case C_MOU:     return "C_MOU";
+  case C_UNTAG:   return "C_UNTAG";
+  case C_TPDEF:   return "C_TPDEF";
   case C_USTATIC: return "C_USTATIC";
-  case C_ENTAG: return "C_ENTAG";
-  case C_MOE: return "C_MOE";
+  case C_ENTAG:   return "C_ENTAG";
+  case C_MOE:     return "C_MOE";
   case C_REGPARM: return "C_REGPARM";
-  case C_FIELD: return "C_FIELD";
+  case C_FIELD:   return "C_FIELD";
   case C_AUTOARG: return "C_AUTOARG";
   case C_LASTENT: return "C_LASTENT";
-  case C_BLOCK: return "C_BLOCK";
-  case C_FCN: return "C_FCN";
-  case C_EOS: return "C_EOS";
-  case C_FILE: return "C_FILE";
-  case C_LINE: return "C_LINE";
-  case C_ALIAS: return "C_ALIAS";
-  case C_HIDDEN: return "C_HIDDEN";
-  case C_EOF: return "C_EOF";
-  case C_LIST: return "C_LIST";
+  case C_BLOCK:   return "C_BLOCK";
+  case C_FCN:     return "C_FCN";
+  case C_EOS:     return "C_EOS";
+  case C_FILE:    return "C_FILE";
+  case C_LINE:    return "C_LINE";
+  case C_ALIAS:   return "C_ALIAS";
+  case C_HIDDEN:  return "C_HIDDEN";
+  case C_EOF:     return "C_EOF";
+  case C_LIST:    return "C_LIST";
   case C_SECTION: return "C_SECTION";
   default:
     snprintf(buffer, sizeof_buffer, "%d", cls);
@@ -476,7 +519,7 @@ static const char *format_sym_class(int cls, char *buffer, size_t sizeof_buffer)
   }
 }
 
-void print_sym_table (gp_object_type *object)
+static void print_sym_table(const gp_object_type *object)
 {
   gp_symbol_type *symbol;
   gp_aux_type *aux;
@@ -498,8 +541,9 @@ void print_sym_table (gp_object_type *object)
     } else if (symbol->section_number == N_UNDEF) {
       section = "UNDEFINED";
     } else {
-      if (symbol->section)
+      if (symbol->section != NULL) {
         section = symbol->section->name;
+      }
       else {
         static char buf[64];
 
@@ -525,55 +569,55 @@ void print_sym_table (gp_object_type *object)
 
       switch (aux->type) {
       case AUX_DIRECT:
-        printf(AUX_INDENT "command = \"%c\"\n",
-               aux->_aux_symbol._aux_direct.command);
-        printf(AUX_INDENT "string = \"%s\"\n",
-               aux->_aux_symbol._aux_direct.string);
+        printf(AUX_INDENT "command = \"%c\"\n", aux->_aux_symbol._aux_direct.command);
+        printf(AUX_INDENT "string = \"%s\"\n", aux->_aux_symbol._aux_direct.string);
         break;
+
       case AUX_FILE:
         if (!state.suppress_names) {
-          printf(AUX_INDENT "file = %s\n",
-                 aux->_aux_symbol._aux_file.filename);
+          printf(AUX_INDENT "file = %s\n", aux->_aux_symbol._aux_file.filename);
         }
-        printf(AUX_INDENT "line included = %li\n",
-               aux->_aux_symbol._aux_file.line_number);
-        printf(AUX_INDENT "flags = %x\n",
-               aux->_aux_symbol._aux_file.flags);
+        printf(AUX_INDENT "line included = %li\n", aux->_aux_symbol._aux_file.line_number);
+        printf(AUX_INDENT "flags = %x\n", aux->_aux_symbol._aux_file.flags);
         break;
+
       case AUX_IDENT:
-        printf(AUX_INDENT "string = \"%s\"\n",
-               aux->_aux_symbol._aux_ident.string);
+        printf(AUX_INDENT "string = \"%s\"\n", aux->_aux_symbol._aux_ident.string);
         break;
+
       case AUX_SCN:
-        printf(AUX_INDENT "length = %li\n",
-               aux->_aux_symbol._aux_scn.length);
-        printf(AUX_INDENT "number of relocations = %i\n",
-               aux->_aux_symbol._aux_scn.nreloc);
-        printf(AUX_INDENT "number of line numbers = %i\n",
-               aux->_aux_symbol._aux_scn.nlineno);
+        printf(AUX_INDENT "length = %li\n", aux->_aux_symbol._aux_scn.length);
+        printf(AUX_INDENT "number of relocations = %i\n", aux->_aux_symbol._aux_scn.nreloc);
+        printf(AUX_INDENT "number of line numbers = %i\n", aux->_aux_symbol._aux_scn.nlineno);
         break;
+
       case AUX_FCN_CALLS: {
         struct gp_symbol_type *callee = aux->_aux_symbol._aux_fcn_calls.callee;
+
         printf(AUX_INDENT "callee = %s\n",
                callee != NULL ? callee->name : "higher order");
         printf(AUX_INDENT "is_interrupt = %lu\n",
                aux->_aux_symbol._aux_fcn_calls.is_interrupt);
         break;
       }
+
       default:
         printf("%ld  ", aux->type);
         for (i = 0; i < object->symbol_size; i++) {
           printf("%02x", aux->_aux_symbol.data[i] & 0xFF);
+
           if (i & 1) {
-            printf(" ");
+            putchar(' ');
           }
         }
         for (i = 0; i < object->symbol_size; i++) {
           int c = aux->_aux_symbol.data[i] & 0xFF;
-          putchar( (isprint(c)) ? c : '.');
+
+          putchar((isprint(c)) ? c : '.');
         }
         putchar('\n');
-      }
+      } /* switch (aux->type) { */
+
       aux = aux->next;
       ++idx;
     }
@@ -585,7 +629,7 @@ void print_sym_table (gp_object_type *object)
   printf("\n");
 }
 
-void export_sym_table(gp_object_type *object)
+static void export_sym_table(gp_object_type *object)
 {
   gp_symbol_type *symbol;
   char buffer[BUFSIZ];
@@ -593,22 +637,17 @@ void export_sym_table(gp_object_type *object)
   symbol = object->symbols;
 
   while (symbol != NULL) {
-
-    if ((state.export.enabled) &&
-        (symbol->class == C_EXT) &&
+    if ((state.export.enabled) && (symbol->class == C_EXT) &&
         (symbol->section_number > 0)) {
-
       coff_type(symbol->type, buffer, sizeof(buffer));
-      fprintf(state.export.f, "  extern %s ; %s\n",
-              symbol->name,
-              buffer);
+      fprintf(state.export.f, "  extern %s ; %s\n", symbol->name, buffer);
     }
 
     symbol = symbol->next;
   }
 }
 
-void print_binary(unsigned char *data, long int file_size)
+static void print_binary(const unsigned char *data, long file_size)
 {
 
   long int i, j;
@@ -623,6 +662,7 @@ void print_binary(unsigned char *data, long int file_size)
 
     for(j = 0; j < 16; j += 2) {
       memory = (data[i + j] << 8) | data[i+j+1];
+
       if ((i+j) >= file_size) {
         printf("     ");
       } else {
@@ -632,13 +672,13 @@ void print_binary(unsigned char *data, long int file_size)
 
     printf(" ");
 
-    for(j = 0; j < 16; j += 2) {
+    for (j = 0; j < 16; j += 2) {
       if ((i+j) < file_size) {
         c = data[i + j];
-        putchar( (isprint(c)) ? c : '.');
+        putchar((isprint(c)) ? c : '.');
 
         c = data[i + j + 1];
-        putchar( (isprint(c)) ? c : '.');
+        putchar((isprint(c)) ? c : '.');
       }
     }
 
@@ -647,51 +687,57 @@ void print_binary(unsigned char *data, long int file_size)
   printf("\n\n");
 }
 
-void show_usage(void)
+static void show_usage(void)
 {
   printf("Usage: gpvo [options] file\n");
   printf("Options: [defaults in brackets after descriptions]\n");
-  printf("  -b, --binary               Print binary data.\n");
-  printf("  -c, --mnemonics            Decode special mnemonics.\n");
-  printf("  -f, --file                 File header.\n");
-  printf("  -h, --help                 Show this usage message.\n");
-  printf("  -n, --no-names             Suppress filenames.\n");
-  printf("  -s, --section              Section data.\n");
-  printf("  -t  --symbol               Symbol table.\n");
-  printf("  -v, --version              Show version.\n");
-  printf("  -x FILE, --export FILE     Export symbols to include file.\n");
-  printf("  -y, --extended             Enable 18xx extended mode.\n");
-  printf("\n");
+  printf("  -b, --binary            Print binary data.\n");
+  printf("  -c, --mnemonics         Decode special mnemonics.\n");
+  printf("  -f, --file              File header.\n");
+  printf("  -h, --help              Show this usage message.\n");
+  printf("  -n, --no-names          Suppress filenames.\n");
+  printf("  -s, --section           Section data.\n");
+  printf("      --strict-options    If this is set, then an option may not be parameter\n"
+         "                          of an another option. For example: -x --symbol\n");
+  printf("  -t  --symbol            Symbol table.\n");
+  printf("  -v, --version           Show version.\n");
+  printf("  -x FILE, --export FILE  Export symbols to include file.\n");
+  printf("  -y, --extended          Enable 18xx extended mode.\n\n");
   printf("Report bugs to:\n");
   printf("%s\n", PACKAGE_BUGREPORT);
   exit(0);
 }
 
+enum {
+  OPT_STRICT_OPTIONS = 0x100
+};
+
 #define GET_OPTIONS "?bcfhnstvx:y"
 
 /* Used: himpsv */
 static struct option longopts[] =
 {
-  { "binary",    no_argument,       NULL, 'b' },
-  { "mnemonics", no_argument,       NULL, 'c' },
-  { "file",      no_argument,       NULL, 'f' },
-  { "help",      no_argument,       NULL, 'h' },
-  { "no-names",  no_argument,       NULL, 'n' },
-  { "section",   no_argument,       NULL, 's' },
-  { "symbol",    no_argument,       NULL, 't' },
-  { "version",   no_argument,       NULL, 'v' },
-  { "export",    required_argument, NULL, 'x' },
-  { "extended",  no_argument,       NULL, 'y' },
-  { NULL,        no_argument,       NULL, '\0'}
+  { "binary",         no_argument,       NULL, 'b' },
+  { "mnemonics",      no_argument,       NULL, 'c' },
+  { "file",           no_argument,       NULL, 'f' },
+  { "help",           no_argument,       NULL, 'h' },
+  { "no-names",       no_argument,       NULL, 'n' },
+  { "section",        no_argument,       NULL, 's' },
+  { "strict-options", no_argument,       NULL, OPT_STRICT_OPTIONS },
+  { "symbol",         no_argument,       NULL, 't' },
+  { "version",        no_argument,       NULL, 'v' },
+  { "export",         required_argument, NULL, 'x' },
+  { "extended",       no_argument,       NULL, 'y' },
+  { NULL,             no_argument,       NULL, '\0'}
 };
 
-#define GETOPT_FUNC getopt_long(argc, argv, GET_OPTIONS, longopts, 0)
-
 int main(int argc, char *argv[])
 {
-  extern int optind;
+  int option_index;
+  const char *command;
   int c;
-  int usage = 0;
+  gp_boolean strict_options = false;
+  gp_boolean usage          = false;
   char buffer[BUFSIZ];
 
   gp_init();
@@ -701,11 +747,32 @@ int main(int argc, char *argv[])
   state.suppress_names = false;
   state.export.enabled = false;
 
-  while ((c = GETOPT_FUNC) != EOF) {
+  /* Scan through the options for the --strict-options flag. */
+  while ((c = getopt_long(argc, argv, GET_OPTIONS, longopts, NULL)) != EOF) {
+    if (c == OPT_STRICT_OPTIONS) {
+      strict_options = true;
+      break;
+    }
+  }
+
+  /* Restores the getopt_long index. */
+  optind = 1;
+  while (true) {
+    /* This is necessary for the gp_exit_is_excluded_arg() function. */
+    option_index = -1;
+    command = argv[optind];
+    if ((c = getopt_long(argc, argv, GET_OPTIONS, longopts, &option_index)) == EOF) {
+      break;
+    }
+
+    if (strict_options) {
+      gp_exit_if_arg_an_option(longopts, ARRAY_SIZE(longopts), option_index, optarg, c, command);
+    }
+
     switch (c) {
     case '?':
     case 'h':
-      usage = 1;
+      usage = true;
       break;
 
     case 'b':
@@ -744,7 +811,12 @@ int main(int argc, char *argv[])
     case 'v':
       fprintf(stderr, "%s\n", GPVO_VERSION_STRING);
       exit(0);
+
+    case OPT_STRICT_OPTIONS:
+      /* do nothing */
+      break;
     }
+
     if (usage)
       break;
   }
@@ -752,7 +824,7 @@ int main(int argc, char *argv[])
   if ((optind + 1) == argc) {
     state.filename = argv[optind];
   } else {
-    usage = 1;
+    usage = true;
   }
 
   if (usage) {
@@ -764,8 +836,8 @@ int main(int argc, char *argv[])
     state.dump_flags = PRINT_HEADER | PRINT_SECTIONS | PRINT_SYMTBL;
   }
 
-  if (gp_identify_coff_file(state.filename) != object_file_v2 &&
-      gp_identify_coff_file(state.filename) != object_file) {
+  if ((gp_identify_coff_file(state.filename) != GP_COFF_OBJECT_V2) &&
+      (gp_identify_coff_file(state.filename) != GP_COFF_OBJECT)) {
     gp_error("\"%s\" is not a valid object file", state.filename);
     exit(1);
   }
@@ -775,6 +847,7 @@ int main(int argc, char *argv[])
 
   if (state.export.enabled) {
     state.export.f = fopen(state.export.filename, "w");
+
     if (state.export.f == NULL) {
       perror(state.export.filename);
       exit(1);
@@ -783,11 +856,8 @@ int main(int argc, char *argv[])
     gp_date_string(buffer, sizeof(buffer));
 
     fprintf(state.export.f, "; %s\n", state.export.filename);
-    fprintf(state.export.f, "; generated by %s on %s\n",
-            GPVO_VERSION_STRING,
-            buffer);
-    fprintf(state.export.f, "; from %s\n\n",
-            state.filename);
+    fprintf(state.export.f, "; generated by %s on %s\n", GPVO_VERSION_STRING, buffer);
+    fprintf(state.export.f, "; from %s\n\n", state.filename);
 
     export_sym_table(state.object);
 
