@@ -65,7 +65,7 @@ Boston, MA 02111-1307, USA.  */
 #endif
 
 #ifndef HAVE_STRNDUP
-/* no system strndup: use the one iplemented in libiberty */
+/* No system strndup: Use the one implemented in libiberty. */
 char *strndup (const char *s, size_t n);
 #endif
 
@@ -89,6 +89,32 @@ char *strndup (const char *s, size_t n);
   #define COPY_CHAR "/"
 #endif
 
-#define GPUTILS_COPYRIGHT_STRING "Copyright (c) 1998-2014 gputils project"
+#define GPUTILS_COPYRIGHT_STRING "Copyright (c) 1998-2015 gputils project"
 
+#if ((__GNUC__ * 100) + __GNUC_MINOR__) >= 402
+  #define GPUTILS_GCC_STR(s)            #s
+  #define GPUTILS_GCC_JOINSTR(x,y)      GPUTILS_GCC_STR(x ## y)
+  #define GPUTILS_GCC_DO_PRAGMA(x)      _Pragma(#x)
+  #define GPUTILS_GCC_PRAGMA(x)         GPUTILS_GCC_DO_PRAGMA(GCC diagnostic x)
+
+  #if ((__GNUC__ * 100) + __GNUC_MINOR__) < 406
+    /* needed for older GCCs that accept only top of the file diagnostic pragmas */
+    #define GPUTILS_GCC_DIAG_TOP(x)     GPUTILS_GCC_PRAGMA(ignored GPUTILS_GCC_JOINSTR(-W,x))
+    #define GPUTILS_GCC_DIAG_OFF(x)
+    #define GPUTILS_GCC_DIAG_ON(x)
+  #else
+    #define GPUTILS_GCC_DIAG_TOP(x)
+
+    #define GPUTILS_GCC_DIAG_OFF(x) \
+                GPUTILS_GCC_PRAGMA(push) \
+                GPUTILS_GCC_PRAGMA(ignored GPUTILS_GCC_JOINSTR(-W,x))
+
+    #define GPUTILS_GCC_DIAG_ON(x)      GPUTILS_GCC_PRAGMA(pop)
+  #endif
+#else
+  #define GPUTILS_GCC_DIAG_TOP(x)
+  #define GPUTILS_GCC_DIAG_OFF(x)
+  #define GPUTILS_GCC_DIAG_ON(x)
 #endif
+
+#endif /* __STDHDR_H__ */
