@@ -26,13 +26,14 @@ Boston, MA 02111-1307, USA.  */
    character is often used to fill the gap. */
 
 int
-gp_archive_count_members(gp_archive_type *archive)
+gp_archive_count_members(const gp_archive_type *archive)
 {
   int number = 0;
 
   /* If present, skip the symbol index. */
-  if (gp_archive_have_index(archive))
+  if (gp_archive_have_index(archive)) {
     archive = archive->next;
+  }
 
   while (archive != NULL) {
     number++;
@@ -42,26 +43,26 @@ gp_archive_count_members(gp_archive_type *archive)
   return number;
 }
 
-/* This function is unused */
+/* This function is unused. */
 
 char *
-gp_archive_member_name(gp_archive_type *archive)
+gp_archive_member_name(const gp_archive_type *archive)
 {
   char name[256];
   char *end;
 
   sscanf(archive->header.ar_name, "%255s/", name);
   end = strchr(&name[0], '/');
-  if (end != NULL)
+  if (end != NULL) {
     *end = '\0';
+  }
 
-  return strdup(name);
+  return GP_Strdup(name);
 }
 
 void
-gp_archive_list_members(gp_archive_type *archive)
+gp_archive_list_members(const gp_archive_type *archive)
 {
-  const char *format = "%-24s  %06i bytes  %s";
   char name[256];
   char *end;
   int date;
@@ -69,40 +70,43 @@ gp_archive_list_members(gp_archive_type *archive)
   int size;
 
   /* If present, skip the symbol index. */
-  if (gp_archive_have_index(archive))
+  if (gp_archive_have_index(archive)) {
     archive = archive->next;
+  }
 
   while (archive != NULL) {
     sscanf(archive->header.ar_name, "%255s/", name);
     sscanf(archive->header.ar_date, "%il", &date);
     sscanf(archive->header.ar_size, "%il", &size);
     end = strchr(&name[0], '/');
-    if (end != NULL)
+    if (end != NULL) {
       *end = '\0';
+    }
     time = date;
-    printf(format, name, size, ctime(&time));
+    printf("%-24s  %06i bytes  %s", name, size, ctime(&time));
     archive = archive->next;
   }
-
-  return;
 }
 
 gp_archive_type *
-gp_archive_find_member(gp_archive_type *archive, char *objectname)
+gp_archive_find_member(gp_archive_type *archive, const char *objectname)
 {
   char name[256];
   char *end;
   gp_archive_type *found = NULL;
 
   /* If present, skip the symbol index. */
-  if (gp_archive_have_index(archive))
+  if (gp_archive_have_index(archive)) {
     archive = archive->next;
+  }
 
   while (archive != NULL) {
     sscanf(archive->header.ar_name, "%255s/", name);
     end = strrchr(&name[0], '/');
-    if (end != NULL)
+    if (end != NULL) {
       *end = '\0';
+    }
+
     if (strcmp(objectname, name) == 0) {
       found = archive;
       break;
@@ -113,21 +117,20 @@ gp_archive_find_member(gp_archive_type *archive, char *objectname)
   return found;
 }
 
-int
+void
 gp_archive_free_member(gp_archive_type *archive)
 {
-
-  if (archive->data.file != NULL)
+  if (archive->data.file != NULL) {
     free(archive->data.file);
-  if (archive != NULL)
-    free(archive);
+  }
 
-  return 0;
+  if (archive != NULL) {
+    free(archive);
+  }
 }
 
 gp_archive_type *
-gp_archive_delete_member(gp_archive_type *archive,
-                         char *objectname)
+gp_archive_delete_member(gp_archive_type *archive, const char *objectname)
 {
   gp_archive_type *object = NULL;
   gp_archive_type *list = NULL;
@@ -156,9 +159,7 @@ gp_archive_delete_member(gp_archive_type *archive,
 }
 
 gp_archive_type *
-gp_archive_add_member(gp_archive_type *archive,
-                      char *filename,
-                      char *objectname)
+gp_archive_add_member(gp_archive_type *archive, const char *filename, const char *objectname)
 {
   gp_archive_type *oldmember = NULL;
   gp_archive_type *newmember = NULL;
@@ -171,7 +172,7 @@ gp_archive_add_member(gp_archive_type *archive,
 
   newobject = gp_read_file(filename);
 
-  newmember = (gp_archive_type *)malloc(sizeof(*newmember));
+  newmember = (gp_archive_type *)GP_Malloc(sizeof(*newmember));
   newmember->next = NULL;
 
   /* Point the archive member file to the object file.  The object is never
@@ -179,7 +180,7 @@ gp_archive_add_member(gp_archive_type *archive,
   newmember->data = *newobject;
 
   /* fill in the archive header */
-  memset(&newmember->header, 0x20, AR_HDR_SIZ); /* fill the header with space */
+  memset(&newmember->header, ' ', AR_HDR_SIZ); /* fill the header with space */
 
   timer = (int)time(NULL);
 
@@ -187,20 +188,11 @@ gp_archive_add_member(gp_archive_type *archive,
   snprintf(date, sizeof(date), "%il", timer);
   snprintf(size, sizeof(size), "%lil", newobject->size);
 
-  /* FIXME:  These functions overwrite the 0x20 that the header is filled
-     with. */
-  strncpy(newmember->header.ar_name,
-          &name[0],
-          sizeof(newmember->header.ar_name));
-  strncpy(newmember->header.ar_date,
-          &date[0],
-          sizeof(newmember->header.ar_date));
-  strncpy(newmember->header.ar_size,
-          &size[0],
-          sizeof(newmember->header.ar_size));
-  strncpy(newmember->header.ar_fmag,
-          ARMAG,
-          sizeof(newmember->header.ar_fmag));
+  /* FIXME: These functions overwrite the ' ' that the header is filled with. */
+  strncpy(newmember->header.ar_name, &name[0], sizeof(newmember->header.ar_name));
+  strncpy(newmember->header.ar_date, &date[0], sizeof(newmember->header.ar_date));
+  strncpy(newmember->header.ar_size, &size[0], sizeof(newmember->header.ar_size));
+  strncpy(newmember->header.ar_fmag, ARMAG,    sizeof(newmember->header.ar_fmag));
 
   oldmember = gp_archive_find_member(archive, objectname);
 
@@ -225,8 +217,7 @@ gp_archive_add_member(gp_archive_type *archive,
 }
 
 int
-gp_archive_extract_member(gp_archive_type *archive,
-                          char *objectname)
+gp_archive_extract_member(gp_archive_type *archive, const char *objectname)
 {
   gp_archive_type *object = NULL;
   char filename[256];
@@ -240,7 +231,7 @@ gp_archive_extract_member(gp_archive_type *archive,
      some libs generated with other tools.  It should not be necessary
      for libs generated by gplib. */
   snprintf(filename, sizeof(filename),
-           "%s%s", objectname, strrchr(filename, '.') == NULL ? ".o" : "");
+           "%s%s", objectname, (strrchr(filename, '.') == NULL) ? ".o" : "");
 
   output_file = fopen(filename, "wb");
   if (output_file == NULL) {
@@ -259,8 +250,7 @@ gp_archive_extract_member(gp_archive_type *archive,
 }
 
 int
-gp_archive_write(gp_archive_type *archive,
-                 char *archivename)
+gp_archive_write(gp_archive_type *archive, const char *archivename)
 {
   FILE *output_file;
   int size;
@@ -302,14 +292,12 @@ gp_archive_update_offsets(gp_archive_type *archive)
     offset += AR_HDR_SIZ + size;
     archive = archive->next;
   }
-
-  return;
 }
 
 /* read a coff archive and store it in memory */
 
 gp_archive_type *
-gp_archive_read(char *filename)
+gp_archive_read(const char *filename)
 {
   FILE *infile;
   gp_archive_type *archive = NULL;
@@ -320,38 +308,36 @@ gp_archive_read(char *filename)
   int object_size;
   char buffer[SARMAG + 1];
 
-  infile = fopen(filename,"rb");
+  infile = fopen(filename, "rb");
   if (infile == NULL) {
     perror(filename);
     exit(1);
   }
 
   /* read the magic number */
-  if (SARMAG != fread(&buffer[0], 1, SARMAG, infile) ||
-    0 != strncmp(buffer, ARMAG, SARMAG)) {
+  if ((fread(&buffer[0], 1, SARMAG, infile) != SARMAG) ||
+      (strncmp(buffer, ARMAG, SARMAG) != 0)) {
     fclose(infile);
     return NULL;
   }
 
   while (feof(infile) == 0) {
-    int n;
-
     /* allocate space for the next archive member */
-    new = (gp_archive_type *)malloc(sizeof(*new));
+    new = (gp_archive_type *)GP_Malloc(sizeof(gp_archive_type));
     new->next = NULL;
 
     /* read the archive header */
-    n = fread(&new->header, 1, AR_HDR_SIZ, infile);
-    if (AR_HDR_SIZ != n)
+    if (fread(&new->header, 1, AR_HDR_SIZ, infile) != AR_HDR_SIZ) {
       gp_error("bad archive \"%s\"", filename);
+    }
 
     /* read the object file or symbol index into memory */
     sscanf(new->header.ar_size, "%il", &object_size);
     new->data.size = object_size;
-    new->data.file = (unsigned char *)malloc(object_size);
-    n = fread(new->data.file, sizeof(char), object_size, infile);
-    if (n != object_size)
+    new->data.file = (unsigned char *)GP_Malloc(object_size);
+    if (fread(new->data.file, sizeof(char), object_size, infile) != object_size) {
       gp_error("bad archive \"%s\"", filename);
+    }
 
     /* insert the new member in the archive list */
     if (archive == NULL) {
@@ -367,8 +353,9 @@ gp_archive_read(char *filename)
        while eof test passes, but there are no other members.  Test for
        it. */
     fgetpos(infile, &position);
-    if (AR_HDR_SIZ != fread(&tmpheader, 1, AR_HDR_SIZ, infile))
+    if (fread(&tmpheader, 1, AR_HDR_SIZ, infile) != AR_HDR_SIZ) {
       break;
+    }
     fsetpos(infile, &position);
   }
 
@@ -381,14 +368,10 @@ gp_archive_read(char *filename)
 
 /* Determine if the archive has a symbol index */
 
-int
-gp_archive_have_index(gp_archive_type *archive)
+gp_boolean
+gp_archive_have_index(const gp_archive_type *archive)
 {
-
-  if ((archive != NULL) && (archive->header.ar_name[0] == '/'))
-    return 1;
-
-  return 0;
+  return (((archive != NULL) && (archive->header.ar_name[0] == '/')) ? true : false);
 }
 
 /* Remove the symbol index if one exists */
@@ -422,14 +405,16 @@ gp_archive_make_index(gp_archive_type *archive,
   char *end;
 
   /* If present, skip the symbol index. */
-  if (gp_archive_have_index(archive))
+  if (gp_archive_have_index(archive)) {
     archive = archive->next;
+  }
 
   while (archive != NULL) {
     sscanf(archive->header.ar_name, "%255s/", name);
     end = strchr(&name[0], '/');
-    if (end != NULL)
+    if (end != NULL) {
       *end = '\0';
+    }
 
     object = gp_convert_file(name, &archive->data);
     assert(object != NULL);
@@ -455,14 +440,17 @@ gp_archive_add_index(struct symbol_table *table,
   long int indexsize = 0;
   const char *symname;
 
-  if ((archive == NULL) || (table == NULL))
+  if ((archive == NULL) || (table == NULL)) {
     return NULL;
+  }
 
   /* sort the index */
-  ps = lst = malloc(table->count * sizeof(lst[0]));
-  for (i = 0; i < HASH_SIZE; i++)
-    for (s = table->hash_table[i]; s; s = s->next)
+  ps = lst = GP_Malloc(table->count * sizeof(lst[0]));
+  for (i = 0; i < HASH_SIZE; i++) {
+    for (s = table->hash_table[i]; s; s = s->next) {
       *ps++ = s;
+    }
+  }
   assert(ps == &lst[table->count]);
   qsort(lst, table->count, sizeof(lst[0]), symbol_compare);
 
@@ -474,31 +462,19 @@ gp_archive_add_index(struct symbol_table *table,
   }
 
   /* create a new member for the index and place it in the archive */
-  newmember = (gp_archive_type *)malloc(sizeof(*newmember));
-  if (!newmember) {
-    fprintf(stderr, " error allocating memory\n");
-    exit(1);
-  }
-  newmember->data.file = (unsigned char *)malloc(indexsize);
-  if(!newmember->data.file) {
-    fprintf(stderr, " error allocating memory\n");
-    exit(1);
-  }
+  newmember = (gp_archive_type *)GP_Malloc(sizeof(*newmember));
+  newmember->data.file = (unsigned char *)GP_Malloc(indexsize);
   newmember->data.size = indexsize;
   newmember->next = NULL;
 
   /* fill in the archive header */
-  memset(&newmember->header, 0x20, AR_HDR_SIZ); /* fill the header with space */
+  memset(&newmember->header, ' ', AR_HDR_SIZ); /* fill the header with space */
 
   newmember->header.ar_name[0] = '/';
   snprintf(size, sizeof(size), "%lil", indexsize);
 
-  strncpy(newmember->header.ar_size,
-          &size[0],
-          sizeof(newmember->header.ar_size));
-  strncpy(newmember->header.ar_fmag,
-          ARMAG,
-          sizeof(newmember->header.ar_fmag));
+  strncpy(newmember->header.ar_size, size,  sizeof(newmember->header.ar_size));
+  strncpy(newmember->header.ar_fmag, ARMAG, sizeof(newmember->header.ar_fmag));
 
   newmember->next = archive;
   archive = newmember;
@@ -521,8 +497,9 @@ gp_archive_add_index(struct symbol_table *table,
 
   /* write the symbol names to the member */
   for (i = 0; i < table->count; i++) {
-    const char*  symbol_name = get_symbol_name(lst[i]);
-    const size_t symbol_len  = strlen(symbol_name) + 1;
+    const char *symbol_name = get_symbol_name(lst[i]);
+    const size_t symbol_len = strlen(symbol_name) + 1;
+
     memcpy(ptr, symbol_name, symbol_len);
     ptr += symbol_len;
   }
@@ -532,23 +509,22 @@ gp_archive_add_index(struct symbol_table *table,
 
 /* place the symbol from the archive symbol index in the archive symbol table */
 
-int
-gp_archive_add_symbol(struct symbol_table *table,
-                      const char *name,
+gp_boolean
+gp_archive_add_symbol(struct symbol_table *table, const char *name,
                       gp_archive_type *member)
 {
   struct symbol *sym;
 
-  /* Search the for the symbol.  If not found, then add it to
-     the global symbol table.  */
+  /* Search the for the symbol. If not found, then add it to
+     the global symbol table. */
   sym = get_symbol(table, name);
   if (sym != NULL) {
-    return 1;
+    return true;
   }
   sym = add_symbol(table, name);
   annotate_symbol(sym, member);
 
-  return 0;
+  return false;
 }
 
 /* This function reads the symbol index in the archive.  The symbols are
@@ -556,8 +532,7 @@ placed in the archive symbol table.  This table stores the name of the
 symbol and the archive member that the symbol is defined in. */
 
 void
-gp_archive_read_index(struct symbol_table *table,
-                      gp_archive_type *archive)
+gp_archive_read_index(struct symbol_table *table, gp_archive_type *archive)
 {
   int number = 0;
   int i;
@@ -576,7 +551,7 @@ gp_archive_read_index(struct symbol_table *table,
 
   /* set the pointers to the offsets and symbol names */
   offset = &file[AR_INDEX_NUMBER_SIZ];
-  name = (const char*)offset + (AR_INDEX_OFFSET_SIZ * number);
+  name = (const char *)offset + (AR_INDEX_OFFSET_SIZ * number);
 
   for (i = 0; i < number; i++) {
     /* get the symbol offset from the symbol index */
@@ -586,8 +561,9 @@ gp_archive_read_index(struct symbol_table *table,
        have the same offset */
     list = archive;
     while (list != NULL) {
-      if (list->offset == offset_value)
+      if (list->offset == offset_value) {
         break;
+      }
       list = list->next;
     }
 
@@ -609,7 +585,6 @@ gp_archive_print_table(struct symbol_table *table)
 {
   const struct symbol **lst, **ps, *s;
   int i;
-  const char *format = "%-32s %s\n";
   const gp_archive_type *member;
   char name[256];
   char *end;
@@ -617,10 +592,13 @@ gp_archive_print_table(struct symbol_table *table)
   assert(table != NULL);
 
   /* sort the index */
-  ps = lst = malloc(table->count * sizeof(lst[0]));
-  for (i = 0; i < HASH_SIZE; i++)
-    for (s = table->hash_table[i]; s; s = s->next)
+  ps = lst = GP_Malloc(table->count * sizeof(lst[0]));
+  for (i = 0; i < HASH_SIZE; i++) {
+    for (s = table->hash_table[i]; s; s = s->next) {
       *ps++ = s;
+    }
+  }
+
   assert(ps == &lst[table->count]);
   qsort(lst, table->count, sizeof(lst[0]), symbol_compare);
 
@@ -631,9 +609,10 @@ gp_archive_print_table(struct symbol_table *table)
     /* determine the archive member name */
     sscanf(member->header.ar_name, "%255s/", name);
     end = strchr(&name[0], '/');
-    if (end != NULL)
+    if (end != NULL) {
       *end = '\0';
+    }
     /* print it */
-    printf(format, get_symbol_name(lst[i]), name);
+    printf("%-32s %s\n", get_symbol_name(lst[i]), name);
   }
 }
