@@ -53,7 +53,7 @@ struct filehdr
 /* file is executable - has no unresolved external symbols */
 #define F_EXEC        0x0002
 /* line numbers have been stripped */
-#define F_LNNO        0x0004
+#define F_LINENO      0x0004
 /* the MPASM assembler object file is from absolute (as opposed to relocatable) assembly code */
 #define F_ABSOLUTE    0x0010
 /* local symbols have been stripped */
@@ -100,118 +100,113 @@ struct scnhdr
 #define SEC_HDR_SIZ_v1 40
 #define SEC_HDR_SIZ_v2 40
 
-/* Section contains executable code */
-#define STYP_TEXT     0x0020
-/* Section contains initialized data */
-#define STYP_DATA     0x0040
-/* Section contains uninitialized data */
-#define STYP_BSS      0x0080
-/* Section contains initialized data for ROM */
-#define STYP_DATA_ROM 0x0100
-/* Section is absolute */
-#define STYP_ABS      0x1000
-/* Section is shared across banks */
-#define STYP_SHARED   0x2000
-/* Section is overlaid with other sections of the same name from different
-   objects modules */
-#define STYP_OVERLAY  0x4000
-/* Section is available using access bit */
-#define STYP_ACCESS   0x8000
-/* Section contains the activation record for a function */
-#define STYP_ACTREC   0x10000
-/* Section has been relocated.  This is a temporary flag used by the linker */
-#define STYP_RELOC    0x20000
-/* Section is byte packed on 16bit devices */
-#define STYP_BPACK    0x40000
+/* Section contains executable code. */
+#define STYP_TEXT     0x00000020
+/* Section contains initialized data. */
+#define STYP_DATA     0x00000040
+/* Section contains uninitialized data. */
+#define STYP_BSS      0x00000080
+/* Section contains initialized data for ROM. */
+#define STYP_DATA_ROM 0x00000100
+/* Section is absolute. */
+#define STYP_ABS      0x00001000
+/* Section is shared across banks. */
+#define STYP_SHARED   0x00002000
+/* Section is overlaid with other sections of the same name from different objects modules. */
+#define STYP_OVERLAY  0x00004000
+/* Section is available using access bit. */
+#define STYP_ACCESS   0x00008000
+/* Section contains the activation record for a function. */
+#define STYP_ACTREC   0x00010000
+/* Section has been relocated.  This is a temporary flag used by the linker. */
+#define STYP_RELOC    0x00020000
+/* Section is byte packed on 16bit devices. */
+#define STYP_BPACK    0x00040000
 
 /* relocation entry */
 struct reloc
 {
   unsigned long   r_vaddr;     /* entry relative virtual address */
   unsigned long   r_symndx;    /* index into symbol table */
-  short           r_offset;    /* offset to be added to address of symbol
-                                  'r_symndx' */
+  short           r_offset;    /* offset to be added to address of symbol 'r_symndx' */
   unsigned short  r_type;      /* relocation type */
 };
 
-#define RELOC_SIZ 12
+#define RELOC_SIZ     12
 
 /* relocation for the CALL instruction (first word only on 18cxx) */
-#define     RELOCT_CALL     1
+#define RELOCT_CALL             1
 /* relocation for the GOTO instruction (first word only on 18cxx) */
-#define     RELOCT_GOTO     2
+#define RELOCT_GOTO             2
 /* relocation for the second 8 bits of an address */
-#define     RELOCT_HIGH     3
+#define RELOCT_HIGH             3
 /* relocation for the low order 8 bits of an address */
-#define     RELOCT_LOW      4
+#define RELOCT_LOW              4
 /* relocation for the 5 bits of address for the P operand of a 17cxx MOVFP
    or MOVPF instruction */
-#define     RELOCT_P        5
-/* relocation to generate the appropriate instruction to bank switch for a
-   symbol */
-#define     RELOCT_BANKSEL  6
-/* relocation to generate the appropriate instruction to page switch for a
-   symbol */
-#define     RELOCT_PAGESEL  7
+#define RELOCT_P                5
+/* relocation to generate the appropriate instruction to bank switch for a symbol */
+#define RELOCT_BANKSEL          6
+/* relocation to generate the appropriate instruction to page switch for a symbol */
+#define RELOCT_PAGESEL          7
 /* FIXME */
-#define     RELOCT_ALL      8
+#define RELOCT_ALL              8
 /* FIXME */
-#define     RELOCT_IBANKSEL 9
+#define RELOCT_IBANKSEL         9
 /* relocation for the 8 bits of address for the F operand of a 17cxx MOVFP or
    MOVPF instruction */
-#define     RELOCT_F        10
+#define RELOCT_F                10
 /* FIXME */
-#define     RELOCT_TRIS     11
+#define RELOCT_TRIS             11
 /* relocation for the MOVLR bank 17cxx banking instruction */
-#define     RELOCT_MOVLR    12
+#define RELOCT_MOVLR            12
 /* relocation for the MOVLB 17cxx and 18cxx banking instruction */
-#define     RELOCT_MOVLB    13
+#define RELOCT_MOVLB            13
 /* relocation for the second word of an 18cxx goto instruction */
-#define     RELOCT_GOTO2    14
+#define RELOCT_GOTO2            14
 /* relocation for the second word of an 18cxx call instruction */
-#define     RELOCT_CALL2    RELOCT_GOTO2
+#define RELOCT_CALL2            RELOCT_GOTO2
 /* relocation for the source register of the 18cxx MOVFF instruction */
-#define     RELOCT_FF1      15
+#define RELOCT_FF1              15
 /* relocation for the destination register of the 18cxx MOVFF instruction */
-#define     RELOCT_FF2      16
+#define RELOCT_FF2              16
 /* relocation for the first word of the 18cxx LFSR instruction */
-#define     RELOCT_LFSR1    17
+#define RELOCT_LFSR1            17
 /* relocation for the second word of the 18cxx LFSR instruction */
-#define     RELOCT_LFSR2    18
+#define RELOCT_LFSR2            18
 /* relocation for the 18cxx BRA instruction */
-#define     RELOCT_BRA      19
+#define RELOCT_BRA              19
 /* relocation for the 18cxx RCALL instruction */
-#define     RELOCT_RCALL    RELOCT_BRA
+#define RELOCT_RCALL            RELOCT_BRA
 /* relocation for the 18cxx relative conditional branch instructions */
-#define     RELOCT_CONDBRA  20
+#define RELOCT_CONDBRA          20
 /* relocation for the highest order 8 bits of a 24-bit address */
-#define     RELOCT_UPPER    21
+#define RELOCT_UPPER            21
 /* relocation for the 18cxx access bit */
-#define     RELOCT_ACCESS   22
+#define RELOCT_ACCESS           22
 /* relocation for selecting the correct page using WREG as scratch */
-#define     RELOCT_PAGESEL_WREG    23
+#define RELOCT_PAGESEL_WREG     23
 /* relocation for selecting the correct page using bit set/clear instructions */
-#define     RELOCT_PAGESEL_BITS    24
+#define RELOCT_PAGESEL_BITS     24
 /* relocation for the size of a section */
-#define     RELOCT_SCNSZ_LOW   25
-#define     RELOCT_SCNSZ_HIGH  26
-#define     RELOCT_SCNSZ_UPPER 27
+#define RELOCT_SCNSZ_LOW        25
+#define RELOCT_SCNSZ_HIGH       26
+#define RELOCT_SCNSZ_UPPER      27
 /* relocation for the address of the end of a section */
-#define     RELOCT_SCNEND_LOW   28
-#define     RELOCT_SCNEND_HIGH  29
-#define     RELOCT_SCNEND_UPPER 30
+#define RELOCT_SCNEND_LOW       28
+#define RELOCT_SCNEND_HIGH      29
+#define RELOCT_SCNEND_UPPER     30
 /* relocation for the address of the end of a section on LFSR */
-#define     RELOCT_SCNEND_LFSR1 31
-#define     RELOCT_SCNEND_LFSR2 32
-#define     RELOCT_TRIS_3BIT    33
+#define RELOCT_SCNEND_LFSR1     31
+#define RELOCT_SCNEND_LFSR2     32
+#define RELOCT_TRIS_3BIT        33
 /* relocation for selecting the correct page using pic14 enhanced MOVLP instruction */
-#define     RELOCT_PAGESEL_MOVLP   34
+#define RELOCT_PAGESEL_MOVLP    34
 
 /* linenumber entry */
 struct lineno
 {
-  unsigned long  l_srcndx;     /* symbol table index of associated source
-                                  file */
+  unsigned long  l_srcndx;     /* symbol table index of associated source file */
   unsigned short l_lnno;       /* line number */
   unsigned long  l_paddr;      /* address of code for this lineno */
   unsigned short l_flags;      /* bit flags for the line number */
@@ -219,23 +214,23 @@ struct lineno
                                   there is one */
 };
 
-#define LINENO_SIZ 16
+#define LINENO_SIZ      16
 
 /* Set if l_fcnndx is valid */
-#define  LINENO_HASFCN  0x01
+#define LINENO_HASFCN   0x01
 
 /* symbol table entry */
 struct syment
 {
   union {
-    char            name[8];   /* symbol name if less then 8 characters */
+    char            name[8];   /* symbol name if less than 8 characters */
     struct {
       unsigned long s_zeros;   /* first four characters are 0 */
       unsigned long s_offset;  /* pointer to the string table */
     } ptr;
   } sym_name;
   unsigned long   value;       /* symbol value */
-  short  sec_num;              /* section number */
+  short           sec_num;     /* section number */
   unsigned long   type;        /* type */
   char            st_class;    /* storage class */
   unsigned char   num_auxsym;  /* number of auxiliary symbols */
@@ -244,11 +239,11 @@ struct syment
 #define SYMBOL_SIZE_v1 18
 #define SYMBOL_SIZE_v2 20
 
-/* Symbol section numbers */
-#define N_DEBUG -2
-#define N_ABS   -1
-#define N_UNDEF 0
-#define N_SCNUM 1
+/* Symbol section numbers. */
+#define N_DEBUG    -2
+#define N_ABS      -1
+#define N_UNDEF     0
+#define N_SCNUM     1
 
 /* Symbol types
 
@@ -425,19 +420,19 @@ struct aux_fcn_calls {
 
 
 /* Auxiliary entries */
-#define X_DIMNUM   4
-#define AUX_NONE   0
-#define AUX_FILE   1  /* detail information for a source file */
-#define AUX_SCN    2  /* detail information for a section */
-#define AUX_TAG    3  /* detail informationfor a struct/union/enum tag */
-#define AUX_EOS    4  /* end of struct/union/enum */
-#define AUX_FCN    5  /* detail information for a function */
-#define AUX_ARR    6  /* FIXME */
-#define AUX_EOBF   7  /* end of block or function */
-#define AUX_BOBF   8  /* beginning of block or function */
-#define AUX_VAR    9  /* variable */
-#define AUX_DIRECT 10 /* direct message */
-#define AUX_IDENT  11 /* ident */
+#define X_DIMNUM      4
+#define AUX_NONE      0
+#define AUX_FILE      1   /* detail information for a source file */
+#define AUX_SCN       2   /* detail information for a section */
+#define AUX_TAG       3   /* detail informationfor a struct/union/enum tag */
+#define AUX_EOS       4   /* end of struct/union/enum */
+#define AUX_FCN       5   /* detail information for a function */
+#define AUX_ARR       6   /* FIXME */
+#define AUX_EOBF      7   /* end of block or function */
+#define AUX_BOBF      8   /* beginning of block or function */
+#define AUX_VAR       9   /* variable */
+#define AUX_DIRECT    10  /* direct message */
+#define AUX_IDENT     11  /* ident */
 #define AUX_FCN_CALLS 12  /* function called by this function */
 
 /* These definitions are for the COFF as stored in memory. */
@@ -491,15 +486,15 @@ typedef struct gp_aux_type
   union {
     struct {
       unsigned char command;
-      char *string;
+      const char *string;
     } _aux_direct;
     struct {
-      char *filename;
+      const char *filename;
       unsigned long line_number;
       unsigned char flags;
     } _aux_file;
     struct {
-      char *string;
+      const char *string;
     } _aux_ident;
     struct {
       unsigned long  length;
@@ -508,7 +503,7 @@ typedef struct gp_aux_type
     } _aux_scn;
     struct {
       struct gp_symbol_type *callee; /* NULL for call through pointer */
-      unsigned long is_interrupt; /* 0 not, 1 low, 2 high */
+      unsigned long is_interrupt;    /* 0 not, 1 low, 2 high */
     } _aux_fcn_calls;
     char data[SYMBOL_SIZE_v2];
   } _aux_symbol;
@@ -524,7 +519,10 @@ typedef struct gp_symbol_type
   char *name;
 
   /* symbol value */
-  long value;
+  gp_symvalue_t value;
+
+  /* section name of symbol */
+  const char *section_name;
 
   /* section number, only for used for determining symbol type:
      N_DEBUG = -2, N_ABS = -1, N_UNDEF = 0, or N_SCNUM = 1 if defined */
@@ -554,6 +552,14 @@ typedef struct gp_symbol_type
   struct gp_symbol_type *next;
 } gp_symbol_type;
 
+/* hash table for find symbol name */
+
+typedef struct gp_hash
+{
+  unsigned int hash;
+  const gp_symbol_type *symbol;
+} gp_hash_type;
+
 /* section linked list */
 
 typedef struct gp_section_type
@@ -630,7 +636,7 @@ typedef struct gp_object_type
   size_t symbol_size;
 
   /* new style coff file? */
-  int isnew;
+  gp_boolean isnew;
 
   /* processor */
   pic_processor_t processor;
@@ -662,6 +668,10 @@ typedef struct gp_object_type
   /* tail of symbols */
   gp_symbol_type *symbols_tail;
 
+  /* hash table of symbols */
+  gp_hash_type *symbol_hashtable;
+  unsigned int symbol_hashtable_size;
+
   /* symbol table pointer, only valid when writing coff file */
   unsigned long symbol_ptr;
 
