@@ -29,7 +29,7 @@ gp_coffopt_remove_weak(gp_object_type *object)
 {
   gp_symbol_type *symbol;
 
-  gp_debug("removing weak symbols from %s", object->filename);
+  gp_debug("Removing weak symbols from \"%s\".", object->filename);
 
   /* Search the symbol table for extern symbols. */
   symbol = object->symbols;
@@ -43,8 +43,6 @@ gp_coffopt_remove_weak(gp_object_type *object)
 
     symbol = symbol->next;
   }
-
-  return;
 }
 
 /* Remove any relocatable section that doesn't have a symbol pointed to by
@@ -55,27 +53,33 @@ gp_coffopt_remove_dead_sections(gp_object_type *object, int pass)
 {
   gp_section_type *section;
   gp_reloc_type *relocation;
+  gp_section_type *rel_sect;
   gp_boolean section_removed = false;
 
-  gp_debug("removing dead sections pass %i", pass);
+  gp_debug("Removing dead sections pass %i.", pass);
 
   section = object->sections;
   while (section != NULL) {
-    /* mark all sections as unused */
+    /* Mark all sections as unused. */
     section->is_used = false;
     section = section->next;
   }
 
   section = object->sections;
   while (section != NULL) {
-    /* mark all sections that relocations point to as unused */
+    /* Mark all sections that relocations point to as used. */
+/*    gp_debug("  section_name: %s", section->name);*/
     relocation = section->relocations;
     while (relocation != NULL) {
-      if (relocation->symbol->section)
-        relocation->symbol->section->is_used = true;
-      else
-        gp_warning("relocation symbol %s has no section",
-            relocation->symbol->name);
+      if ((rel_sect = relocation->symbol->section) != NULL) {
+        if (rel_sect != section) {
+/*          gp_debug("    reloc_section_name: %s", rel_sect->name);*/
+          rel_sect->is_used = true;
+        }
+      }
+      else {
+        gp_warning("Relocation symbol %s has no section.", relocation->symbol->name);
+      }
       relocation = relocation->next;
     }
     section = section->next;
@@ -83,10 +87,9 @@ gp_coffopt_remove_dead_sections(gp_object_type *object, int pass)
 
   section = object->sections;
   while (section != NULL) {
-    /* FIXME: maybe don't remove if it is in protected memory */
-    if ((!section->is_used) &&
-        !(section->flags & STYP_ABS)) {
-      gp_debug("removing section %s", section->name);
+    /* FIXME: Maybe don't remove if it is in protected memory. */
+    if ((!section->is_used) && !(section->flags & STYP_ABS)) {
+      gp_debug("Removing section \"%s\".", section->name);
       gp_coffgen_delsectionsyms(object, section);
       gp_coffgen_delsection(object, section);
       section_removed = true;
@@ -99,6 +102,4 @@ gp_coffopt_remove_dead_sections(gp_object_type *object, int pass)
     /* take another pass */
     gp_coffopt_remove_dead_sections(object, pass++);
   }
-  
-  return;
 }
